/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.core;

import java.util.Hashtable;
import java.util.Iterator;

import alefpp.tools.Types;

/**
 * Class base of Alef++ Arrays
 * @author Adrabi Abderrahim
 *
 * @param <TYPE>
 * @version 1.8
 */

public class AArray<TYPE> implements Iterable{
	
	private Hashtable<String,TYPE> array	= null	;
	
	/**
	 * Create a new instance of class
	 *
	 */
	public AArray(){
		this.array = new Hashtable<String,TYPE>();
	}
	
	/**
	 * Add to last new value
	 * @param $1 (TYPE : Value)
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public TYPE push(TYPE $1){
		if ( this.array.put( "" + this.array.size() , ($1 instanceof AValue) ? $1 : (TYPE)new AValue(null, Types.getObjectType($1) , $1 )) != null ){
			return $1;
		}
		return null;
	}

	/**
	 * Remove last value and return value
	 * @return
	 */
	public TYPE pop(){
		return this.array.remove( this.array.size() - 1 );
	}
	
	/**
	 * Remove first value and return value
	 * @return
	 */
	public TYPE shift(){
		return this.array.remove( 0 );
	}
	
	/**
	 * Add to top of array a new value
	 * @param $1 (TYPE : Value)
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public TYPE unshift(TYPE $1){
		Hashtable<String,TYPE> tmp = new Hashtable<String,TYPE>();
		
		synchronized(tmp){
			tmp.put( "0" , ($1 instanceof AValue) ? $1 : (TYPE)new AValue(null, Types.getObjectType($1) , $1 ) );
			for( int $i = 0 ; $i < this.array.size() ; $i++ ){
				tmp.put( "" + ($i + 1) , this.array.get( $i ) );
			}
		}
		this.array = tmp;
		return $1;
	}
	
	/**
	 * Return an array size
	 * @return
	 */
	public int size(){
		return this.array.size();
	}

	/**
	 * Injecting a value in a specified index
	 * @param $1
	 * @param $2
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public Object inject(int $1, TYPE $2){
		this.array.put( "" + $1, ($2 instanceof AValue) ? $2 : (TYPE)new AValue(null, Types.getObjectType($1) , $1 )  );
		return null;
	}
	
	/**
	 * Get a value by index
	 * @param $1 (int : value index)
	 * @return
	 */
	public Object index(int $1){
		if( !this.array.containsKey( "" + $1 ) ){
			return new AValue(null, "walo" );
		}
		return this.array.get( "" + $1 );
	}
	
	/**
	 * Get a value by index and set type if not exists
	 * @param $1 (int : value index)
	 * @return
	 
	public Object _index(int $1, boolean $2){
		return this.array.get( "" + $1 );
	}
	*/
	
	/**
	 * Iterate an array values
	 */
	public Iterator iterator() {
		return this.array.values().iterator();
	}
	
	public String toString(){
		StringBuilder builder = new StringBuilder("[");
		AValue[] tmp = this.array.values().toArray(new AValue[]{});
		for(int index = tmp.length - 1 ; index >= 0  ; index -- ){
			builder.append( tmp[ index ] );
			if( index - 1 >= 0 ){
				builder.append(", " );
			}
		}
		builder.append("]");
		return builder.toString();
	}
}
