/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.core;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import alefpp.tools.Types;

/**
 * Class base of Alef++ Hashes
 * @author Adrabi Abderrahim
 *
 * @param <TYPE>
 * @version 1.2
 */
public class AHash<TYPE> implements Iterable{

	private Hashtable<String,TYPE> hash = null;
	
	public AHash(){
		this.hash = new Hashtable<String,TYPE>();
	}
	
	/**
	 * Add value with key in hash or replace
	 * @param $1 (String : Key)
	 * @param $2 (String : Value)
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public boolean val(String $1, TYPE $2){
		return this.hash.put($1 , ($2 instanceof AValue) ? $2 : (TYPE) new AValue(null, Types.getObjectType($1) , $1 )) != null;
	}
	
	/**
	 * Get value from key
	 * @param $1 (String : $1)
	 * @return
	 */
	public Object val(Object $1){
		return this.hash.get( $1 );
	}
	
	/**
	 * Remove Key/Value from hash
	 * @param $1 (String : Key)
	 * @return
	 */
	public Object remove(Object $1){
		return this.hash.remove( $1 );
	}
	
	/**
	 * Iterate hash values
	 */
	public Iterator iterator() {
		return this.hash.values().iterator();
	}

	/**
	 * Values iterator
	 * @return
	 */
	public List vals(){
		LinkedList<Object> vals = new LinkedList<Object>();
		Iterator e = this.hash.values().iterator();
		while( e.hasNext() ){
			vals.add( e.next() );
		}
		return vals;
	}
	
	/**
	 * Keys iterator
	 * @return
	 */
	public List keys(){
		LinkedList<Object> keys = new LinkedList<Object>();
		Enumeration<String> e = this.hash.keys();
		while( e.hasMoreElements() ){
			Object tmp = e.nextElement();
			keys.add( new AValue(null, Types.getObjectType( tmp ) , tmp ) );
		}
		return keys;
	}
	
	/**
	 * Return a hash size
	 * @return
	 */
	public int size(){
		return this.hash.size();
	}
	
	public String toString(){
		StringBuilder builder = new StringBuilder();
		builder.append("{");
		
		Enumeration<String> e = this.hash.keys();
		while( e.hasMoreElements() ){
			String key = e.nextElement();
			builder.append( key );
			builder.append(" => ");
			builder.append( this.hash.get( key ) );
			if( e.hasMoreElements() ){
				builder.append(", ");
			}
		}
		builder.append("}");
		return builder.toString();
	}
}
