/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.core;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;

/**
 * A Subroutine class
 * @author Adrabi Abderrahim
 * @version 1.3
 */

public class ASubroutine extends Scope implements Iterable{

	private String subroutineName					=	null;
	private boolean hasParameters					=	true;
	//private Modifier modifier						=	null;
	private AStatement parent						=	null;
	private Hashtable<String,AStatement> children	=	null;
	private Object block							=	null;
	private LinkedList<ASubroutineParameter> parameters=null;
	private String returnType						=	"java.lang.Object";
	
	public ASubroutine(String $1){
		this.children = new Hashtable<String,AStatement>();
		this.parameters = new LinkedList<ASubroutineParameter>();
		this.subroutineName = $1;
	}
	
	public ASubroutine(String $1, Object $2){
		this.children = new Hashtable<String,AStatement>();
		this.parameters = new LinkedList<ASubroutineParameter>();
		this.subroutineName = $1;
		this.block = $2;
	}
	
	@Override
	public boolean child(String $1, AStatement $2) {
		$2.parent(this);
		return this.children.put($1, $2) != null;
	}

	@Override
	public AStatement child(String $1) {
		if( this.children.containsKey($1) ){
			return this.children.get($1);
		}
		else if( this.parent instanceof Scope ){
			Scope scope = (Scope)this.parent;
			if( scope.existsChild($1) ){
				return scope.child($1);
			}
		}
		return null;
	}

	@Override
	public boolean existsChild(String $1) {
		if( this.children.containsKey($1) ){
			return true;
		}
		else if( this.parent instanceof Scope ){
			return ((Scope)this.parent).existsChild($1);
		}
		return false;
	}

	@Override
	public AStatement localChild(String $1) {
		return this.children.get( $1 );
	}

	@Override
	public boolean localExistsChild(String $1) {
		return this.children.containsKey( $1 ) ;
	}

	public AStatement parent() {
		return this.parent;
	}

	public boolean parent(AStatement $1) {
		return (this.parent = $1) != null ;
	}

	//public Modifier getModifier() {
	//	return modifier;
	//}

	//public void setModifier(Modifier modifier) {
	//	this.modifier = modifier;
	//}

	public String getReturnType() {
		return returnType;
	}

	public void setReturnType(String returnType) {
		this.returnType = returnType;
	}

	public String getSubroutineName() {
		return subroutineName;
	}

	public Object getBlock() {
		return block;
	}

	public void setBlock(Object block) {
		this.block = block;
	}

	public LinkedList<ASubroutineParameter> getParameters() {
		return parameters;
	}

	public void setParameters(LinkedList<ASubroutineParameter> parameters) {
		this.parameters = parameters;
	}

	public boolean pushParameter(ASubroutineParameter $1){
		return this.parameters.add( $1 );
	}
	
	public boolean shiftParameter(ASubroutineParameter $1){
		return this.parameters.removeLast() != null ;
	}
	
	public int paramertesCount() {
		return this.parameters.size();
	}

	/**
	 * Iterate subroutine parameters
	 */
	public Iterator iterator() {
		return this.parameters.iterator();
	}

	public boolean isHasParameters() {
		return hasParameters;
	}

	public void setHasParameters(boolean hasParameters) {
		this.hasParameters = hasParameters;
	}
	
	
}
