/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.core;

/**
 * Class base of Alef++ variables and variables metadata, in this class
 * the name, type and mofidiers can't be changed after initialisation.
 * @author Adrabi Abderrahim
 * @version 1.1
 */

public class AVariable extends AStatement{

	private String	name		=	null;
	private String	type		=	null;
	private Object	value		=	null;
	private AStatement parent	=	null;
	//Modifiers
	private boolean	isFinal		=	false;
	private boolean	isPrivate	=	false;
	private boolean	isPublic	=	false;
	private boolean	isProtected	=	false;
	private boolean	isStatic	=	false;
	
	/**
	 * Initial an Alef++ variable for interpreter
	 * @param $1 (String : Variable Name) 
	 * @param $2 (String : Variable Type)
	 * @param $3 (Object : Variable value)
	 * @param $4 (AStatement : Variable Parent)
	 * @param $5 (Boolean: Variable Modifier)
	 */
	public AVariable(String $1 , String $2 , Object $3 , AStatement $4 , boolean ... $5 ){
		this.name		= $1;
		this.type		= $2;
		this.value		= $3;
		this.parent		= $4;
		this.isFinal	= $5.length > 0 ? $5[0] : false;
		this.isPrivate	= $5.length > 1 ? $5[1] : false;
		this.isPublic	= $5.length > 2 ? $5[2] : false;
		this.isProtected= $5.length > 3 ? $5[3] : false;
		this.isStatic	= $5.length > 4 ? $5[4] : false;
	}

	public Object getValue() {
		return value;
	}

	public void setValue(Object value) {
		this.value = value;
	}

	public boolean isFinal() {
		return isFinal;
	}

	public boolean isPrivate() {
		return isPrivate;
	}

	public boolean isProtected() {
		return isProtected;
	}

	public boolean isPublic() {
		return isPublic;
	}

	public boolean isStatic() {
		return isStatic;
	}

	public String getName() {
		return name;
	}

	public String getType() {
		return type;
	}

	public AStatement parent() {
		return this.parent;
	}

	public boolean parent(AStatement $1) {
		return (this.parent = $1) != null;
	}
	
}
