/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.core;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.LinkedList;

/**
 * Collections of all packages loaded in Alef++
 * @author Adrabi Abderrahim
 * @version 1.3
 */

public class Pkgs {
	private static LinkedList<String> packages = new LinkedList<String>();
	
	/**
	 * Add package to Alef++ packages list
	 * @param $1 (String : Package Name)
	 * @return
	 */
	public static boolean pushPackage(String $1){
		return packages.add($1);
	}
	
	/**
	 * Add package to Alef++ packages list from path
	 * @param $1 (String : Package Name)
	 * @param $2 (String : Package Path)
	 * @return
	 */
	@SuppressWarnings({ "deprecation", "unchecked" })
	public static boolean pushPackage(String $1, String $2){
		File file = new File($2.substring(1, $2.length() - 1));
		if( !file.exists() ){
			file = new File( $2 );
		}
		
		if( file.exists() ){
			
			try {
				
				URL url = file.toURL();
				URLClassLoader sysloader = (URLClassLoader) ClassLoader.getSystemClassLoader();
		        Class sysclass = java.net.URLClassLoader.class;

		        Method method = sysclass.getDeclaredMethod( "addURL", new Class[] { URL.class } );
		        method.setAccessible( true );
		        method.invoke( sysloader, new Object[] { url } );
		        
		        return packages.add($1);
		        
		        
			} catch (MalformedURLException e) {
				
			} catch (SecurityException e) {
			
			} catch (NoSuchMethodException e) {
				
			} catch (IllegalArgumentException e) {
			
			} catch (IllegalAccessException e) {
				
			} catch (InvocationTargetException e) {
				
			}
			
		}
		return false;
	}
	
	/**
	 * Get and Remove first package from packages list
	 * @return
	 */
	public static String shiftPackage(){
		return packages.removeFirst();
	}
	
	/**
	 * Get all imported packages in a string array 
	 * @return
	 */
	public static String[] packages(){
		return (String[])packages.toArray(new String[]{});
	}
	
	/**
	 * Get class package
	 * @param $1
	 * @return
	 */
	public static String classPackage(String $1){
		String pkg = null;
		for( String tmp : packages() ){
			
			//test imported class like  "import java.util.LinkedList;"
			if( tmp.endsWith($1) ){
				pkg = tmp;
				break;
			}
			//test imported package like "import java.util.*;"
			else if( tmp.endsWith("*") ){
				try{
					tmp = tmp.replace("*", "");
					Class.forName(tmp + $1);
					pkg = tmp;
					break;
				}catch(ClassNotFoundException e){
					
				}
			}
		}
		return pkg;
	}
}
