/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.core.exceptions;

/**
 * Conversion Type Exception for conversion and typed assgin. 
 * @author Adrabi Abderrahim
 * @version 1.1
 */
public class ConversionTypeException extends Exception {

	private static final long serialVersionUID = 1L;
	private String message	=	null;
	
	/**
	 * 
	 * @param $1 (String : From Type)
	 * @param $2 (String : To Type)
	 * @param $3 (Long[] : Position)
	 */
	public ConversionTypeException(String $1, String $2, long[] $3){
		this.message = "Type mismatch: cannot convert from " + getNativeType($1) + " to " + getNativeType($2) + " @[" + $3[0] + "," + $3[1] + "]";
	}
	
	@Override
	public String getMessage() {
		return this.message;
	}

	@Override
	public String toString() {
		return this.getMessage();
	}

	/**
	 * convert Alef++ type to native type
	 * @param $1 (String : Alef++ Type)
	 * @return
	 */
	private String getNativeType(String $1){
		if( $1.equals("-Z") ){
			return "boolean";
		}
		else if( $1.equals("-I") ){
			return "int" ;
		}
		else if( $1.equals("-L") ){
			return "long" ;
		}
		else if( $1.equals("-D") ){
			return "double" ;
		}
		else{
			return $1;
		}
	}
}
