/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.core.exceptions;

import java.util.Iterator;
import java.util.LinkedList;

/**
 * Duplicate Subroutine Exception for duplicating name of subroutine and parameters type in module
 * @author Adrabi Abderrahim
 * @version 1.1
 */
public class DuplicateSubroutineException extends Exception {

	private static final long serialVersionUID = 1L;
	private String message = null ;
	
	/**
	 * 
	 * @param $1 (String : Subroutine Name)
	 * @param $2 (String : Return Type)
	 * @param $3 (LibkedList : Parameters Type)
	 * @param $4 (Long[] : Position)
	 */
	public DuplicateSubroutineException(String $1, String $2, LinkedList $3, long[] $4){
		StringBuilder builder = new StringBuilder( $1 );
		builder.append( " " );
		
		if( $2 != null ){
			builder.append( ":" );
			builder.append( $2 );
		}
		
		if( $3.size() > 0 ) {
			builder.append("(");
			Iterator i = $3.iterator();
			while( i.hasNext() ){
				builder.append( getNativeType( i.next().toString() ) );
				if( i.hasNext() ){
					builder.append( ", " );
				}
			}
			builder.append( ")" );
		}
		
		this.message = "Duplicate subroutine " + builder.toString() + " @[" + $4[0] + "," + $4[1] + "]";
	}

	/**
	 * convert Alef++ type to native type
	 * @param $1 (String : Alef++ Type)
	 * @return
	 */
	private String getNativeType(String $1){
		if( $1.equals("-Z") ){
			return "boolean";
		}
		else if( $1.equals("-I") ){
			return "int" ;
		}
		else if( $1.equals("-L") ){
			return "long" ;
		}
		else if( $1.equals("-D") ){
			return "double" ;
		}
		else{
			return $1;
		}
	}

	@Override
	public String getMessage() {
		return this.message;
	}
	
	
}
