/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.interpreter;

import java.util.Arrays;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;

import alefpp.core.*;
import alefpp.core.exceptions.ConversionTypeException;
import alefpp.core.exceptions.DuplicateLocalVariableException;
import alefpp.core.exceptions.DuplicateSubroutineException;
import alefpp.core.exceptions.FinalLocalVariableException;
import alefpp.core.exceptions.MissingLabelException;
import alefpp.core.exceptions.ModuleCastException;
import alefpp.core.exceptions.OutOfParametersException;
import alefpp.core.exceptions.PackageNotFoundException;
import alefpp.core.exceptions.UndefinedFieldException;
import alefpp.core.exceptions.UndefinedObjectException;
import alefpp.core.exceptions.UndefinedSubroutineException;
import alefpp.core.exceptions.UndefinedTypeException;
import alefpp.core.exceptions.UnsupportedFeatureException;
import alefpp.core.exceptions.VariableNotFoundException;
import alefpp.core.exceptions.VoidSubroutinesException;
import alefpp.reflect.Cast;
import alefpp.reflect.Reflection;
import alefpp.tools.Types;
import alefpp.parser.syntaxtree.AdditiveExpression;
import alefpp.parser.syntaxtree.Alefpp;
import alefpp.parser.syntaxtree.ArgumentList;
import alefpp.parser.syntaxtree.Arguments;
import alefpp.parser.syntaxtree.Array;
import alefpp.parser.syntaxtree.ArrayIndex;
import alefpp.parser.syntaxtree.BitwiseExpression;
import alefpp.parser.syntaxtree.Block;
import alefpp.parser.syntaxtree.BlockStatement;
import alefpp.parser.syntaxtree.CastStatement;
import alefpp.parser.syntaxtree.ConditionalAndExpression;
import alefpp.parser.syntaxtree.ConditionalExpression;
import alefpp.parser.syntaxtree.ConditionalOrExpression;
import alefpp.parser.syntaxtree.DieStatement;
import alefpp.parser.syntaxtree.DoWhileStatement;
import alefpp.parser.syntaxtree.EachInit;
import alefpp.parser.syntaxtree.EachStatement;
import alefpp.parser.syntaxtree.EqualityExpression;
import alefpp.parser.syntaxtree.EvalStatement;
import alefpp.parser.syntaxtree.ExclusiveOrExpression;
import alefpp.parser.syntaxtree.Expression;
import alefpp.parser.syntaxtree.FullIdentifier;
import alefpp.parser.syntaxtree.GoInit;
import alefpp.parser.syntaxtree.GoStatement;
import alefpp.parser.syntaxtree.GotoStatement;
import alefpp.parser.syntaxtree.Hash;
import alefpp.parser.syntaxtree.HashKey;
import alefpp.parser.syntaxtree.IfStatement;
import alefpp.parser.syntaxtree.InDeCrementExpression;
import alefpp.parser.syntaxtree.JumpStatement;
import alefpp.parser.syntaxtree.Label;
import alefpp.parser.syntaxtree.Last;
import alefpp.parser.syntaxtree.Main;
import alefpp.parser.syntaxtree.MemberAccess;
import alefpp.parser.syntaxtree.MissStatement;
import alefpp.parser.syntaxtree.Module;
import alefpp.parser.syntaxtree.ModuleSubroutineCalling;
import alefpp.parser.syntaxtree.MultiplicativeExpression;
import alefpp.parser.syntaxtree.Next;
import alefpp.parser.syntaxtree.Node;
import alefpp.parser.syntaxtree.NodeChoice;
import alefpp.parser.syntaxtree.NodeList;
import alefpp.parser.syntaxtree.NodeListOptional;
import alefpp.parser.syntaxtree.NodeOptional;
import alefpp.parser.syntaxtree.NodeSequence;
import alefpp.parser.syntaxtree.NodeToken;
import alefpp.parser.syntaxtree.ParameterType;
import alefpp.parser.syntaxtree.PreInDeCrementExpression;
import alefpp.parser.syntaxtree.ReferenceStatement;
import alefpp.parser.syntaxtree.RelationalExpression;
import alefpp.parser.syntaxtree.ReturnStatement;
import alefpp.parser.syntaxtree.Scalar;
import alefpp.parser.syntaxtree.ScalarAssign;
import alefpp.parser.syntaxtree.ScalarValue;
import alefpp.parser.syntaxtree.ShiftExpression;
import alefpp.parser.syntaxtree.Statement;
import alefpp.parser.syntaxtree.SubroutineDeclaration;
import alefpp.parser.syntaxtree.SubroutineType;
import alefpp.parser.syntaxtree.SynchonizedStatement;
import alefpp.parser.syntaxtree.UseStatement;
import alefpp.parser.syntaxtree.ValuesExpression;
import alefpp.parser.syntaxtree.VariableDeclaration;
import alefpp.parser.syntaxtree.WaloExpression;
import alefpp.parser.syntaxtree.WhileStatement;
import alefpp.parser.syntaxtree.XiSubroutine;
import alefpp.interpreter.utils.*;

/**
 * Alef++, Simple Interpreter for interpreting a Alef++ code source, by using a reflection
 * this interpreter can be hack any Java class for accessing in private/protected or defauts 
 * fields or subroutines.
 * 
 * by defaults Alef++ interpreter designed to hack Java with a *Very Easy* (stupid) souce code
 * for any one can implement her features.
 * @author Adrabi Abderrahim
 * @version 1.54
 */
public class Interpret implements Interpreter{

	private AModule module = null;
	
	public AModule getGlobalModule(){
		return this.module;
	}
	
	/**
	 * f0 -> ( "+" | "-" )
	 * f1 -> ( Expression() )+
	 * @throws Throwable 
	 */
	@SuppressWarnings("unchecked")
	public Object visit(AdditiveExpression $1, Object $2, Object ... $3) throws Throwable {
		// Get values
		LinkedList values = new LinkedList();
		Enumeration e = $1.f1.elements();
		while( e.hasMoreElements() ){
			values.add( this.visit( (Expression) e.nextElement() , $2) );
		}
		
		// Add operation
		if( $1.f0.choice.toString().equals("+") ){
			return Operations.addOperation(  values.toArray( new AValue[]{} ) );
		}
		// Minus operation 
		else if( $1.f0.choice.toString().equals("-") ) {
			return Operations.minusOperation(values.toArray( new AValue[]{} ));
		}
		
		return null;
	}

	/**
	 * f0 -> ( UseStatement() )*
	 * f1 -> ( EmptyStatement() | VariableDeclaration() ";" | SubroutineDeclaration() | Module() )*
	 * f2 -> ( Main() )?
	 * f3 -> <EOF>
	 * @throws Throwable 
	 */
	public Object visit(Alefpp $1 , String[] $2) throws Throwable {
		this.module = new AModule("AlefppMain");
		this.module.setLineNumber(1);
		
		//~
		Enumeration<Node> e = $1.f0.nodes.elements();
		while(e.hasMoreElements()){
			this.visit( (UseStatement)e.nextElement(), null );
		}
		
		
		e = $1.f1.elements();
		while( e.hasMoreElements() ){
			NodeChoice nc = (NodeChoice)e.nextElement() ;
			
			//~ Subroutines
			if( nc.choice instanceof SubroutineDeclaration ){
				this.visit( (SubroutineDeclaration) nc.choice, this.module );
			}
			//~ Global variables
			else if( nc.choice instanceof NodeSequence ){
				NodeSequence ns = (NodeSequence) nc.choice;
				if( ns.elementAt(0) instanceof  VariableDeclaration){
					this.visit( (VariableDeclaration) ns.elementAt(0), this.module );
				}
			}
			
		}
		
		//~
		if( $1.f2.node instanceof Main ){
			ASubroutine main = new ASubroutine("main");
			module.child("main", main);
			
			//add command line arguments
			main.child( "$0" , new AVariable( "$0", "Array" , 
					new AValue(null, "Array", alefpp.tools.Tools.to_a_a( $2 ) ), main , true ) );
			
			this.visit((Main)$1.f2.node, main);
		}
		
		return null;
	}

	/**
	 * f0 -> Expression()
	 * f1 -> ( "," Expression() )*
	 * @throws Throwable 
	 */
	@SuppressWarnings("unchecked")
	public Object visit(ArgumentList $1, Object $2, Object ... $3) throws Throwable {
		LinkedList params = new LinkedList();
		params.add( this.visit( $1.f0 , $2) );
		//~ ...
		Enumeration e = $1.f1.elements();
		while( e.hasMoreElements() ){
			NodeSequence ns = (NodeSequence)e.nextElement();
			params.add( this.visit( (Expression)ns.elementAt(1) , $2) );
		}
		//~ ...
		return params;
	}

	/**
	 * f0 -> "["
	 * f1 -> [ ArgumentList() ]
	 * f2 -> "]"
	 * @throws Throwable 
	 */
	@SuppressWarnings("unchecked")
	public Object visit(Arguments $1, Object $2, Object ... $3) throws Throwable {
		LinkedList<AValue> params = null;
		
		if( $1.f1.node instanceof ArgumentList ){
			params = (LinkedList) this.visit((ArgumentList)$1.f1.node , $2);
		}

		return params != null ? params.toArray(new AValue[]{}) : null;
	}

	/**
	 * f0 -> "["
	 * f1 -> ( Expression() ( "," Expression() )* )?
	 * f2 -> "]"
	 * @throws Throwable 
	 */
	public Object visit(Array $1, Object $2, Object ... $3) throws Throwable {
		String type  = null;
		AArray<Object> array = new AArray<Object>();

		if( $1.f1.node != null ){
			boolean firstElement = true;
			Enumeration e = null ;
			
			do{
				
				NodeSequence ns = firstElement ? (NodeSequence)$1.f1.node : (NodeSequence) e.nextElement();
				AValue value = (AValue) this.visit( (Expression)ns.elementAt( firstElement ? 0 : 1 ) , $2, $3);
				array.push(value);
				
				// get a value type
				if( type == null ){
					type = value.getType();
				}
				//test type
				else if ( !type.equals(value.getType()) ){
					type =  "java.lang.Object";
				}
				
				//~
				if ( firstElement ){
					NodeListOptional nlo = (NodeListOptional) ns.elementAt(1);
					e = nlo.elements();
					firstElement = false;
				}
				
			}while( !firstElement && e.hasMoreElements()  );
			
		}
		
		type = type != null && ( type.equals("java.lang.Object") || type.equals("Array") || type.equals("Hash") ) ? "Array" : "Array:" + type;
		return new AValue(null , type , array);
	}

	
	/**
	 * f0 -> "#"
	 * f1 -> ( ModuleSubroutineCalling() | <INTEGER_LITERAL> | Expression() )
	 * @throws Throwable 
	 */
	public Object visit(ArrayIndex $1, Object $2, Object ... $3) throws Throwable {
		if( $1.f1.choice instanceof NodeToken ){
			if( NativeTypes.isInt( $1.f1.choice.toString() ) ){
				return new AValue(null , "-I", $1.f1.choice.toString() );
			}
			else{
				return new AValue(null, "java.lang.Object");
			}
		}
		else if( $1.f1.choice instanceof Expression ){
			return this.visit( (Expression) $1.f1.choice , $2, $3);
		}
		return null;
	}

	/**
	 * f0 -> ( "&" | "|" )
	 * f1 -> ( Expression() )+
	 */
	@SuppressWarnings("unchecked")
	public Object visit(BitwiseExpression $1, Object $2, Object ... $3) throws Throwable {
		// Get values
		LinkedList values = new LinkedList();
		Enumeration e = $1.f1.elements();
		while( e.hasMoreElements() ){
			values.add( this.visit( (Expression) e.nextElement() , $2) );
		}
		
		// bitwise "AND" operation
		if( $1.f0.choice.toString().equals("&") ){
			return Operations.bitwiseAndOperation( values.toArray( new AValue[]{} ) );
		}
		// bitwise "OR" operation 
		else if( $1.f0.choice.toString().equals("|") ) {
			return Operations.bitwiseOrOperation( values.toArray( new AValue[]{} ));
		}
		return null;
	}

	/**
	 * f0 -> "{"
	 * f1 -> ( BlockStatement() )*
	 * f2 -> "}"
	 */
	public Object visit(Block $1, Object $2, Object ... $3) throws Throwable {
	
		Scope parent = (Scope) $2;
		
		Enumeration e = $1.f1.elements();
		while( e.hasMoreElements() ){
			try {
				this.visit((BlockStatement)e.nextElement(), $2);
			} 
			catch( ALast last ){
				//~ test labeled block like Java
				// label:
				//	{
				//		do something ...
				//		break/continue label;
				//	}
				if( last.hasLabel() ){
					if( parent.existsChild( last.getLabelName() ) ){
						if( !parent.localExistsChild( last.getLabelName() ) ){
							throw last;
						}
					}
					else{
						throw new MissingLabelException( last.getLabelName() , last.getPosition() );
					}
				}
				else{
					throw last;
				}
				
			}
			catch ( ANext the_next ) {
			  	throw the_next;
			}	
		}
		
		return null;
	}
	
	/**
	 * f0 -> EmptyStatement()
	 *       | Block()
	 *       | Statement()
	 *       | ReturnStatement()
	 */
	public Object visit(BlockStatement $1, Object $2, Object ... $3) throws Throwable {

		if( $1.f0.choice instanceof Block ){
			ABlock block = new ABlock();
			block.parent( (AStatement) $2);
			this.visit( (Block) $1.f0.choice , block );
		}
		else if( $1.f0.choice instanceof Statement ){
			this.visit( (Statement)$1.f0.choice, $2 );
		}
		else if( $1.f0.choice instanceof ReturnStatement ){
			this.visit( (ReturnStatement) $1.f0.choice , $2 , $3 ) ;
		}
		
		return null;
	}

	/**
	 * f0 -> "to"
	 * f1 -> Expression()
	 * f2 -> FullIdentifier()
	 * @throws Throwable 
	 */
	public Object visit(CastStatement $1, Object $2, Object ... $3) throws Throwable {
		long[] position = {$1.f0.beginLine , $1.f0.beginColumn};
		//Current value
		AValue value = (AValue) this.visit($1.f1, $2, $3);
		AFullIdentifier identifier = (AFullIdentifier) this.visit($1.f2, $2, $3);
		
		Object o = Cast.tryCast(identifier.getIdentifier() , value.getValue() );
		if( o != null ){
			return new AValue(null, Reflection.fullIdentifier( identifier.getIdentifier() ) , o ) ;
		}
		
		throw new ModuleCastException( value.getType() , identifier.getIdentifier() , position );
	}

	/**
	 * f0 -> EqualityExpression()
	 * f1 -> ( "&&" EqualityExpression() )*
	 */
	public Object visit(ConditionalAndExpression $1, Object $2, Object ... $3) throws Throwable {
		AValue tmp = (AValue) this.visit($1.f0 , $2);
		
		if( $1.f1.size() > 0 ){
			AValue tmp2 = null ;
			NodeListOptional nlo = $1.f1;
			Enumeration e = nlo.elements();
			while( e.hasMoreElements() ){
				NodeSequence ns = (NodeSequence)e.nextElement();
				if( ns.elementAt(1) instanceof EqualityExpression ){
					tmp2 = (AValue) this.visit((EqualityExpression)ns.elementAt(1), $2);
				} 
				tmp = Conditionals.conditionalAnd( tmp , tmp2 );
			}
		}
		return tmp;
	}

	/**
	 * f0 -> "="
	 * f1 -> Expression()
	 * f2 -> ","
	 * f3 -> ValuesExpression()
	 * f4 -> [ "," ValuesExpression() ]
	 */
	public Object visit(ConditionalExpression $1, Object $2, Object ... $3) throws Throwable {
		// test value
		AValue test 		= (AValue) this.visit($1.f1, $2, $3); 
		AValue trueValue	= (AValue) this.visit($1.f3, $2, $3); 
		AValue falseValue	= new AValue(null, "walo");
		if( $1.f4.node != null ){
			NodeSequence ns = (NodeSequence) $1.f4.node;
			falseValue		= (AValue) this.visit( (ValuesExpression)ns.elementAt(1)  , $2, $3);
		}
		return Normalize.normalizeBoolean( test.getValue() ) ? trueValue : falseValue;
	}

	/**
	 * f0 -> ConditionalAndExpression()
	 * f1 -> ( "||" ConditionalAndExpression() )*
	 */
	public Object visit(ConditionalOrExpression $1, Object $2, Object ... $3) throws Throwable {
		AValue tmp = (AValue) this.visit($1.f0 , $2);
		
		if( $1.f1.size() > 0 ){
			AValue tmp2 = null ;
			NodeListOptional nlo = $1.f1;
			Enumeration e = nlo.elements();
			//~ for test current expression
			//	if 'True' non needed to continue test because an 'Or' expression
			while( e.hasMoreElements() && !Normalize.normalizeBoolean( tmp.getValue() )){
				NodeSequence ns = (NodeSequence)e.nextElement();
				if( ns.elementAt(1) instanceof ConditionalAndExpression ){
					tmp2 = (AValue) this.visit((ConditionalAndExpression)ns.elementAt(1), $2);
				} 
				tmp = Conditionals.conditionalOr( tmp , tmp2 );
			}
		}
		return tmp;
	}

	/**
	 * f0 -> "die"
	 * f1 -> Expression()
	 */
	public Object visit(DieStatement $1, Object $2, Object ... $3) throws Throwable{
		AValue throwable = (AValue) this.visit($1.f1, $2, $3);
		if( throwable.getValue() instanceof Throwable ){
			throw (Throwable) throwable.getValue();
		}
		
		throw new Throwable( throwable.getValue().toString() );
	}

	/**
	 * f0 -> Expression()
	 * f1 -> "!loop"
	 * f2 -> BlockStatement()
	 */
	public Object visit(DoWhileStatement $1, Object $2, Object ... $3) throws Throwable {

		Scope parent = (Scope) $2;
		
		ABlock __dowhile__ = new ABlock();
		__dowhile__.parent( parent );
		
		AValue value = null;
		do{
			try {
				this.visit($1.f2, __dowhile__, $3);			
			} catch ( ALast last ) {
				
				if( last.hasLabel() ){
					if(  parent.existsChild( last.getLabelName() )  ){
						if( parent.localExistsChild( last.getLabelName() ) ){
							return null;
						}
						else{
							throw last;
						}
					}
					else{
						//~ throw exception label not founded
						throw new MissingLabelException( last.getLabelName() , last.getPosition() ) ;
					}
				}
				else{
					return null;
				}
			} catch ( ANext next ) {
			  	
				if( next.hasLabel() ){
					if(  parent.existsChild( next.getLabelName() )  ){
						if( parent.localExistsChild( next.getLabelName() ) ){
							continue;
						}
						else{
							throw next;
						}
					}
					else{
						//~ throw exception label not founded
						throw new MissingLabelException( next.getLabelName() , next.getPosition() ) ;
					}
				}
				else{
					continue;
				}
			}
			finally{
				//Update value
				value = (AValue) this.visit($1.f0, $2, $3);
			}
		}while( Normalize.normalizeBoolean( value.getValue() ) );
		return null;
	}

	/**
	 * f0 -> "my" <SCALAR> [ FullIdentifier() ]
	 *       | ScalarValue() [ MemberAccess() ]
	 *       | ModuleSubroutineCalling()
	 * @throws Throwable 
	 */
	@SuppressWarnings("unchecked")
	public Object visit(EachInit $1, Object $2, Object ... $3) throws Throwable {
		Scope block = (Scope) $2;
		//~ 
		if( $1.f0.choice instanceof NodeSequence ){
			NodeSequence ns = (NodeSequence) $1.f0.choice;
			//~ variable declaration 
			if( ns.elementAt(0) instanceof NodeToken ){
				String type = "java.lang.Object";
				//~ get type
				if( ns.elementAt(2) instanceof FullIdentifier ){
					AFullIdentifier identifier = (AFullIdentifier) this.visit( (FullIdentifier) ns.elementAt(2) , $2, $3);
					type = Types.getObjectType(identifier.getIdentifier() );
					if( type == null ){
						//~ type not founded
						throw new UndefinedTypeException(identifier.getIdentifier() , new long[]{ identifier.getLineNumber() , identifier.getColumnNumber() });
					}
				}
				
				//~ test variable existens
				NodeToken var = (NodeToken) ns.elementAt(1);
				if( !block.existsChild( var.tokenImage ) ){
					return new AVariable( var.tokenImage ,type , $3[0] , block , true);
				}
				else{
					//~
					throw new DuplicateLocalVariableException( var.tokenImage , new long[]{ var.beginLine , var.beginColumn } );
				}
				
			}
			
			//~ local variable
			else if( ns.elementAt(0) instanceof ScalarValue ){
				//Variable assignment
				NodeOptional nMember = (NodeOptional) ns.elementAt(1);
				if( nMember.node == null  ){
					return this.visit( (ScalarValue)ns.elementAt(0) , $2, $3[0] );
				}
				else{
					//~ Scalar AValue
					AValue value 	= (AValue) this.visit( (ScalarValue)ns.elementAt(0) , $2  ) ;
					Object o = value.getValue();
					
					//~ New Value for Scalar
					AValue tmp = (AValue) $3[0];
					
					//~
					LinkedList members = (LinkedList) this.visit( (MemberAccess)nMember.node , $2 , this.visit( (ScalarValue) ns.elementAt(0), $2) );
					Iterator<AMemberAccess> i = members.iterator();
					while(i.hasNext()){
						AMemberAccess member = i.next();
						
						if( member.getType() == AStatementType.Field ){
							if( i.hasNext() && Reflection.existsField(o, member.getName() ) ){
								o = Reflection.getFieldObject(o, member.getName() );
							}
							else if( !i.hasNext() && Reflection.existsField(o, member.getName() ) ){
								o = Reflection.setFieldObject(o, member.getName() , tmp );
							}
							else{
								throw new UndefinedFieldException(member.getName() , new long[]{member.getLineNumber(), member.getColumnNumber()} );
							}
						}
						else if ( member.getType() == AStatementType.Subroutine ){
							
							AValue[] args = (AValue[]) this.visit( (Arguments) member.getObject() , $2);
							String[] types = Types.getParametersType( args ) ;
							if( Reflection.existsSubroutine(o, member.getName(), args)){
								o = Reflection.invokeInstanceSubroutine( o , member.getName() , args );
							}
							else{
								throw new UndefinedSubroutineException(member.getName(), o , types , new long[]{member.getLineNumber(), member.getColumnNumber()});
							}
						}
					}
					
					// return value
					return new AValue( null, Types.getObjectType(o) , o );
				}
			}
		}
		//~ static calls
		else if( $1.f0.choice instanceof ModuleSubroutineCalling ){
			return this.visit( (ModuleSubroutineCalling )$1.f0.choice, $2 , $3) ;
		}
		return null;
	}

	/**
	 * f0 -> EachInit()
	 * f1 -> ","
	 * f2 -> Expression()
	 * f3 -> "each"
	 * f4 -> BlockStatement()
	 * @throws Throwable 
	 */
	public Object visit(EachStatement $1, Object $2, Object ... $3) throws Throwable {
		
		Scope parent = (Scope) $2;
		
		AValue elements = (AValue) this.visit($1.f2, $2, $3); 
		
		//~ for each or values
		for( Object value : new AForeach(elements.getValue()).itr() ){
			try{
				ABlock __each__ = new ABlock();
				__each__.parent( parent );
				Object tmp = this.visit($1.f0, __each__, value);
				
				//~ this *ONLY* in local variable
				// my $var , [0,1,2,3] each {
				// 	 java.lang.System.out.println[ $var ];
				// }
				if( tmp instanceof AVariable ){
					AVariable var = (AVariable) tmp;
					var.setValue( value );
					__each__.child( var.getName() , var);
				}
				//~ do
				this.visit($1.f4, __each__, $3);
			}
			catch ( ALast last ) {
				
				if( last.hasLabel() ){
					if(  parent.existsChild( last.getLabelName() )  ){
						if( parent.localExistsChild( last.getLabelName() ) ){
							return null;
						}
						else{
							throw last;
						}
					}
					else{
						//~ throw exception label not founded
						throw new MissingLabelException( last.getLabelName() , last.getPosition() ) ;
					}
				}
				else{
					return null;
				}
			} catch ( ANext next ) {

				if( next.hasLabel() ){
					if(  parent.existsChild( next.getLabelName() )  ){
						if( parent.localExistsChild( next.getLabelName() ) ){
							continue;
						}
						else{
							throw next;
						}
					}
					else{
						//~ throw exception label not founded
						throw new MissingLabelException( next.getLabelName() , next.getPosition() ) ;
					}
				}
				else{
					continue;
				}
			}	
		}
		return null;
	}

	/**
	 * f0 -> RelationalExpression()
	 * f1 -> ( ( "==" | "!=" ) RelationalExpression() )*
	 */
	public Object visit(EqualityExpression $1, Object $2, Object ... $3) throws Throwable {
		AValue tmp = (AValue) this.visit($1.f0 , $2);
		
		if( $1.f1.size() > 0 ){
			AValue test = null;
			
			AValue tmp2 = null ;
			NodeListOptional nlo = $1.f1;
			Enumeration e = nlo.elements();
			while( e.hasMoreElements() ){
				NodeSequence ns = (NodeSequence)e.nextElement();
				if( ns.elementAt(1) instanceof RelationalExpression ){
					tmp2 = (AValue) this.visit((RelationalExpression)ns.elementAt(1), $2);
				}
				
				// test "==" or "!="
				NodeChoice nc = (NodeChoice)ns.elementAt(0);
				if( test == null ){
					if( nc.choice.toString().equals("==") ){
						test = Conditionals.equality( tmp , tmp2 );
					}
					else if( nc.choice.toString().equals("!=") ){
						test = Conditionals.notEquality( tmp , tmp2 );
					}
				}
				else{
					if( nc.choice.toString().equals("==") ){
						test = Conditionals.conditionalAnd(test, Conditionals.equality( tmp , tmp2 ) );
					}
					else if( nc.choice.toString().equals("!=") ){
						test = Conditionals.conditionalAnd(test, Conditionals.notEquality( tmp , tmp2 ) );
					}
				}
				
				tmp = tmp2;
			}
			
			return test;
		}
		return tmp;
	}

	/**
	 * f0 -> "eval"
	 * f1 -> BlockStatement()
	 * f2 -> ( "error" "(" ( <SCALAR> | "my" <SCALAR> [ FullIdentifier() ] ) ")" BlockStatement() )+
	 * f3 -> [ "end" BlockStatement() ]
	 */
	public Object visit(EvalStatement $1, Object $2, Object ... $3) throws Throwable {
		
		Scope block = (Scope) $2;
		AEval eval = new AEval();
		
		//~ set parent
		eval.parent( block );
		
		//~ set position
		eval.setLineNumber( $1.f0.beginLine );
		eval.setColumnNumber( $1.f0.beginColumn );
		
		//~ eval block
		try{
			  this.visit($1.f1, eval, $3);
		}
		catch( ARet ret){
			throw ret;
		}
		catch ( ANext next ){
			throw next;
		}
		catch ( ALast last ){
			throw last;
		}
		catch(Throwable thr ){
			eval.setException( thr );
		}
		
		//~ catch blocks
		if ( eval.getException() != null ){ //~~
			
			ACatch catch_ = null;
			AVariable varException = null;
			
			Enumeration e = $1.f2.elements();
			while( e.hasMoreElements() ){
				NodeSequence ns = (NodeSequence) e.nextElement();
				NodeChoice nc = (NodeChoice) ns.elementAt(2);
				
				// variable declaration like catch ( my $e ) or catch ( my $e Exception )
				if( nc.choice instanceof NodeSequence ){
					NodeSequence varDeclaration = (NodeSequence) nc.choice; 
					NodeOptional no = (NodeOptional) varDeclaration.elementAt(2);
					NodeToken nt = (NodeToken) varDeclaration.elementAt(1);
					
					if( block.existsChild( nt.tokenImage ) ){
						throw new DuplicateLocalVariableException( nt.tokenImage , new long[]{ nt.beginLine , nt.beginColumn } );
					}
					
					//~ test variable declaration with type like catch( my $e Exception )
					if( no.node != null ){
						AFullIdentifier identifier = (AFullIdentifier) this.visit( (FullIdentifier) no.node, $2);
						if( Reflection.instanceOf( Reflection.fullIdentifier(identifier.getIdentifier()) ,  eval.getException() ) ){
							catch_ = new ACatch( ns.elementAt(4) );
							varException = new AVariable( varDeclaration.elementAt(1).toString(), Reflection.fullIdentifier(identifier.getIdentifier()) , new AValue( null, Reflection.fullIdentifier(identifier.getIdentifier()) , eval.getException() ) ,catch_ , false );
							catch_.parent(block);
							catch_.child(varDeclaration.elementAt(1).toString(), varException);
							varException.parent(catch_);
							break;
						}
						else{
							throw new UndefinedTypeException(identifier.getIdentifier() , new long[]{ identifier.getLineNumber() , identifier.getColumnNumber() });
						}
					}
					
					//~ test variable declaration with default type like catch ( my $e )
					else if( no.node == null ){
						catch_ = new ACatch( ns.elementAt(4) );
						varException = new AVariable( varDeclaration.elementAt(1).toString(), "java.lang.Throwable"  , new AValue( null, "java.lang.Throwable" , eval.getException() ) , catch_, false );
						
						catch_.parent(block);
						catch_.child(varDeclaration.elementAt(1).toString(), varException);
						varException.parent(catch_);
					}
				}
				
				//~ exists variables
				if( nc.choice instanceof NodeToken ){
					NodeToken nt = (NodeToken) nc.choice;
					if( block.existsChild( nt.tokenImage ) ){
						AVariable	tmp = (AVariable) block.child( nt.tokenImage );
						if( tmp.getType().equals("java.lang.Object") || Reflection.instanceOf( tmp.getType() , eval.getException() ) ){
							AValue val = (AValue) tmp.getValue();
							val.setValue( eval.getException() );
							
							catch_ = new ACatch( ns.elementAt(4) );
							catch_.parent(block);
						}
					}
					else{
						throw new VariableNotFoundException( nt.tokenImage , new long[]{ nt.beginLine , nt.beginColumn } );
					}
				}
			}
			
			if( catch_ != null ){
				this.visit( (BlockStatement)catch_.getBlock() , catch_, $3);
			}
			else{
				throw eval.getException();
			}
			
		} //~~
		
		
		//~ finally block
		if( $1.f3.node != null ){
			NodeSequence ns = (NodeSequence) $1.f3.node;
			NodeToken nt = (NodeToken) ns.elementAt(0);
			
			ABlock end = new ABlock();
			end.setLineNumber( nt.beginLine );
			end.setColumnNumber( nt.beginColumn );
			end.parent( block );
			
			this.visit( (BlockStatement) ns.elementAt(1), end, $3);
		}
		
		return null ;
	}

	/**
	 * f0 -> ValuesExpression()
	 * f1 -> ( "^" ValuesExpression() )*
	 */
	public Object visit(ExclusiveOrExpression $1, Object $2, Object ... $3) throws Throwable {
		AValue tmp = (AValue) this.visit($1.f0 , $2);

		if( $1.f1.size() > 0 ){
			AValue tmp2 = null ;
			NodeListOptional nlo = $1.f1;
			Enumeration e = nlo.elements();
			while( e.hasMoreElements() ){
				NodeSequence ns = (NodeSequence)e.nextElement();
				if( ns.elementAt(1) instanceof ValuesExpression ){
					tmp2 = (AValue) this.visit((ValuesExpression)ns.elementAt(1), $2);
				}
				
				// test "^"
				tmp = Conditionals.conditionalXOr( tmp , tmp2 );
				
			}
		}
		return tmp;
	}

	/**
	 * f0 -> CastStatement()
	 *       | ReferenceStatement()
	 *       | XiSubroutine()
	 *       | ConditionalExpression()
	 *       | ConditionalOrExpression()
	 *       | AdditiveExpression()
	 *       | MultiplicativeExpression()
	 *       | BitwiseExpression()
	 *       | ShiftExpression()
	 *       | Array()
	 *       | Hash()
	 * @throws Throwable 
	 */
	public Object visit(Expression $1, Object $2, Object ... $3) throws Throwable {
		if ( $1.f0.choice instanceof ConditionalOrExpression ){
			return this.visit( (ConditionalOrExpression) $1.f0.choice , $2);
		}
		else if ( $1.f0.choice instanceof AdditiveExpression ){
			return this.visit( (AdditiveExpression) $1.f0.choice , $2);
		}
		else if ( $1.f0.choice instanceof MultiplicativeExpression ){
			return this.visit( (MultiplicativeExpression) $1.f0.choice , $2);
		}
		else if ( $1.f0.choice instanceof BitwiseExpression ){
			return this.visit( (BitwiseExpression) $1.f0.choice , $2);
		}
		else if ( $1.f0.choice instanceof ShiftExpression ){
			return this.visit( (ShiftExpression) $1.f0.choice , $2);
		}
		else if ( $1.f0.choice instanceof CastStatement ){
			return this.visit( (CastStatement) $1.f0.choice , $2);
		}
		else if ( $1.f0.choice instanceof Array ){
			return this.visit( (Array) $1.f0.choice , $2);
		}
		else if ( $1.f0.choice instanceof Hash ){
			return this.visit( (Hash) $1.f0.choice , $2);
		}
		else if ( $1.f0.choice instanceof XiSubroutine ){
			return this.visit( (XiSubroutine) $1.f0.choice , $2 , $3);
		}
		else if ( $1.f0.choice instanceof ConditionalExpression ){
			return this.visit( (ConditionalExpression) $1.f0.choice , $2 , $3);
		}
		else if ( $1.f0.choice instanceof ReferenceStatement ){
			return this.visit( (ReferenceStatement) $1.f0.choice , $2 , $3);
		}
		return null;
	}

	/**
	 * f0 -> <IDENTIFIER>
	 * f1 -> ( "." <IDENTIFIER> )*
	 */
	public Object visit(FullIdentifier $1, Object $2, Object ... $3) {
		StringBuilder builder = new StringBuilder($1.f0.tokenImage);
		Enumeration e = $1.f1.elements();
		while( e.hasMoreElements() ){
			NodeSequence ns = (NodeSequence)e.nextElement();
			builder.append( ns.elementAt(0) );
			builder.append( ns.elementAt(1) );
		}
		
		AFullIdentifier identifier = new AFullIdentifier(builder.toString());
		identifier.setColumnNumber($1.f0.beginColumn);
		identifier.setLineNumber($1.f0.beginLine);
		return identifier;
	}

	/**
	 * f0 -> GoInit()
	 * f1 -> ","
	 * f2 -> Expression()
	 * f3 -> "go"
	 * f4 -> [ "(" Expression() ")" ]
	 * f5 -> BlockStatement()
	 */
	public Object visit(GoStatement $1, Object $2, Object ... $3) throws Throwable {
		
		Scope parent = (Scope) $2;
		ABlock __for__ = new ABlock();
		__for__.parent( parent );
		
		//~ for init
		AVariable init = null;
		{
			Object tmp = this.visit($1.f0 , parent , $3);
			if( tmp instanceof AVariable ){
				init = (AVariable) tmp;
				if( !__for__.existsChild( init.getName() ) ){
					init.parent( __for__ );
					__for__.child( init.getName() , init );
				}
			}
			
		}
		
		AValue expression = (AValue) this.visit($1.f2, __for__, $3);
		while( Normalize.normalizeBoolean( expression.getValue() ) ){
			
			try{
				this.visit($1.f5 , __for__ , $3);
			}
			catch ( ALast last ) {
				if( last.hasLabel() ){
					if(  parent.existsChild( last.getLabelName() )  ){
						if( parent.localExistsChild( last.getLabelName() ) ){
							return null;
						}
						else{
							throw last;
						}
					}
					else{
						//~ throw exception label not founded
						throw new MissingLabelException( last.getLabelName() , last.getPosition() ) ;
					}
				}
				else{
					return null;
				}
			} catch ( ANext next ) {
				if( next.hasLabel() ){
					if(  parent.existsChild( next.getLabelName() )  ){
						if( parent.localExistsChild( next.getLabelName() ) ){
							continue;
						}
						else{
							throw next;
						}
					}
					else{
						//~ throw exception label not founded
						throw new MissingLabelException( next.getLabelName() , next.getPosition() ) ;
					}
				}
				else{
					continue;
				}
			}	
			finally{
				//
				//~ increment / decrement
				//
				if( $1.f4.node != null ){
					NodeSequence ns = (NodeSequence) $1.f4.node;
					this.visit( (Expression) ns.elementAt(1) , __for__, $3);
				}
				else{
					//~ default increment by 1
					AValue value = (AValue) init.getValue();
					Number num	 = (Number) value.getValue();
					
					// test value type
					if( num instanceof Integer ){
						value.setValue( num.intValue() + 1  );
					}
					else if( num instanceof Long ){
						value.setValue( num.longValue() + 1  );
					}
					else if( num instanceof Double ){
						value.setValue( num.doubleValue() + 1  );
					}
				}
				
				//
				//~ test
				//
				expression = (AValue) this.visit($1.f2, __for__, $3);
			}
			
		}
		
		return null;
	}

	/**
	 * f0 -> "my" <SCALAR> [ FullIdentifier() ] [ ScalarAssign() ]
	 *       | ValuesExpression()
	 */
	public Object visit(GoInit $1, Object $2, Object... $3) throws Throwable {
		
		//~ local variable declaration
		if( $1.f0.choice instanceof NodeSequence ){
			NodeSequence ns = (NodeSequence) $1.f0.choice;
			// variable declaration in for loop 
			// my $var int <- 0 , $var < 10 go { ... }
			if( ns.size() == 4 ){

				//~ first check existens variable type
				String	identifier		= null;
				AValue	value			= null;
				
					//# optional identifier
				NodeOptional oIdentifier = (NodeOptional) ns.elementAt(2);
				if( oIdentifier.node != null ){
					AFullIdentifier tmp = (AFullIdentifier) this.visit( (FullIdentifier)oIdentifier.node , $2, $3);
					identifier = Types.getObjectType( tmp.getIdentifier() );
					if( identifier == null ){
						throw new UndefinedTypeException( tmp.getIdentifier() , new long[]{tmp.getLineNumber() , tmp.getColumnNumber()} );
					}
				}
				else{
					identifier = "-I"; // int
				}
				
					//# optional value
				NodeOptional oValue		= (NodeOptional) ns.elementAt(3);
				if( oValue.node != null ){
					value = (AValue) this.visit( (ScalarAssign) oValue.node , $2, $3);
					if( !value.getType().equals( identifier ) ){
						throw new ConversionTypeException( identifier , value.getType() , new long[]{ value.getLineNumber() , value.getColumnNumber() } );
					}
				}
				else{
					value = new AValue(null, "-I", 0);
				}
				
				//~ variable name
				NodeToken varName = (NodeToken) ns.elementAt( 1 );
				
				return new AVariable( varName.tokenImage,identifier, value, null, true );
			}
			// using existans variables
			else if( ns.size() == 3 ){
				
			}
			
		}
		else{
			return this.visit( (ValuesExpression) $1.f0.choice, $2 ,$3);
		}
		
		return null;
	}

	/**
	 * f0 -> "goto"
	 * f1 -> Expression()
	 * f2 -> ":"
	 * f3 -> BlockStatement()
	 */
	public Object visit(GotoStatement $1, Object $2, Object ... $3) throws Throwable {
		AGoto gt = new AGoto();
		gt.setLineNumber( $1.f0.beginLine );
		gt.setColumnNumber( $1.f0.beginColumn );
		gt.parent( (Scope) $2 );
		
		//~ get a goto expression value
		AValue expression = (AValue) this.visit($1.f1, $2, $3);
		
		//~ set goto block expression value and block to be executed
		gt.setTestExpression( expression );
		gt.setExecutionBlock( $1.f3 );
		return gt;
	}

	/**
	 * f0 -> "{"
	 * f1 -> ( <IDENTIFIER> "=>" Expression() ( "," <IDENTIFIER> "=>" Expression() )* )?
	 * f2 -> "}"
	 * @throws Throwable
	 */
	public Object visit(Hash $1, Object $2, Object ... $3) throws Throwable {
		String type  = null;
		// Hash creation
		AHash<Object> hash = new AHash<Object>();
		
		if( $1.f1.node != null ){
			boolean firstElement = true;
			Enumeration e = null ;
			
			do{
				
				NodeSequence ns = firstElement ? (NodeSequence)$1.f1.node : (NodeSequence) e.nextElement();
				AValue value = (AValue) this.visit( (Expression)ns.elementAt( firstElement ? 2 : 3 ) , $2, $3);
				
				// get a value type
				if( type == null ){
					type = value.getType();
				}
				//test type
				else if ( !type.equals(value.getType()) ){
					type =  "java.lang.Object";
				}
				
				//~ get key
				AValue key = null;
				if( firstElement ){
					NodeChoice nc = (NodeChoice) ns.elementAt(0);
					if( nc.choice instanceof NodeToken ){
						key = new AValue( null, Types.getObjectType( nc.choice.toString() ), nc.choice.toString() );
					}
					else{
						key = (AValue) this.visit( (Expression) nc.choice , $2, $3) ;
					}
				}else{
					NodeChoice nc = (NodeChoice) ns.elementAt(1);
					if( nc.choice instanceof NodeToken ){
						key = new AValue( null, Types.getObjectType( nc.choice.toString() ), nc.choice.toString() );
					}
					else{
						key = (AValue) this.visit( (Expression) nc.choice , $2, $3) ;
					}
				}
						
				//~ set value
				hash.val( key.getValue().toString() , value );
				
				//~
				if ( firstElement ){
					NodeListOptional nlo = (NodeListOptional) ns.elementAt(3);
					e = nlo.elements();
					firstElement = false;
				}
				
			}while( !firstElement && e.hasMoreElements()  );
			
		}
		
		type = type != null || (type != null && ( type.equals("java.lang.Object") || type.equals("Array") || type.equals("Hash") ))  ? "Hash" : "Hash:" + type;
		return new AValue(null , type , hash);
	}

	/**
	 * f0 -> "\\"
	 * f1 -> ( <IDENTIFIER> | Expression() )
	 * @throws Throwable 
	 * 
	 */
	public Object visit(HashKey $1, Object $2, Object ... $3) throws Throwable {
		Object key = null;
		
		//~ identifier
		if( $1.f1.choice instanceof NodeToken ){
			key = $1.f1.choice.toString();
		}
		//~ expression
		else{
			key = this.visit( (Expression) $1.f1.choice, $2 ,$3 );
		}
		
		return new AValue(null, Types.getObjectType( key ), key );
	}

	/**
	 * f0 -> Expression()
	 * f1 -> "=>"
	 * f2 -> BlockStatement()
	 * f3 -> [ "," BlockStatement() ]
	 */
	public Object visit(IfStatement $1, Object $2, Object ... $3) throws Throwable {

		AValue value = (AValue) this.visit( $1.f0 , $2 );
		if( Normalize.normalizeBoolean(  value.getValue() ) ){
			return this.visit($1.f2, $2, $3);
		}
		else{
			if( $1.f3.node != null ){
				NodeSequence ns = (NodeSequence) $1.f3.node;
				return this.visit( (BlockStatement)ns.elementAt(1) , $2, $3);
			}
		}
		return null;
	}

	/**
	 * f0 -> ( ScalarValue() [ MemberAccess() ] | ModuleSubroutineCalling() )
	 * f1 -> ( "++" | "--" )
	 */
	@SuppressWarnings("unchecked")
	public Object visit(InDeCrementExpression $1, Object $2, Object ... $3) throws Throwable {
		int increment = 0;
		
		NodeChoice nc = (NodeChoice) $1.f1;
		if( nc.choice.toString().equals("++") ){
			increment++;
		}
		else{
			increment--;
		}
		
		if( $1.f0.choice instanceof NodeSequence){
			Number num = null;
			NodeSequence ns = (NodeSequence) $1.f0.choice;
			NodeOptional no = (NodeOptional) ns.elementAt(1);
			//~ Scalar only
			if( ns.elementAt(0) instanceof ScalarValue && no.node == null ){
				AValue value = (AValue) this.visit( (ScalarValue)ns.elementAt(0)  , $2 , $3);
				// this a new value after increment
				AValue newValue	= new AValue( null , value.getType() , null );
				num = (Number) value.getValue();
				
				// test value type
				if( num instanceof Integer ){
					newValue.setValue( num.intValue() + increment  );
				}
				else if( num instanceof Long ){
					newValue.setValue( num.longValue() + increment  );
				}
				else if( num instanceof Double ){
					newValue.setValue( num.doubleValue() + increment  );
				}
				this.visit( (ScalarValue)ns.elementAt(0)  , $2 , newValue );
				return value;
			}
			// Scalar
			else if( ns.elementAt(0) instanceof ScalarValue && no.node != null ){
				
				//~ Get number value
				AValue value 	= (AValue) this.visit( (ScalarValue)ns.elementAt(0) , $2 ) ;
				Object o = value.getValue();
				//~
				LinkedList members = (LinkedList) this.visit( (MemberAccess)no.node , $2 , this.visit( (ScalarValue) ns.elementAt(0), $2) );
				Iterator<AMemberAccess> i = members.iterator();
				while(i.hasNext()){
					AMemberAccess member = i.next();
					if( member.getType() == AStatementType.Field ){
						if( Reflection.existsField(o, member.getName() ) ){
							o = Reflection.getFieldObject(o, member.getName() );
						}
						else{
							throw new UndefinedFieldException(member.getName() , new long[]{member.getLineNumber(), member.getColumnNumber()} );
						}
					}
					else if ( member.getType() == AStatementType.Subroutine ){
						
						AValue[] args = (AValue[]) this.visit( (Arguments) member.getObject() , $2);
						String[] types = Types.getParametersType( args ) ;
						if( Reflection.existsSubroutine(o, member.getName(), args)){
							o = Reflection.invokeInstanceSubroutine( o , member.getName() , args );
						}
						else{
							throw new UndefinedSubroutineException(member.getName(), o , types , new long[]{member.getLineNumber(), member.getColumnNumber()});
						}
					}
				}
				
				//~ current value
				AValue currentValue = new AValue( null, Types.getObjectType(o) , o);
				AValue newValue = new AValue( null, currentValue.getType() , null );
				num = (Number)o;
				
				//test value type
				if( num instanceof Integer ){
					newValue.setValue( num.intValue() + increment  );
				}
				else if( num instanceof Long ){
					newValue.setValue( num.longValue() + increment  );
				}
				else if( num instanceof Double ){
					newValue.setValue( num.doubleValue() + increment  );
				}
				
				//~ Assign 
				o = new Object() ;
				o =  value.getValue() ;
				members = (LinkedList) this.visit( (MemberAccess)no.node , $2 , this.visit( (ScalarValue) ns.elementAt(0), $2) );
				i = members.iterator();
				while(i.hasNext()){
					AMemberAccess member = i.next();
					
					if( member.getType() == AStatementType.Field ){
						if( i.hasNext() && Reflection.existsField(o, member.getName() ) ){
							o = Reflection.getFieldObject(o, member.getName() );
						}
						else if( !i.hasNext() && Reflection.existsField(o, member.getName() ) ){
							o = Reflection.setFieldObject(o, member.getName() , newValue );
						}
						else{
							throw new UndefinedFieldException(member.getName() , new long[]{member.getLineNumber(), member.getColumnNumber()} );
						}
					}
					else if ( member.getType() == AStatementType.Subroutine ){
						
						AValue[] args = (AValue[]) this.visit( (Arguments) member.getObject() , $2);
						String[] types = Types.getParametersType( args ) ;
						if( Reflection.existsSubroutine(o, member.getName(), args)){
							o = Reflection.invokeInstanceSubroutine( o , member.getName() , args );
						}
						else{
							throw new UndefinedSubroutineException(member.getName(), o , types , new long[]{member.getLineNumber(), member.getColumnNumber()});
						}
					}
				}
				
				
				//~ return ... 
				return currentValue;
				
			}
		}
		
		//~ null
		return null;
	}

	/**
	 * f0 -> "jump"
	 * f1 -> Expression()
	 * f2 -> ( GotoStatement() )+
	 * f3 -> [ MissStatement() ]
	 * @throws Throwable 
	 */
	public Object visit(JumpStatement $1, Object $2, Object ... $3) throws Throwable {
		//~ current scope
		Scope scope = (Scope)$2;
		
		//~ create a jump block
		ABlock jump = new ABlock();
		jump.parent( scope );
		
		//~ get test value
		AValue test = (AValue) this.visit($1.f1, $2, $3);
		
		//~ get all goto blocks
		Vector<AGoto> gotos = new Vector<AGoto>();
		{
			Enumeration e = $1.f2.elements();
			while( e.hasMoreElements() ){
				gotos.add( (AGoto) this.visit( (GotoStatement) e.nextElement(), jump , $3)  );
			}
		}
		
		//~ check founded goto expressions
		{	
			Enumeration<AGoto> e = gotos.elements();
			while( e.hasMoreElements() ){
				AGoto tmp = e.nextElement();
				AValue gotoExpression = (AValue)tmp.getTestExpression();
				if( ! gotoExpression.getValue().equals( test.getValue() ) ){
					gotos.remove( tmp );
					e = gotos.elements();
				}
			}
		}
		
		//~ if goto checked
		if( gotos.size() > 0 ){
			//~ execute founded blocks
			Enumeration<AGoto> e = gotos.elements();
			while( e.hasMoreElements() ){
				AGoto tmp = e.nextElement();
				this.visit( (BlockStatement)tmp.getExecutionBlock() , tmp, $3);
			}
		}
		else{
			//~ if a miss block is founded
			if( $1.f3.node instanceof MissStatement ){
				this.visit( (MissStatement) $1.f3.node , jump , $3);
			}
		}
		
		return null;
	}

	/**
	 * f0 -> <IDENTIFIER>
	 * f1 -> ":"
	 */	
	public Object visit(Label $1, Object $2, Object ... $3) {

		ALabel label = new ALabel( $1.f0.tokenImage );
	
		NodeToken tok = $1.f0;
		label.setLineNumber( tok.beginLine );
		label.setColumnNumber( tok.beginColumn );
	
		Scope the_scope = (Scope)$2;
		the_scope.child( label.getLabelName(), label );
		return null;
	}

	/**
	 * f0 -> "last"
	 * f1 -> [ <IDENTIFIER> ]
	 */
	public Object visit(Last $1, Object $2, Object ... $3) throws ALast {

		if ( $1.f1.node == null ) {
		  throw new ALast( new long[]{ $1.f0.beginLine , $1.f0.beginColumn } );
		} 
		else if( $1.f1.node != null ) {
			NodeToken label = (NodeToken) $1.f1.node;
			throw new ALast( new long[]{ $1.f0.beginLine , $1.f0.beginColumn } , label.tokenImage);
		}
		
		return null;
	}

	/**
	 * f0 -> "main"
	 * f1 -> [ DiesStatement() ]
	 * f2 -> Block()
	 */
	public Object visit(Main $1, Object $2, Object ... $3) throws Throwable {
		ASubroutine main = (ASubroutine)$2;
		main.setLineNumber($1.f0.beginLine);
		main.setColumnNumber($1.f0.beginColumn);
		
		//~ dies statement
		//~
		
		this.visit($1.f2, $2);
		return null;
	}

	/**
	 * f0 -> "->"
	 * f1 -> ( <IDENTIFIER> [ Arguments() ] | Scalar() [ ArrayIndex() | HashKey() ] )
	 * f2 -> ( MemberAccess() )*
	 */
	@SuppressWarnings("unchecked")
	public Object visit(MemberAccess $1, Object $2, Object ... $3) {
		
		AStatement parent 	= 	$2 instanceof AMemberAccess ? ((AMemberAccess)$2).parent() : 
								$2 instanceof ASubroutine 	? (ASubroutine)$2  : (ABlock)$2;
								
		LinkedList<AMemberAccess> members = $3.length == 0 || !($3[0] instanceof LinkedList)? new LinkedList<AMemberAccess>() : (LinkedList<AMemberAccess>)$3[0];
		AMemberAccess member= null;
		
		if( $1.f1.choice instanceof NodeSequence ){
			NodeSequence ns = (NodeSequence)$1.f1.choice;
			if( ns.size() == 2 ){
				NodeToken tok = (NodeToken)ns.elementAt(0);
				NodeOptional no = (NodeOptional)ns.elementAt(1);
				if( no.node == null ){
					
					// Java members
					if( ns.elementAt(0) instanceof NodeToken ){
						member = new AMemberAccess(AStatementType.Field, parent , tok.tokenImage, null) ;
						member.setColumnNumber(tok.beginColumn);
						member.setLineNumber(tok.beginLine);
					}
					
				}
				else{
					
					member =  new AMemberAccess(AStatementType.Subroutine, parent , tok.tokenImage, no.node) ;
					member.setColumnNumber(tok.beginColumn);
					member.setLineNumber(tok.beginLine);
					
				}
			}
		}
		
		members.add(member);
		
		Enumeration e = $1.f2.elements();
		while( e.hasMoreElements() ){
			this.visit((MemberAccess) e.nextElement(), parent , members ) ;
		}
		
		return members;
	}

	/**
	 * f0 -> "miss"
	 * f1 -> ":"
	 * f2 -> BlockStatement()
	 * 
	 * @throws Throwable
	 */
	public Object visit(MissStatement $1, Object $2, Object ... $3) throws Throwable  {
		return this.visit($1.f2, $2, $3);
	}

	public Object visit(Module $1, Object $2, Object ... $3) throws UnsupportedFeatureException {
		// TODO Auto-generated method stub
		throw new UnsupportedFeatureException();
	}

	/**
	 * f0 -> FullIdentifier()
	 * f1 -> ( ( "::" )? Arguments() )?
	 * f2 -> [ MemberAccess() ]
	 * @throws Throwable 
	 */
	@SuppressWarnings("unchecked")
	public Object visit(ModuleSubroutineCalling $1, Object $2, Object ... $3) throws Throwable {
		
		AFullIdentifier fullIdentifier 	= (AFullIdentifier)this.visit($1.f0, $2);
		
		//~ for methods addeds by plugins (Hacking) to Alef++
		//~ this methods is really external from Alef++ core
		//~ has been added to support more features like mutlithreading in Alef++

		if ( Methods.methods.containsKey( fullIdentifier.getIdentifier() ) ){
			//~ get a standare arguments
			NodeSequence ns = (NodeSequence) $1.f1.node;
			Object argsv = ns.elementAt(1);
			AValue[] args = (AValue[]) this.visit( (Arguments) argsv , $2);
			
			return Methods.forge( fullIdentifier.getIdentifier() , this , args );
		}
		
		
		//
		//~ for local subroutines
		//~ subroutine created by using Alef++ code non-Java methods
		//
		if( $2 instanceof AStatement && this.module.existsSubroutine( fullIdentifier.getIdentifier() ) ){
			
			Object child = null ;
			
			//~ get Arrguments
			NodeSequence ns = (NodeSequence)$1.f1.node;
			Object argsv = ns.elementAt(1);
			AValue[] args = (AValue[]) this.visit( (Arguments) argsv , $2);
			
			if( args != null && args.length > 0 ){
				
				ASubroutine sepcialSubroutine = null;
				
				//~ overloading subroutines
				LinkedList subs = new LinkedList();
				Enumeration e = this.module.subroutines( fullIdentifier.getIdentifier() );
				while( e.hasMoreElements() ){
					ASubroutine sub = (ASubroutine) e.nextElement();
					if( sub.getParameters().size() == args.length ){
						subs.add( sub );
					}
					else if( sub.getParameters().size() == 0 && sub.isHasParameters() ){
						sepcialSubroutine = sub ;
					}
				}
				
				//~ Get Subroutine
				Iterator i = subs.iterator();
				String[] types = Types.getParametersType( args ) ;
				
				while( i.hasNext() ){
					boolean check = true ;
					ASubroutine sub = (ASubroutine) i.next();
					ASubroutineParameter[] spar = sub.getParameters().toArray( new ASubroutineParameter[]{} ); 
					
					//~ test parameters type
					for( int index = 0 ; index < types.length ; index++ ){
						
						check = check && spar[ index ].getType().equals( types[ index ] );
						
					}
					
					// founded sub
					if( check ){
						child = sub;
						break;
					}
					
				}
				
				// sepcial subroutine
				// sub SUB{
				// 	...
				// }
				if( child == null && sepcialSubroutine != null ){
					child = sepcialSubroutine ;
				}
				
			}
			else{
				child = this.module.subroutines( fullIdentifier.getIdentifier() ).nextElement();
			}
			
			
			if( child instanceof ASubroutine ){
				
				ASubroutine sub = (ASubroutine) child;
				ABlock block = new ABlock();
				block.parent( sub );
				
				if( args != null && sub.paramertesCount() > 0 ){
					String[] types = Types.getParametersType( args ) ;
					ASubroutineParameter[] subParams = sub.getParameters().toArray(new ASubroutineParameter[]{});
					
					//~
					if( subParams.length == types.length && sub.isHasParameters() ){
						
						for( int index = 0 ; index < types.length ; index++ ){
							// test basic types
							if(  types[ index ].equals( subParams[ index ].getType() ) ){
								AVariable var = new AVariable( "$" + ( index + 1 ) , types[ index ] , new AValue(null, types[ index ] , args[ index ].getValue()  ) , block, true );
								block.child( "$" + ( index + 1 ) , var );
							}
							// test extends types
							else if ( Reflection.instanceOf( subParams[ index ].getType() , args[ index ].getValue()  ) ){
								AVariable var = new AVariable( "$" + ( index + 1 ) , types[ index ] , new AValue(null, types[ index ] , args[ index ].getValue()  ) , block, true );
								block.child( "$" + ( index + 1 ) , var );
							}
							else{
								throw new ConversionTypeException( types[ index ] , subParams[ index ].getType() , new long[]{ args[ index ].getLineNumber() , args[ index ].getColumnNumber() } );
							}
						}
					}
					else if( subParams.length < types.length ){
						throw new OutOfParametersException( fullIdentifier.getIdentifier() , new long[]{ fullIdentifier.getLineNumber() , fullIdentifier.getColumnNumber() } ) ;
					}
				}
				
				/**
				 * test it subroutine has an array parameters like Perl or no one
				 * like void getSomething()
				 * if true create an array parametres called "$0"
				 */
				else if ( sub.isHasParameters() ){
					AArray array = new AArray();
					
					if( args != null ){
						for( AValue value : args ){
							array.push( value );
						}
					}
					
					AVariable var = new AVariable("$0" , "Array" , new AValue(null, "Array", array) , block, true );
					block.child("$0",  var);
				}
				else if( !sub.isHasParameters() && args != null && args.length > 0 ) {
					throw new OutOfParametersException( fullIdentifier.getIdentifier() , new long[]{ fullIdentifier.getLineNumber() , fullIdentifier.getColumnNumber() } ) ;
				}
				
				//~ test returned value
				try{
					
					this.visit( (Block) ((ASubroutine)child).getBlock() , block );
					
				}catch( ARet ret ){
					//retun value like 
					//ret 13;
					AValue retValue = ret.getReturnValue() ; 
					if( retValue.getType().equals( sub.getReturnType() ) ||  Reflection.instanceOf( sub.getReturnType() , retValue.getValue() ) ){
						return retValue;
					}
					//~ if type convertion error
					else if( sub.getReturnType() != null ) {
						throw new ConversionTypeException(retValue.getType() , sub.getReturnType() , new long[]{ sub.getLineNumber() , sub.getColumnNumber() } );
					}
					//~ for void subroutines
					else{
						throw new VoidSubroutinesException( sub.getSubroutineName() , new long[]{ sub.getLineNumber() , sub.getColumnNumber() } );
					}
				}
				
				return null;
			}
		}
		
		//
		//~ for reflection and calling
		//
		fullIdentifier.setIdentifier( Reflection.fullIdentifier( fullIdentifier.getIdentifier() ) );
		
		boolean isStatic		= $1.f1.node == null;
		LinkedList<AMemberAccess> members = null ;
		
		if( $1.f2.node instanceof MemberAccess ){
			members = (LinkedList<AMemberAccess>) this.visit( (MemberAccess)$1.f2.node , $2);
		}
		
		
		// is static member like (System.out)
		if( isStatic && fullIdentifier.getIdentifier() != null ){
			Object o = null;
			Iterator<AMemberAccess> i = members.iterator();
			while(i.hasNext()){
				AMemberAccess ma = i.next();
				//for fields
				if( ( i.hasNext() || $3.length == 0) && o == null &&  ma.getType() == AStatementType.Field ){
					o = Reflection.getFieldObject( fullIdentifier.getIdentifier() , ma.getName());
				}
				else if ( ( i.hasNext() || $3.length == 0) && o != null &&  ma.getType() == AStatementType.Field ){
					o = Reflection.getFieldObject(o, ma.getName());
				}
				else if( !i.hasNext() && $3.length > 0 && o == null &&  ma.getType() == AStatementType.Field ){
					o = Reflection.setFieldObject( fullIdentifier.getIdentifier() , ma.getName() , (AValue)$3[0]);
				}
				else if ( !i.hasNext() && $3.length > 0 && o != null &&  ma.getType() == AStatementType.Field ){
					o = Reflection.setFieldObject(o, ma.getName(), (AValue) $3[0]);
				}
				
				//for subroutines
				else if ( o == null &&  ma.getType() == AStatementType.Subroutine ){
					if( (o = Reflection.getClassObject( fullIdentifier.getIdentifier() )) != null ){
						
						AValue[] args = (AValue[]) this.visit( (Arguments) ma.getObject() , $2);
						String[] types = Types.getParametersType( args ) ;
						
						if( Reflection.existsSubroutine(o, ma.getName(), args)){
							o = Reflection.invokeStaticSubroutine(o, ma.getName(), args);
						}
						else{
							throw new UndefinedSubroutineException(ma.getName(), o , types , new long[]{ma.getLineNumber(), ma.getColumnNumber()});
						}
					}
				}
				else if ( o != null &&  ma.getType() == AStatementType.Subroutine ){
					
					AValue[] args = (AValue[]) this.visit( (Arguments) ma.getObject() , $2);
					String[] types = Types.getParametersType( args ) ;
					
					if( Reflection.existsSubroutine(o, ma.getName(), args)){
						o = Reflection.invokeStaticSubroutine(o, ma.getName(), args);
					}
					else{
						throw new UndefinedSubroutineException(ma.getName(), o , types , new long[]{ma.getLineNumber(), ma.getColumnNumber()});
					}
				}
			}
			
			// return value
			return new AValue(null,Types.getObjectType(o) , o ) ;
		}
		else if ( fullIdentifier.getIdentifier() != null ){
			
			NodeSequence ns = (NodeSequence) $1.f1.node;
			
			// is a module constructor
			NodeOptional no = (NodeOptional) ns.elementAt(0);
			
			// get arguments
			Arguments ma 	= (Arguments)ns.elementAt(1);
			
			if( no.node != null ){ // here module constructor
				AValue[] args = (AValue[]) this.visit( ma , $2);
				String[] types = Types.getParametersType( args ) ;
				Object o = null;
				
				// Create constructor
				if ( fullIdentifier.getIdentifier() != null && Reflection.existsConstructor( fullIdentifier.getIdentifier() , args ) ){
					o =  Reflection.invokeConstructor( fullIdentifier.getIdentifier() , args);
				}
				else{
					throw new UndefinedTypeException( $1.f0.f0.toString() , new long[]{fullIdentifier.getLineNumber() , fullIdentifier.getColumnNumber()} );
				}
				
				//test has members like (StringBuilder::[]->length[])
				if( $1.f2.node != null ){
					// ~
					LinkedList _members = (LinkedList) this.visit( (MemberAccess)$1.f2.node , $2 );
					Iterator<AMemberAccess> _i = _members.iterator();
					while(_i.hasNext()){
						AMemberAccess member = _i.next();
						if( member.getType() == AStatementType.Field ){
							if( Reflection.existsField(o, member.getName() ) ){
								o = Reflection.getFieldObject(o, member.getName() );
							}
							else{
								throw new UndefinedFieldException(member.getName() , new long[]{member.getLineNumber(), member.getColumnNumber()} );
							}
						}
						else if ( member.getType() == AStatementType.Subroutine ){
							
							AValue[] _args = (AValue[]) this.visit( (Arguments) member.getObject() , $2);
							//String[] _types = Types.getParametersType( _args ) ;
							if( Reflection.existsSubroutine(o, member.getName(), _args)){
								o = Reflection.invokeInstanceSubroutine( o , member.getName() , _args );
							}
							else{
								throw new UndefinedSubroutineException(member.getName(), o , types , new long[]{member.getLineNumber(), member.getColumnNumber()});
							}
						}
					}
				}
				
				// return value
				return new AValue( null, Types.getObjectType(o) , o );
			}
		}
		
		// get type for error
		fullIdentifier 	= (AFullIdentifier)this.visit($1.f0, $2);
		throw new UndefinedObjectException( fullIdentifier.getIdentifier() , new long[]{ fullIdentifier.getLineNumber(), fullIdentifier.getColumnNumber() });
	}

	/**
	 * f0 -> ( "*" | "/" | "%" | "//" | "**" )
	 * f1 -> ( Expression() )+
	 */
	@SuppressWarnings("unchecked")
	public Object visit(MultiplicativeExpression $1, Object $2, Object ... $3) throws Throwable {
		
		//	Get values
		LinkedList values = new LinkedList<AValue>();
		Enumeration e = $1.f1.elements();
		while( e.hasMoreElements() ){
			values.add( this.visit( (Expression) e.nextElement() , $2) );
		}
		
		// Multiply operation
		if( $1.f0.choice.toString().equals("*") ){
			return Operations.multiplyOperation( values.toArray( new AValue[]{} ));
		}
		// Divide operation 
		else if( $1.f0.choice.toString().equals("/") ) {
			return Operations.divideOperation(values.toArray( new AValue[]{} ));
		}
		// Modulo operation
		else if( $1.f0.choice.toString().equals("%") ) {
			return Operations.moduloOperation(values.toArray( new AValue[]{} ));
		}
		// Power operation
		else if( $1.f0.choice.toString().equals("**") ) {
			return Operations.powerOperation(values.toArray( new AValue[]{} ));
		}
		// IEEERemainder operation
		else if( $1.f0.choice.toString().equals("//") ) {
			return Operations.IEEERemainderOperation(values.toArray( new AValue[]{} ));
		}
		
		return null;
	}

	/**
	 * f0 -> "next"
	 * f1 -> [ <IDENTIFIER> ]
	 */
	public Object visit(Next $1, Object $2, Object ... $3) throws ANext {

		if ( $1.f1.node == null ) {
		  throw new ANext( new long[]{ $1.f0.beginLine , $1.f0.beginColumn } );
		}
		else if( $1.f1.node != null ){
			NodeToken label = (NodeToken) $1.f1.node;
			throw new ANext( new long[]{ $1.f0.beginLine , $1.f0.beginColumn } , label.tokenImage );
		}

		return null;
	}

	/**
	 * f0 -> FullIdentifier()
	 * f1 -> [ ( "[" )+ | ( "@" | "%" )+ ]
	 */
	public Object visit(ParameterType $1, Object $2, Object ... $3) {
		
		boolean nativeArray = true;
		
		AFullIdentifier identifier  = (AFullIdentifier) this.visit( (FullIdentifier) $1.f0 , $2);
		
		// for arrays
		StringBuilder builder = new StringBuilder();
		if( $1.f1.node != null ){
			
			if( $1.f1.node instanceof NodeChoice ){
				
				NodeChoice nc = (NodeChoice) $1.f1.node;
				if( nc.choice instanceof NodeList ){
					NodeList nl = (NodeList) nc.choice ;
					Enumeration e = nl.elements();
					while( e.hasMoreElements() ){
						Object tmp = e.nextElement();
						nativeArray = nativeArray && tmp instanceof NodeToken ;
						if( nativeArray ){
							builder.append( tmp );
						}
						else{
							NodeChoice nc_type = (NodeChoice) tmp;
							if( nc_type.choice.toString().equals("@") ){
								builder.append("Array:");
							}else{
								builder.append("Hash:");
							}
						}
						
					}
				}
				
			}
			
		}
		
		if( nativeArray ){
			String type = builder.length() > 0 ? Types.getJavaArrayType( identifier.getIdentifier() , builder.toString() ) 
					: identifier.getIdentifier() ;

			return Types.getObjectType( type );
		}
		else{
			return builder.toString() + Types.getObjectType( identifier.getIdentifier() );
		}
		
	}
	
	/**
	 * f0 -> ( "++" | "--" )
	 * f1 -> ( ScalarValue() [ MemberAccess() ] | ModuleSubroutineCalling() )
	 * @throws Throwable 
	 */
	@SuppressWarnings("unchecked")
	public Object visit(PreInDeCrementExpression $1, Object $2, Object ... $3) throws Throwable {
		int increment = 0;
		
		NodeChoice nc = (NodeChoice) $1.f0;
		if( nc.choice.toString().equals("++") ){
			increment++;
		}
		else{
			increment--;
		}
		
		if( $1.f1.choice instanceof NodeSequence){
			Number num = null;
			NodeSequence ns = (NodeSequence) $1.f1.choice;
			NodeOptional no = (NodeOptional) ns.elementAt(1);
			//~ Scalar only
			if( ns.elementAt(0) instanceof ScalarValue && no.node == null ){
				AValue currentValue = (AValue) this.visit( (ScalarValue)ns.elementAt(0)  , $2 , $3);
				// this a new value after increment
				num = (Number) currentValue.getValue();
				
				// test value type
				if( num instanceof Integer ){
					currentValue.setValue( num.intValue() + increment  );
				}
				else if( num instanceof Long ){
					currentValue.setValue( num.longValue() + increment  );
				}
				else if( num instanceof Double ){
					currentValue.setValue( num.doubleValue() + increment  );
				}
				this.visit( (ScalarValue)ns.elementAt(0)  , $2 , currentValue );
				return currentValue;
			}
			// Scalar
			else if( ns.elementAt(0) instanceof ScalarValue && no.node != null ){
				
				//~ Get number value
				AValue value 	= (AValue) this.visit( (ScalarValue)ns.elementAt(0) , $2 ) ;
				Object o = value.getValue();
				//~
				LinkedList members = (LinkedList) this.visit( (MemberAccess)no.node , $2 , this.visit( (ScalarValue) ns.elementAt(0), $2) );
				Iterator<AMemberAccess> i = members.iterator();
				while(i.hasNext()){
					AMemberAccess member = i.next();
					if( member.getType() == AStatementType.Field ){
						if( Reflection.existsField(o, member.getName() ) ){
							o = Reflection.getFieldObject(o, member.getName() );
						}
						else{
							throw new UndefinedFieldException(member.getName() , new long[]{member.getLineNumber(), member.getColumnNumber()} );
						}
					}
					else if ( member.getType() == AStatementType.Subroutine ){
						
						AValue[] args = (AValue[]) this.visit( (Arguments) member.getObject() , $2);
						String[] types = Types.getParametersType( args ) ;
						if( Reflection.existsSubroutine(o, member.getName(), args)){
							o = Reflection.invokeInstanceSubroutine( o , member.getName() , args );
						}
						else{
							throw new UndefinedSubroutineException(member.getName(), o , types , new long[]{member.getLineNumber(), member.getColumnNumber()});
						}
					}
				}
				
				//~ current value
				AValue currentValue = new AValue( null, Types.getObjectType(o) , o);
				num = (Number)o;
				
				//test value type
				if( num instanceof Integer ){
					currentValue.setValue( num.intValue() + increment  );
				}
				else if( num instanceof Long ){
					currentValue.setValue( num.longValue() + increment  );
				}
				else if( num instanceof Double ){
					currentValue.setValue( num.doubleValue() + increment  );
				}
				
				//~ Assign 
				o = new Object() ;
				o =  value.getValue() ;
				members = (LinkedList) this.visit( (MemberAccess)no.node , $2 , this.visit( (ScalarValue) ns.elementAt(0), $2) );
				i = members.iterator();
				while(i.hasNext()){
					AMemberAccess member = i.next();
					
					if( member.getType() == AStatementType.Field ){
						if( i.hasNext() && Reflection.existsField(o, member.getName() ) ){
							o = Reflection.getFieldObject(o, member.getName() );
						}
						else if( !i.hasNext() && Reflection.existsField(o, member.getName() ) ){
							o = Reflection.setFieldObject(o, member.getName() , currentValue );
						}
						else{
							throw new UndefinedFieldException(member.getName() , new long[]{member.getLineNumber(), member.getColumnNumber()} );
						}
					}
					else if ( member.getType() == AStatementType.Subroutine ){
						
						AValue[] args = (AValue[]) this.visit( (Arguments) member.getObject() , $2);
						String[] types = Types.getParametersType( args ) ;
						if( Reflection.existsSubroutine(o, member.getName(), args)){
							o = Reflection.invokeInstanceSubroutine( o , member.getName() , args );
						}
						else{
							throw new UndefinedSubroutineException(member.getName(), o , types , new long[]{member.getLineNumber(), member.getColumnNumber()});
						}
					}
				}
				
				
				//~ return ... 
				return currentValue;
				
			}
		}
		
		//~ null
		return null;
	}

	/**
	 * f0 -> "ref"
	 * f1 -> Expression()
	 * f2 -> FullIdentifier()
	 * @throws Throwable 
	 * 
	 * @throws UnsupportedFeatureException 
	 */
	public Object visit(ReferenceStatement $1, Object $2, Object ... $3) throws Throwable  {
		boolean isReference = false ;
		// get object value
		AValue value = (AValue) this.visit($1.f1, $2, $3);
		// get identifier
		AFullIdentifier indentifer = (AFullIdentifier) this.visit($1.f2, $2 , $3) ;
		// get a full identifier and test existens
		String fullidentifier = Reflection.fullIdentifier( indentifer.getIdentifier() ) ;
		if( fullidentifier != null ){
			isReference = Reflection.instanceOf(fullidentifier, value.getValue());
		}
		else{
			
		}
		return new AValue( null, Types.getObjectType(isReference) , isReference );
	}

	/**
	 * f0 -> ExclusiveOrExpression()
	 * f1 -> ( ( "<" | ">" | "<=" | ">=" ) ExclusiveOrExpression() )*
	 * @throws Throwable 
	 */
	public Object visit(RelationalExpression $1, Object $2, Object ... $3) throws Throwable {
		
		AValue tmp = (AValue) this.visit($1.f0 , $2);
		
		if( $1.f1.size() > 0 ){
			AValue test= null;
			
			AValue tmp2 = null ;
			NodeListOptional nlo = $1.f1;
			Enumeration e = nlo.elements();
			while( e.hasMoreElements() ){
				NodeSequence ns = (NodeSequence)e.nextElement();
				if( ns.elementAt(1) instanceof ExclusiveOrExpression ){
					tmp2 = (AValue) this.visit((ExclusiveOrExpression)ns.elementAt(1), $2);
				}
				
				// test ">" , ">=" , "<=" or "<"
				NodeChoice nc = (NodeChoice)ns.elementAt(0);
				if( test == null ){
					if( nc.choice.toString().equals(">") ){
						test = Conditionals.greater( tmp , tmp2 );
					}
					else if( nc.choice.toString().equals(">=") ){
						test = Conditionals.geaterOrEqual( tmp , tmp2 );
					}
					else if( nc.choice.toString().equals("<=") ){
						test = Conditionals.lessOrEqual( tmp , tmp2 );
					}
					else if( nc.choice.toString().equals("<") ){
						test = Conditionals.less( tmp , tmp2 );
					}
				}
				else{
					//here test last value and current value
					//for "0 < 1 < 2" this is (0 < 1) && (1 < 2)
					//
					if( nc.choice.toString().equals(">") ){
						test = Conditionals.conditionalAnd(test, Conditionals.greater( tmp , tmp2 ) );
					}
					else if( nc.choice.toString().equals(">=") ){
						test = Conditionals.conditionalAnd(test, Conditionals.geaterOrEqual( tmp , tmp2 ) );
					}
					else if( nc.choice.toString().equals("<=") ){
						test = Conditionals.conditionalAnd(test, Conditionals.lessOrEqual( tmp , tmp2 ) );
					}
					else if( nc.choice.toString().equals("<") ){
						test = Conditionals.conditionalAnd(test, Conditionals.less( tmp , tmp2 ) );
					}
				}
				
				tmp = tmp2;
			}
			
			// return the result
			return test;
		}
		 
		return tmp;
	}

	/**
	 * f0 -> "ret"
	 * f1 -> Expression()
	 * f2 -> ";"
	 * @throws Throwable 
	 * @throws UnsupportedFeatureException 
	 */
	public Object visit(ReturnStatement $1, Object $2, Object ... $3) throws Throwable  {
		Object value = this.visit( $1.f1 , $2, $3);
		if( value instanceof AValue ){
			throw new ARet( (AValue)value );
		}
		return new Walo();
	}

	/**
	 * f0 -> <SPCIAL_SCALAR>
	 *       | <SCALAR>
	 *       | "$"
	 *       
	 * @throws ConversionTypeException 
	 * @throws VariableNotFoundException 
	 * @throws FinalLocalVariableException 
	 */
	public Object visit(Scalar $1, Object $2, Object ... $3) throws ConversionTypeException, VariableNotFoundException, FinalLocalVariableException {
		Scope sub = (Scope)$2;
		if( sub.existsChild($1.f0.choice.toString()) && $3.length == 0 ){
			
			AVariable var = (AVariable) sub.child( $1.f0.choice.toString() );
			
			if( var.getValue() instanceof AValue ){
				// Variable position
				NodeToken nt = (NodeToken) $1.f0.choice;
				AValue value = (AValue) var.getValue();
				value.setLineNumber( nt.beginLine );
				value.setColumnNumber( nt.beginColumn );
				
			}
			
			return var.getValue();
			
		}else if( sub.existsChild($1.f0.choice.toString()) && $3.length > 0 ){
			AVariable var	= (AVariable) sub.child( $1.f0.choice.toString() );
			String type		= var.getType();
			AValue local	= (AValue)var.getValue();	// current local variable value
			AValue value	= (AValue) $3[0];
			NodeToken tok	= (NodeToken) $1.f0.choice;
			long[] position	= {tok.beginLine , tok.beginLine};
			//Assign Value
			if( type.equals(value.getType()) || type.equals(Object.class.getName()) && (!var.isFinal() | local.getValue() == null) ){
				var.setValue( value );
				var.setLineNumber(position[0]);
				var.setColumnNumber(position[1]);
				return value;
			}
			else if( var.isFinal() ){
				throw new FinalLocalVariableException(var.getName() , position);
			}
			else {
				throw new ConversionTypeException(value.getType(), type, position);
			}
			
		}
		else if( ! sub.existsChild($1.f0.choice.toString()) ){
			NodeToken tok	= (NodeToken) $1.f0.choice;
			throw new VariableNotFoundException(tok.tokenImage, new long[]{tok.beginLine, tok.beginColumn});
		}
		return null;
	}

	/**
	 * f0 -> "<-"
	 * f1 -> Expression()
	 * @throws Throwable 
	 */
	public Object visit(ScalarAssign $1, Object $2, Object ... $3) throws Throwable {
		return this.visit($1.f1 , $2);
	}

	/**
	 * f0 -> "my"
	 * f1 -> [ "`" ]
	 * f2 -> <SCALAR>
	 * f3 -> [ FullIdentifier() ]
	 * f4 -> ( "@" | "%" )*
	 * f5 -> [ ScalarAssign() ]
	 * @throws Throwable 
	 */
	@SuppressWarnings("unchecked")
	public Object visit(ScalarValue $1, Object $2, Object ... $3) throws Throwable {
		//Scalar value
		if( $1.f1.size() == 0){
			return this.visit($1.f0, $2 , $3);
		}
		else if( $1.f1.size() > 0 ){
		
			//Get Array Value
			AValue value = (AValue) this.visit($1.f0, $2);
			AArray array = null;
			
			// Get Index value
			AValue index = null;
			AValue indexValue = null ;
			
			//test for type indexes
			if( value.getType().contains( ":" ) ){
				int arrayFixSize = value.getType().split(":").length ;
				if( arrayFixSize <= $1.f1.size() ){
					throw new ConversionTypeException("fixed Array[" + ( arrayFixSize - 1 ) + "]" , "Array[" + $1.f1.size() + "]", new long[]{ value.getLineNumber(), value.getColumnNumber() });
				}
			}
			
			Enumeration e = $1.f1.elements();
			while( e.hasMoreElements() ){
				NodeChoice nc = (NodeChoice) e.nextElement();
				if( nc.choice instanceof ArrayIndex ){
					
					index = (AValue)this.visit( (ArrayIndex)nc.choice , $2);
					
					//test is array
					//first array value (dimension) $array#0 or $array#13 or $array#XXXX like $array[X]
					if( indexValue == null && value.getValue() instanceof AArray ){
						array = (AArray) value.getValue();
						indexValue = (AValue) array.index( Normalize.normalizeInteger( index.getValue())  );
						if( indexValue.getValue() instanceof Walo ){
							array.inject(  Normalize.normalizeInteger( index.getValue() ) , indexValue ) ;
							//~ set type for testing
							indexValue.setType(List.forgeType( value.getType() ));
						}
						
					}
					
					//~ create array if not exists
					else if( indexValue != null && indexValue.getValue() instanceof Walo ){
						indexValue.setValue( new AArray());
						array = (AArray)indexValue.getValue();
						AValue tmp = (AValue) array.index( Normalize.normalizeInteger( index.getValue()) );
						if( tmp.getValue() instanceof Walo ){
							indexValue = new AValue( null , List.forgeType( indexValue.getType() ) , List.forgeValue( indexValue.getType() ) );
							array.inject(  Normalize.normalizeInteger( index.getValue() ) , indexValue ) ;
						}
						else{
							indexValue = tmp;
						}
					}
					
					//for multiarrays (dimensions)
					else if( indexValue != null && indexValue.getValue() instanceof AArray ){
						array = (AArray)indexValue.getValue();
						
						AValue tmp = (AValue) array.index( Normalize.normalizeInteger( index.getValue())  );
						if( tmp.getValue() instanceof Walo ){
							indexValue = new AValue( null , List.forgeType( indexValue.getType() ) , List.forgeValue( indexValue.getType() ) );
							array.inject(  Normalize.normalizeInteger( index.getValue() ) , indexValue ) ;
						}
						else{
							indexValue = tmp;
						}
					}
					else{
						//~ creating and injecting a value in array (creation of first element)
						//~ ex : 
						// my $array;
						// $array#0#1#0#1 <- 13; 
						if( value.getType().startsWith("Array") ){
							indexValue = new AValue( null , List.forgeType( value.getType() ) , List.forgeValue( value.getType() ) );
						}
						else{
							throw new ConversionTypeException(value.getType() , "Array", new long[]{ value.getLineNumber(), value.getColumnNumber() });
						}
						
						
						if( value.getValue() instanceof Walo ){
							array = new AArray();
							value.setValue( array );
						}
						else{
							array = (AArray) value.getValue();
						}
						array.inject(  Normalize.normalizeInteger( index.getValue() ) , indexValue ) ;
						
					}
				}
				else if( nc.choice instanceof HashKey ){
					
					//get hash index (KEY)
					index = (AValue) this.visit( (HashKey)nc.choice , $2, $3);
					
					if( indexValue == null ){
						//~ test value is an instance of AHash
						if( value.getValue() instanceof Walo ){
							if( value.getType().startsWith("Hash" )){
								value.setValue( new AHash() );
							}
							else{
								throw new ConversionTypeException(value.getType() , "Hash", new long[]{ value.getLineNumber(), value.getColumnNumber() });
							}
						}
						AHash hash =  (AHash) value.getValue();
						
						// if null or is nothing
						if( hash == null ){
							hash = new AHash<Object>();
							value.setValue(hash);
						}
							
						Object tmp =  hash.val( index.getValue().toString() );
						if( tmp != null ){
							indexValue = new AValue(null, List.forgeType( value.getType() ), ((AValue)tmp).getValue());
						}
						else{
							indexValue = new AValue(null, List.forgeType( value.getType() ), new Walo() );
							hash.val( index.getValue().toString() , indexValue);
						}
					}
					else{
						if( ! (indexValue.getValue() instanceof AHash) ){
							indexValue.setValue( new AHash() );
						}
						AHash hash = (AHash) indexValue.getValue();
						AValue tmp = (AValue) hash.val( index.getValue().toString()  );
						if( tmp != null ){
							indexValue = new AValue(null, List.forgeType( indexValue.getType() ), tmp.getValue());
						}
						else{
							indexValue = new AValue(null, List.forgeType( indexValue.getType() ), new Walo() );
							hash.val( index.getValue().toString() , indexValue);
						}
					}
					
				}
			}
			
			// Assign Array Index / Hash Key
			if( $3.length > 0 ){
				AValue newValue = (AValue)$3[0];
				if(	newValue.getType().equals(indexValue.getType())	|| 
					value.getType().equals("Array")					|| 
					value.getType().equals("Hash")					||
					value.getType().equals("java.lang.Object") 		){
					
					// Change current index value (type & value)
					indexValue.setType( newValue.getType() );
					indexValue.setValue( newValue.getValue() );

				}
				else{
					throw new ConversionTypeException(indexValue.getType() , newValue.getType(), new long[]{ value.getLineNumber(), value.getColumnNumber() });
				}
			}
			// Return Array value
			else{
				return indexValue ;
			}
		}
		
		return null;
	}

	/**
	 * f0 -> ( "<<" | ">>" )
	 * f1 -> ( Expression() )+
	 * @throws Throwable 
	 */
	@SuppressWarnings("unchecked")
	public Object visit(ShiftExpression $1, Object $2, Object ... $3) throws Throwable {
		// Get values
		LinkedList values = new LinkedList();
		Enumeration e = $1.f1.elements();
		while( e.hasMoreElements() ){
			values.add( this.visit( (Expression) e.nextElement() , $2) );
		}
		
		// right shift ">>" operation
		if( $1.f0.choice.toString().equals(">>") ){
			return Operations.rightShiftOperation( values.toArray( new AValue[]{} ) );
		}
		// left shift "<<" operation
		else if( $1.f0.choice.toString().equals("<<") ) {
			return Operations.leftShiftOperation( values.toArray( new AValue[]{} ));
		}
		return null;
	}

	/**
	 * f0 -> DoWhileStatement()
	 *       | WhileStatement()
	 *       | EachStatement()
	 *       | GoStatement()
	 *       | AIfStatement()
	 *       | Label()
	 *       | Last() ";"
	 *       | $1ext() ";"
	 *       | VariableDeclaration() ";"
	 *       | PreInDeCrementExpression() ";"
	 *       | InDeCrementExpression() ";"
	 *       | ScalarValue() [ MemberAccess() ] [ ScalarAssign() ] ";"
	 *       | ModuleSubroutineCalling() [ ScalarAssign() ] ";"
	 *       | DieStatement() ";"
	 *       | JumpStatement() ";"
	 *       | EvalStatement()
	 *       | SynchonizedStatement()
	 * @throws Throwable 
	 */
	@SuppressWarnings("unchecked")
	public Object visit(Statement $1, Object $2, Object ... $3) throws Throwable {

		if( $1.f0.choice instanceof NodeSequence ){
			NodeSequence ns = (NodeSequence)$1.f0.choice;
			
			if( ns.size() == 4){
				NodeOptional nMember = (NodeOptional)ns.elementAt(1);
				NodeOptional nAssign = (NodeOptional)ns.elementAt(2);
				
				// Variable assignment
				if( ns.elementAt(0) instanceof  ScalarValue &&  nMember.node == null && nAssign.node != null ){
					AValue value 	= (AValue) this.visit( (ScalarAssign)nAssign.node , $2);
					return this.visit( (ScalarValue)ns.elementAt(0) , $2, value );
				}
				
				// Variable members (get value)
				else if( ns.elementAt(0) instanceof  ScalarValue &&  nMember.node != null && nAssign.node == null ){
					AValue value 	= (AValue) this.visit( (ScalarValue)ns.elementAt(0) , $2 ) ;
					Object o = value.getValue();
					//~
					LinkedList members = (LinkedList) this.visit( (MemberAccess)nMember.node , $2 , this.visit( (ScalarValue) ns.elementAt(0), $2) );
					Iterator<AMemberAccess> i = members.iterator();
					while(i.hasNext()){
						AMemberAccess member = i.next();
						if( member.getType() == AStatementType.Field ){
							if( Reflection.existsField(o, member.getName() ) ){
								o = Reflection.getFieldObject(o, member.getName() );
							}
							else{
								throw new UndefinedFieldException(member.getName() , new long[]{member.getLineNumber(), member.getColumnNumber()} );
							}
						}
						else if ( member.getType() == AStatementType.Subroutine ){
							
							AValue[] args = (AValue[]) this.visit( (Arguments) member.getObject() , $2);
							String[] types = Types.getParametersType( args ) ;
							if( Reflection.existsSubroutine(o, member.getName(), args)){
								o = Reflection.invokeInstanceSubroutine( o , member.getName() , args );
							}
							else{
								throw new UndefinedSubroutineException(member.getName(), o , types , new long[]{member.getLineNumber(), member.getColumnNumber()});
							}
						}
					}
					
					// return value
					return new AValue( null, Types.getObjectType(o) , o );
				}
				// test assign scalar->member->member-> ... (set value)
				else if( ns.elementAt(0) instanceof  ScalarValue &&  nMember.node != null && nAssign.node != null ){
					//~ Scalar AValue
					AValue value 	= (AValue) this.visit( (ScalarValue)ns.elementAt(0) , $2 ) ;
					Object o = value.getValue();
					
					//~ New Value for Scalar
					AValue tmp = (AValue) this.visit( (ScalarAssign) nAssign.node , $2, $3);
					
					//~
					LinkedList members = (LinkedList) this.visit( (MemberAccess)nMember.node , $2 , this.visit( (ScalarValue) ns.elementAt(0), $2) );
					Iterator<AMemberAccess> i = members.iterator();
					while(i.hasNext()){
						AMemberAccess member = i.next();
						
						if( member.getType() == AStatementType.Field ){
							if( i.hasNext() && Reflection.existsField(o, member.getName() ) ){
								o = Reflection.getFieldObject(o, member.getName() );
							}
							else if( !i.hasNext() && Reflection.existsField(o, member.getName() ) ){
								o = Reflection.setFieldObject(o, member.getName() , tmp );
							}
							else{
								throw new UndefinedFieldException(member.getName() , new long[]{member.getLineNumber(), member.getColumnNumber()} );
							}
						}
						else if ( member.getType() == AStatementType.Subroutine ){
							
							AValue[] args = (AValue[]) this.visit( (Arguments) member.getObject() , $2);
							String[] types = Types.getParametersType( args ) ;
							if( Reflection.existsSubroutine(o, member.getName(), args)){
								o = Reflection.invokeInstanceSubroutine( o , member.getName() , args );
							}
							else{
								throw new UndefinedSubroutineException(member.getName(), o , types , new long[]{member.getLineNumber(), member.getColumnNumber()});
							}
						}
					}
					
					// return value
					return new AValue( null, Types.getObjectType(o) , o );
				}
			}
			//Calling a new module instance or/and static 
			else if( ns.size() == 3 ){
				NodeOptional no = (NodeOptional)ns.elementAt(1);
				if( ns.elementAt(0) instanceof ModuleSubroutineCalling && no.node == null ){
					return this.visit( (ModuleSubroutineCalling)ns.elementAt(0) , $2);
				}else if( ns.elementAt(0) instanceof ModuleSubroutineCalling && no.node != null ){
					Object value  = (AValue) this.visit( (ScalarAssign)no.node  , $2);
					return this.visit( (ModuleSubroutineCalling)ns.elementAt(0) , $2 , value);
				}
			}
			//Variable declaration
			//Last
			//Next
			//
			else if ( ns.size() == 2 ){
				// for variables declarations
				if( ns.elementAt(0) instanceof VariableDeclaration ){
					return this.visit( (VariableDeclaration)ns.elementAt(0) , $2 , $3);
				}
				// loops controls (last)
				else if ( ns.elementAt(0) instanceof Last ) {
					return this.visit( (Last)ns.elementAt(0), $2, $3 );
				}
				// loops controls (next)
				else if ( ns.elementAt(0) instanceof Next ) {
					return this.visit( (Next)ns.elementAt(0), $2 , $3 );
				}
				// die statements or throw exceptions
				else if ( ns.elementAt(0) instanceof DieStatement ) {
					return this.visit( (DieStatement)ns.elementAt(0), $2 ,$3 );
				}
				// Jump statement
				else if ( ns.elementAt(0) instanceof JumpStatement ) {
					return this.visit( (JumpStatement)ns.elementAt(0), $2 , $3 );
				}
				// increment and decrement statement $X (++, --)
				else if (  ns.elementAt(0) instanceof InDeCrementExpression ){
					return this.visit( (InDeCrementExpression) ns.elementAt(0) , $2 , $3);
				}
				// pre-increment and pre-decrement statement (++, --) $X
				else if (  ns.elementAt(0) instanceof PreInDeCrementExpression ){
					return this.visit( (PreInDeCrementExpression) ns.elementAt(0) , $2, $3);
				}
			}
			
		}
		// evaluate block statement
		else if ($1.f0.choice instanceof EvalStatement ){
			return this.visit( (EvalStatement) $1.f0.choice, $2, $3);
		}
		else if ($1.f0.choice instanceof IfStatement ){
			return this.visit( (IfStatement) $1.f0.choice, $2, $3);
		}
		else if( $1.f0.choice instanceof WhileStatement ){
			return this.visit( (WhileStatement) $1.f0.choice, $2, $3);
		}
		else if( $1.f0.choice instanceof DoWhileStatement ){
			return this.visit( (DoWhileStatement) $1.f0.choice, $2, $3);
		}
		else if ( $1.f0.choice instanceof GoStatement ) {
			return this.visit( (GoStatement) $1.f0.choice, $2, $3 );
		}
		else if ( $1.f0.choice instanceof Label ) {
			return this.visit( (Label) $1.f0.choice, $2, $3 );
		}
		// for each statement
		else if( $1.f0.choice instanceof EachStatement ){
			return this.visit( (EachStatement) $1.f0.choice, $2, $3);
		}
		// synchronized block
		else if( $1.f0.choice instanceof SynchonizedStatement ){
			return this.visit( (SynchonizedStatement) $1.f0.choice, $2, $3);
		}
		
		return null;
	}

	/**
	 * f0 -> "sub"
	 * f1 -> <IDENTIFIER>
	 * f2 -> [ SubroutineType() ]
	 * f3 -> ( "(" [ ParameterType() ( "," ParameterType() )* ] ")" )?
	 * f4 -> [ DiesStatement() ]
	 * f5 -> Block()
	 * @throws UnsupportedFeatureException 
	 * @throws DuplicateSubroutineException 
	 */
	@SuppressWarnings("unchecked")
	public Object visit(SubroutineDeclaration $1, Object $2, Object ... $3) throws DuplicateSubroutineException  {
		
		AModule module = (AModule) $2;
		
		ASubroutine sub = new ASubroutine( $1.f1.tokenImage );
		sub.setLineNumber( $1.f0.beginLine );
		sub.setColumnNumber( $1.f0.beginColumn );
		
		//~ return type
		if( $1.f2.node != null ){
			String type =  this.visit( (SubroutineType)$1.f2.node , $2, $3).toString() ;
			
			if( !type.equals( "void" ) ){
				
				sub.setReturnType( type  );
			}
			else{
				sub.setReturnType( null );
			}
		}
		
		//~Typed subroutine paramerts
		if( $1.f3.node != null  ){
			NodeSequence ns_paramerts = (NodeSequence) $1.f3.node;
			NodeOptional no = (NodeOptional) ns_paramerts.elementAt(1);
			if( no.node != null ){
				
				NodeSequence ns_arguments = (NodeSequence) no.node;
				int parameterName = 1;
				
				//~ first parameters
				String type = this.visit( (ParameterType)ns_arguments.elementAt(0) , $2).toString();
				ASubroutineParameter subParam = new ASubroutineParameter( "$" + (parameterName++) , type );
				sub.pushParameter( subParam );
				
				//~ all
				NodeListOptional nlo = (NodeListOptional) ns_arguments.elementAt(1) ;
				Enumeration e = nlo.elements();
				while( e.hasMoreElements() ){
					NodeSequence ns_arguments_paramerets = (NodeSequence) e.nextElement();
					
					type = this.visit( (ParameterType) ns_arguments_paramerets.elementAt(1) , $2).toString();
					subParam = new ASubroutineParameter( "$" + (parameterName++) , type );
					sub.pushParameter( subParam );
				}
			}
			else{
				sub.setHasParameters(false);
			}
		}
		
		
		//~ Testing subroutine existans
		if( !module.existsSubroutine( $1.f1.tokenImage ) ){
			module.subroutine( $1.f1.tokenImage , sub );
		}
		else{
			Enumeration e = module.subroutines( $1.f1.tokenImage );
			while( e.hasMoreElements() ){
				ASubroutine tmp = (ASubroutine)e.nextElement();
				
				//~ 
				// like a classical overloading
				// in Java 
				// void sub(){} & int sub(){} throw exception
				//~
				
				ASubroutineParameter[] subp = sub.getParameters().toArray( new ASubroutineParameter[]{} );
				ASubroutineParameter[] tmpp = tmp.getParameters().toArray( new ASubroutineParameter[]{} );
				boolean check = true ;
				
				if( subp.length == tmpp.length ){
					for( int index = 0 ; index < subp.length ; index++ ){
						check = check && subp[ index ].getType().equals( tmpp[ index ].getType() );
					}
				}
				else{
					check = false;
				}
				
				if( check ){
					LinkedList prms = new LinkedList();
					Iterator i = sub.getParameters().iterator();
					while( i.hasNext() ){
						prms.add(  ((ASubroutineParameter)i.next()).getType()  );
					}
					throw new DuplicateSubroutineException( sub.getSubroutineName() , sub.getReturnType() , prms , new long[]{ sub.getLineNumber() , sub.getColumnNumber() } );
				}
			}
			
			//~
			module.subroutine( $1.f1.tokenImage , sub );
		}
		
		//~ Final set
		sub.setBlock( $1.f5 );
		
		return null;
	}

	/**
	 * f0 -> ":"
	 * f1 -> FullIdentifier()
	 * f2 -> ( ( "[" )+ | ( "@" | "%" )+ )?
	 */
	public Object visit(SubroutineType $1, Object $2, Object ... $3) {
		
		boolean nativeArray = true;
		AFullIdentifier identifier = (AFullIdentifier)this.visit($1.f1 , $2, $3);
		
		//*builder*  variable is a variable for storing an arrays types 
		//(Alef++ arrays/hashs or Native Java arrays) by storing her dimensions,
		StringBuilder builder = new StringBuilder();
		if( $1.f2.node != null ){
			if( $1.f2.node instanceof NodeChoice ){
				NodeChoice nc = (NodeChoice) $1.f2.node;
				NodeList nl  = (NodeList) nc.choice;
				Enumeration e = nl.elements() ;
				while( e.hasMoreElements() ){
					Object tmp = e.nextElement();
					nativeArray = nativeArray && tmp instanceof NodeToken;
					
					if( nativeArray ){
						builder.append( tmp );
					}
					else{
						NodeChoice nc_type = (NodeChoice) tmp;
						if( nc_type.choice.toString().equals("@") ){
							builder.append( "Array:" );
						}else{
							builder.append( "Hash:" );
						}
					}
				}
				
			}
		}
		
		if( nativeArray ){
			String type = builder.length() > 0 ? Types.getJavaArrayType( identifier.getIdentifier() , builder.toString() ) 
					: identifier.getIdentifier() ;

			return Types.getObjectType( type );
		}
		else{
			return  builder.toString() + Types.getObjectType( identifier.getIdentifier() );
		}
		
	}

	/**
	 * f0 -> "synchro"
	 * f1 -> "("
	 * f2 -> Expression()
	 * f3 -> ")"
	 * f4 -> Block()
	 * @throws Throwable 
	 */
	public Object visit(SynchonizedStatement $1, Object $2, Object ... $3) throws Throwable {
		//~ get value 
		AValue value = (AValue) this.visit($1.f2, $2, $3);
		synchronized( value.getValue() ){
			return this.visit($1.f4, $2, $3);
		}
	}

	/**
	 * f0 -> "use"
	 * f1 -> <IDENTIFIER>
	 * f2 -> ( "." ( <IDENTIFIER> | "*" ) )*
	 * f3 -> [ ":" <STRING_LITERAL> ]
	 * f4 -> ";"
	 * 
	 * @throws PackageNotFoundException 
	 */
	public Object visit(UseStatement $1, Object $2, Object ... $3) throws PackageNotFoundException {
		StringBuilder builder = new StringBuilder($1.f1.tokenImage);
		
		Enumeration e = $1.f2.elements();
		while( e.hasMoreElements() ){
			NodeSequence ns = (NodeSequence)e.nextElement();
			builder.append( ns.elementAt(0) );
			builder.append( ((NodeChoice)ns.elementAt(1)).choice );
		}
		
		//load jars
		if( $1.f3.node != null ){
			NodeSequence ns = (NodeSequence)$1.f3.node;
			if ( ! Pkgs.pushPackage(builder.toString(), ns.elementAt(1).toString() ) ){
				throw new PackageNotFoundException(builder.toString() , ns.elementAt(1).toString());
			}
			
		}
		else{
			Pkgs.pushPackage(builder.toString());
		}
		
		return null;
	}

	/**
	 * f0 -> WaloExpression()
	 *       | <STRING_LITERAL>
	 *       | <INTEGER_LITERAL>
	 *       | <FLOATING_POINT_LITERAL>
	 *       | PreInDeCrementExpression()
	 *       | InDeCrementExpression()
	 *       | ScalarValue() [ MemberAccess() ] [ ScalarAssign() ]
	 *       | ModuleSubroutineCalling() [ ScalarAssign() ]
	 *       | "(" Expression() ")"
	 *       | "!" Expression()
	 * @throws Throwable 
	 */
	@SuppressWarnings("unchecked")
	public Object visit(ValuesExpression $1, Object $2, Object ... $3) throws Throwable {
		if( $1.f0.choice instanceof NodeToken ){
			String tmp = $1.f0.choice.toString();
			// String
			if( NativeTypes.isString(tmp) ){
				return new AValue(null,"java.lang.String", Normalize.normalizeString(tmp));
			}
			//int
			else if( NativeTypes.isInt(tmp) ){
				return new AValue(null,"-I",Normalize.normalizeInteger(tmp));
			}
			//long
			else if( NativeTypes.isLong(tmp) ){
				return new AValue(null,"-L",Normalize.normalizeLong(tmp));
			}
			//double
			else if( NativeTypes.isDouble(tmp) ){
				return new AValue(null,"-D",Double.parseDouble(tmp));
			}
			//boolean
			if( NativeTypes.isBoolean(tmp) ){
				return new AValue(null,"-Z",Boolean.parseBoolean(tmp));
			}
		}
		else if( $1.f0.choice instanceof NodeSequence ){
			//"(" Expression ")"
			NodeSequence ns = (NodeSequence)$1.f0.choice;
			if( ns.size() == 4 &&  ns.elementAt(0).toString().equals("(") && ns.elementAt(2).toString().equals(")")  ){
				//~ get member like (String::['text'])->length[]
				NodeOptional nMember = (NodeOptional) ns.elementAt(3);
				
				//~ test if member founded 
				if( nMember.node == null ){
					return this.visit( (Expression) ns.elementAt(1) , $2 );
				}
				else{
					//~ get expression value
					AValue value = (AValue)this.visit( (Expression) ns.elementAt(1) , $2 ) ;
					Object o = value.getValue();
					//~
					LinkedList members = (LinkedList) this.visit( (MemberAccess)nMember.node , $2 , $3 );
					Iterator<AMemberAccess> i = members.iterator();
					while(i.hasNext()){
						AMemberAccess member = i.next();
						if( member.getType() == AStatementType.Field ){
							if( Reflection.existsField(o, member.getName() ) ){
								o = Reflection.getFieldObject(o, member.getName() );
							}
							else{
								throw new UndefinedFieldException(member.getName() , new long[]{member.getLineNumber(), member.getColumnNumber()} );
							}
						}
						else if ( member.getType() == AStatementType.Subroutine ){
							
							AValue[] args = (AValue[]) this.visit( (Arguments) member.getObject() , $2);
							String[] types = Types.getParametersType( args ) ;
							if( Reflection.existsSubroutine(o, member.getName(), args)){
								Object tmp = Reflection.invokeInstanceSubroutine( o , member.getName() , args ); 
								o = tmp != null ? tmp : o ;
							}
							else{
								throw new UndefinedSubroutineException(member.getName(), o , types , new long[]{member.getLineNumber(), member.getColumnNumber()});
							}
						}
					}
					
					// return value
					return new AValue( null, Types.getObjectType(o) , o );
				}
			}
			
			//Scalar value like ($var)
			//Scalar member value like ($var->getSomething[])
			// or Assign them and get values
			else if( ns.size() == 3 && ns.elementAt(0) instanceof ScalarValue){
				NodeOptional nMember = (NodeOptional) ns.elementAt(1);
				NodeOptional nAssign = (NodeOptional) ns.elementAt(2);
				// return current scalar value
				if( nMember.node == null && nAssign.node == null ){
					return this.visit( (ScalarValue) ns.elementAt(0) , $2);
				}
				// assign scalar and return new value
				else if( nMember.node == null && nAssign.node != null ){
					AValue value = (AValue) this.visit( (ScalarAssign)nAssign.node , $2, $3);
					return this.visit( (ScalarValue)ns.elementAt(0) , $2, value);
				}
				// assign scalar member and return new value
				else if( nMember.node != null && nAssign.node == null ){
					AValue value = (AValue)this.visit((ScalarValue)ns.elementAt(0) , $2);
					Object o = value.getValue();
					//~
					LinkedList members = (LinkedList) this.visit( (MemberAccess)nMember.node , $2 , this.visit( (ScalarValue) ns.elementAt(0), $2) );
					Iterator<AMemberAccess> i = members.iterator();
					while(i.hasNext()){
						AMemberAccess member = i.next();
						if( member.getType() == AStatementType.Field ){
							if( Reflection.existsField(o, member.getName() ) ){
								o = Reflection.getFieldObject(o, member.getName() );
							}
							else{
								throw new UndefinedFieldException(member.getName() , new long[]{member.getLineNumber(), member.getColumnNumber()} );
							}
						}
						else if ( member.getType() == AStatementType.Subroutine ){
							
							AValue[] args = (AValue[]) this.visit( (Arguments) member.getObject() , $2);
							String[] types = Types.getParametersType( args ) ;
							if( Reflection.existsSubroutine(o, member.getName(), args)){
								Object tmp = Reflection.invokeInstanceSubroutine( o , member.getName() , args ); 
								o = tmp != null ? tmp : o ;
							}
							else{
								throw new UndefinedSubroutineException(member.getName(), o , types , new long[]{member.getLineNumber(), member.getColumnNumber()});
							}
						}
					}
					
					// return value
					return new AValue( null, Types.getObjectType(o) , o );
				}
				
			}
			//"!" Expression
			else if( ns.size() == 2 && ns.elementAt(0).toString().equals("!") ){
				return Conditionals.conditionalNot( this.visit( (Expression) ns.elementAt(1) , $2 ) );
			}
			// ModuleSubroutineCalling
			else if( ns.size() == 2 && ns.elementAt(0) instanceof ModuleSubroutineCalling ){
				NodeOptional no = (NodeOptional)ns.elementAt(1);
				Object o = this.visit( (ModuleSubroutineCalling)ns.elementAt(0) , $2);
				// return value
				if( no.node == null ){
					return o;
				}
				else{
					
				}
			}
		}
		
		// increment and decrement
		else if( $1.f0.choice instanceof InDeCrementExpression ){
			return this.visit( (InDeCrementExpression) $1.f0.choice, $2);
		}
		// pre-increment and pre-decrement
		else if( $1.f0.choice instanceof PreInDeCrementExpression ){
			return this.visit( (PreInDeCrementExpression) $1.f0.choice, $2);
		}
		// like nothing or null
		else if( $1.f0.choice instanceof WaloExpression ){
			return this.visit( (WaloExpression) $1.f0.choice, $2);
		}
		
		return null;
	}

	/**
	 * f0 -> "my"
	 * f1 -> [ "`" ]
	 * f2 -> <SCALAR>
	 * f3 -> [ FullIdentifier() ]
	 * f4 -> ( "@" | "%" )*
	 * f5 -> [ ScalarAssign() ]
	 * @throws Throwable 
	 */
	public Object visit(VariableDeclaration $1, Object $2, Object ... $3) throws Throwable {
		//Parent
		Scope sub = (Scope)$2;
		
		//Get variale name position
		long[] position = {$1.f2.beginLine, $1.f0.beginColumn - 1};
		
		//Check variable is a final value
		boolean isFinal = $1.f1.node != null;
		
		//Get a variable name
		String name		= $1.f2.tokenImage;
		
		//Get variable type
		String type 	= null;
		if( $1.f3.node != null ){
			AFullIdentifier identifier  = (AFullIdentifier) this.visit( (FullIdentifier) $1.f3.node , $2);
			type = Types.getObjectType( identifier.getIdentifier() );
			type = type == null ? identifier.getIdentifier() : type;
		}else{
			type = "java.lang.Object";
		}
		
		//for typed Arrays and Hashs
		if ( type != null && !type.equals("java.lang.Object" ) ){
			
			StringBuilder list	= new StringBuilder();
			Enumeration e = $1.f4.elements();
			while( e.hasMoreElements() ){
				NodeChoice nc = (NodeChoice)e.nextElement();
				//Array Type
				if( nc.choice.toString().equals("@") ){
					list.append("Array:");
				}
				//Hash Type
				else{
					list.append("Hash:");
				}
			}
			
			type = list.toString() + type;
		}

		
		//test variable existens (for optimization has been moved befor "value")
		if( sub.existsChild(name) ){
			throw new DuplicateLocalVariableException(name , position);
		}
		
		//Get Assign value
		AValue value	= null;
		if( $1.f5.node instanceof ScalarAssign ){
			value = (AValue) this.visit( (ScalarAssign)$1.f5.node , $2);
		}
		else{
			value = new AValue(null, type);
		}

		//Create variable
		//test like my $var int <- 13; or my $var <- 13;
		if( type != null && ( type.equals(value.getType()) || ( type.equals(Object.class.getName()) && !value.getType().contains( ":" ) ) ) ){
			AVariable variable = new AVariable(name,type,value,(AStatement)$2,isFinal);
			variable.setLineNumber(position[0]);
			variable.setColumnNumber(position[1]);
			
			sub.child(variable.getName(), variable);
		}
		//test like my $array <- [];
		else if( type != null && type.equals(Object.class.getName()) && value.getType().contains( ":" ) ){
			//change type from Array:null to Array
			String[] valType = value.getType().split(":");
			value.setType( valType[0] ) ;
			
			AVariable variable = new AVariable(name, valType[0] ,value,(AStatement)$2,isFinal);
			variable.setLineNumber(position[0]);
			variable.setColumnNumber(position[1]);
			
			sub.child(variable.getName(), variable);
		}
		else if( type != null && value.getType().endsWith( ":null" ) ){
			String[] varType = type.split(":");
			String[] valType = value.getType().split(":");
			
			//do chcking types
			if( varType.length == valType.length ){
				for( int index = 0 ; index < varType.length - 1 ; index++ ){
					if( ! varType[ index ].equals( valType[ index ] ) ){
						throw new ConversionTypeException(Arrays.toString(valType) , Arrays.toString(varType) , position);
					}
				}
			}
			else{
				throw new ConversionTypeException( value.getType().replace(":null", ":(Type)") , type , position);
			}
			
			//~ set empty values
			value.setType( type );
			AVariable variable = new AVariable(name,type,value,(AStatement)$2,isFinal);
			variable.setLineNumber(position[0]);
			variable.setColumnNumber(position[1]);
			
			sub.child(variable.getName(), variable);
			
		}
		else {
			throw new ConversionTypeException(value.getType(), type, position);
		}
		
		return null;
	}

	/**
	 * f0 -> "walo"
	 */
	public Object visit(WaloExpression $1, Object $2, Object ... $3) {
		
		return new AValue(null,"walo");
	}

	/**
	 * f0 -> Expression()
	 * f1 -> "loop"
	 * f2 -> BlockStatement()
	 * @throws Throwable 
	 */
	public Object visit(WhileStatement $1, Object $2, Object ... $3) throws Throwable {

		Scope parent = (Scope) $2;

		ABlock __while__ = new ABlock();
		__while__.parent( parent );
		
		AValue value = (AValue) this.visit($1.f0, $2, $3);
		while( Normalize.normalizeBoolean( value.getValue() ) ){
			try {
				this.visit($1.f2, __while__  , $3);			
			} 
			catch ( ALast last ) {

				if( last.hasLabel() ){
					if(  parent.existsChild( last.getLabelName() )  ){
						if( parent.localExistsChild( last.getLabelName() ) ){
							return null;
						}
						else{
							throw last;
						}
					}
					else{
						//~ throw exception label not founded
						throw new MissingLabelException( last.getLabelName() , last.getPosition() ) ;
					}
				}
				else{
					return null;
				}
			} 
			catch ( ANext next ) {
				
				if( next.hasLabel() ){
					if(  parent.existsChild( next.getLabelName() )  ){
						if( parent.localExistsChild( next.getLabelName() ) ){
							continue;
						}
						else{
							throw next;
						}
					}
					else{
						//~ throw exception label not founded
						throw new MissingLabelException( next.getLabelName() , next.getPosition() ) ;
					}
				}
				else{
					continue;
				}
			}	
			finally{
				//Update value
				value = (AValue) this.visit($1.f0, $2, $3);
			}
		}
		return null;
	}

	/**
	 * f0 -> Block()
	 * f1 -> Arguments()
	 * @throws Throwable 
	 * 
	 */
	public Object visit(XiSubroutine $1, Object $2, Object ... $3) throws Throwable  {
		//~ get parameters values
		AValue[] args = (AValue[])this.visit($1.f1, $2 , $3);
		
		//~ create a sepcial variable (array) $0 
		AArray<Object> params = new AArray<Object>();
		for( AValue param : args ){
			params.push( param );
		}
		
		//~ create xi block
		ABlock xi = new ABlock();
		xi.child("$0", new AVariable("$0","Array", new AValue(null, "Array" , params), xi , true ));
		try{
			this.visit( $1.f0 , xi );
		}catch(ARet ret){
			return ret.getReturnValue();
		}
		
		return new AValue(null, "walo");
	}

}
