/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.interpreter.utils;

import alefpp.core.AValue;

/**
 * This class is a class base for all Alef++ conditionals ( and (&&), or (||), ... )
 * and greater , less ... ect
 * @author Adrabi Abderrahim
 * @version 1.3
 */

public class Conditionals {
	
	/**
	 * Alef++ conditional "OR" (||) test
	 * @param $1 (AArgument : list of the value)
	 * @return
	 */
	public static AValue conditionalOr(AValue ... $1){
		Object value = null;
		for( int index = 0 ; index < $1.length - 1 ; index ++ ){
			if( value == null ){
				value = Normalize.normalizeBoolean( $1[ index++ ].getValue() ) || Normalize.normalizeBoolean($1[ index++ ].getValue());
			}
			else{
				value = Normalize.normalizeBoolean( value ) || Normalize.normalizeBoolean($1[ index++ ].getValue());
			}
		}
		return new AValue( null, "-Z", value );
	}
	
	/**
	 * Alef++ conditional "OR" (||) test
	 * @param $1 (Object : list of the value)
	 * @return
	 */
	public static AValue conditionalOr(Object ... $1){
		return conditionalOr( (AValue[])$1 );
	}
	
	/**
	 * Alef++ conditional "AND" (&&) test
	 * @param $1 (AArgument : list of the value)
	 * @return
	 */
	public static AValue conditionalAnd(AValue ... $1){
		Object value = null;
		for( int index = 0 ; index < $1.length - 1 ; index ++ ){
			if( value == null ){
				value = Normalize.normalizeBoolean( $1[ index++ ].getValue() ) && Normalize.normalizeBoolean($1[ index++ ].getValue());
			}
			else{
				value = Normalize.normalizeBoolean( value ) && Normalize.normalizeBoolean($1[ index++ ].getValue());
			}
		}
		return new AValue( null, "-Z", value );
	}
	
	/**
	 * Alef++ conditional "AND" (&&) test
	 * @param $1 (Object : list of the value)
	 * @return
	 */
	public static AValue conditionalAnd(Object ... $1){
		return conditionalAnd( (AValue[])$1 );
	}
	
	/**
	 * Alef++ conditional "XOR" (^) test
	 * @param $1 (AArgument : list of the value)
	 * @return
	 */
	public static AValue conditionalXOr(AValue ... $1){
		Object value = null;
		for( int index = 0 ; index < $1.length - 1 ; index ++ ){
			if( value == null ){
				value = Normalize.normalizeBoolean( $1[ index++ ].getValue() ) ^ Normalize.normalizeBoolean($1[ index++ ].getValue());
			}
			else{
				value = Normalize.normalizeBoolean( value ) ^ Normalize.normalizeBoolean($1[ index++ ].getValue());
			}
		}
		return new AValue( null, "-Z", value );
	}
	
	/**
	 * Alef++ conditional "XOR" (^&) test
	 * @param $1 (Object : list of the value)
	 * @return
	 */
	public static AValue conditionalXOr(Object ... $1){
		return conditionalXOr( (AValue[])$1 );
	}
	
	/**
	 * Alef++ Equality (==) test
	 * @param $1 (AArgument : list of the value)
	 * @return
	 */
	public static AValue equality(AValue ... $1){
		Object value = null;
		for( int index = 0 ; index < $1.length - 1 ; index ++ ){
			if( value == null ){
				value = $1[ index++ ].getValue().equals($1[ index++ ].getValue());
			}
			else{
				value = value.equals( $1[ index++ ].getValue() );
			}
		}
		return new AValue( null, "-Z", value );
	}
	
	/**
	 * Alef++ Equality (==) test
	 * @param $1 (Object : list of the value)
	 * @return
	 */
	public static AValue equality(Object ... $1){
		return equality( (AValue[])$1 );
	}
	
	/**
	 * Alef++ Not Equality (!=) test
	 * @param $1 (AArgument : list of the value)
	 * @return
	 */
	public static AValue notEquality(AValue ... $1){
		Object value = null;
		for( int index = 0 ; index < $1.length - 1 ; index ++ ){
			if( value == null ){
				value = ! $1[ index++ ].getValue().equals($1[ index++ ].getValue());
			}
			else{
				value = ! value.equals( $1[ index++ ].getValue() );
			}
		}
		return new AValue( null, "-Z", value );
	}
	
	/**
	 * Alef++ Equality (!=) test
	 * @param $1 (Object : list of the value)
	 * @return
	 */
	public static AValue notEquality(Object ... $1){
		return notEquality( (AValue[])$1 );
	}
	
	/**
	 * Alef++ greater than (>) test
	 * @param $1 (AArgument : list of the value)
	 * @return
	 */
	public static AValue greater(AValue ... $1){
		Object value = null;
		Object tmp = null;
		for( int index = 0 ; index < $1.length - 1 ; index ++ ){
			if( value == null ){
				value = Normalize.normalizeDouble( $1[ index++ ].getValue() ) > Normalize.normalizeDouble($1[ index++ ].getValue());
			}
			else{
				value = Normalize.normalizeDouble( tmp ) > Normalize.normalizeDouble($1[ index++ ].getValue());
			}
			
			tmp = $1[ index - 1 ];
		}
		return new AValue( null, "-Z", value );
	}
	
	/**
	 * Alef++ greater than (>) test
	 * @param $1 (Object : list of the booleans value)
	 * @return
	 */
	public static AValue greater(Object ... $1){
		return greater( (AValue[])$1 );
	}

	/**
	 * Alef++ less than (<) test
	 * @param $1 (AArgument : list of the value)
	 * @return
	 */
	public static AValue less(AValue ... $1){
		Object value = null;
		Object tmp = null;
		for( int index = 0 ; index < $1.length - 1 ; index ++ ){
			if( value == null ){
				value = Normalize.normalizeDouble( $1[ index++ ].getValue() ) < Normalize.normalizeDouble($1[ index++ ].getValue());
			}
			else{
				value = Normalize.normalizeDouble( tmp ) < Normalize.normalizeDouble($1[ index++ ].getValue());
			}
			tmp = $1[ index - 1 ];
		}
		return new AValue( null, "-Z", value );
	}
	
	/**
	 * Alef++ less than (<) test
	 * @param $1 (Object : list of the booleans value)
	 * @return
	 */
	public static AValue less(Object ... $1){
		return less( (AValue[])$1 );
	}
	
	/**
	 * Alef++ greater than or equal (>=) test
	 * @param $1 (AArgument : list of the value)
	 * @return
	 */
	public static AValue geaterOrEqual(AValue ... $1){
		Object value = null;
		Object tmp = null;
		for( int index = 0 ; index < $1.length - 1 ; index ++ ){
			if( value == null ){
				value = Normalize.normalizeDouble( $1[ index++ ].getValue() ) >= Normalize.normalizeDouble($1[ index++ ].getValue());
			}
			else{
				value = Normalize.normalizeDouble( tmp ) >= Normalize.normalizeDouble($1[ index++ ].getValue());
			}
			tmp = $1[ index - 1 ];
		}
		return new AValue( null, "-Z", value );
	}
	
	/**
	 * Alef++ greater than or equal (>=) test
	 * @param $1 (Object : list of the value)
	 * @return
	 */
	public static AValue geaterOrEqual(Object ... $1){
		return geaterOrEqual( (AValue[])$1 );
	}
	
	/**
	 * Alef++ less than or equal (<=) test
	 * @param $1 (AArgument : list of the value)
	 * @return
	 */
	public static AValue lessOrEqual(AValue ... $1){
		Object value = null;
		Object tmp = null;
		for( int index = 0 ; index < $1.length - 1 ; index ++ ){
			if( value == null ){
				value = Normalize.normalizeDouble( $1[ index++ ].getValue() ) <= Normalize.normalizeDouble($1[ index++ ].getValue());
			}
			else{
				value = Normalize.normalizeDouble( tmp ) <= Normalize.normalizeDouble($1[ index++ ].getValue());
			}
			tmp = $1[ index - 1 ];
		}
		return new AValue( null, "-Z", value );
	}
	
	/**
	 * Alef++ less than (<=) test
	 * @param $1 (Object : list of the value)
	 * @return
	 */
	public static AValue lessOrEqual(Object ... $1){
		return lessOrEqual( (AValue[])$1 );
	}
	
	/**
	 * Alef++ conditional "NOT" (!) test
	 * @param $1 (AArgument : list of the value)
	 * @return
	 */
	public static AValue conditionalNot(AValue  $1){
		Object value = ! Normalize.normalizeBoolean( $1.getValue() );
		return new AValue( null, "-Z", value );
	}
	
	/**
	 * Alef++ conditional "NOT" (!) test
	 * @param $1 (AArgument : list of the value)
	 * @return
	 */
	public static AValue conditionalNot(Object  $1){
		return conditionalNot( (AValue)$1 );
	}
}
