/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.interpreter.utils;

/**
 * This class for data nozmalization to Java primitive types
 *(integers, doubles, ...)
 * @author Adrabi Abderrahim
 * @version 1.5
 */
public class Normalize{
	
	/**
	 * Normalize an booelan data by retrieving boolean value from Alef++
	 * @param $1 (String : Boolean Value)
	 * @return
	 */
	public static boolean normalizeBoolean(String $1){
		if( $1 == null || $1.equals("false") || $1.length() == 0 || $1.equals("0") || $1.equals(""+(char)0) ){
			return false;
		}
		else{
			return true;
		}
		//return Boolean.parseBoolean($1);
	}
	
	/**
	 * Normalize an boolean data by retrieving boolean value from object
	 * @param $1 (String : Boolean Object)
	 * @return
	 */
	public static boolean normalizeBoolean(Object $1){
		return normalizeBoolean($1.toString());
	}
	
	/**
	 * Normalize an byte data by retrieving byte value from Alef++
	 * @param $1 (String : Byte Value)
	 * @return
	 */
	public static byte normalizeByte(String $1){
		if( $1.toLowerCase().startsWith("0x") ){
			$1 =  $1.toLowerCase().replace("0x", "");
			return Byte.parseByte($1,16);
		}
		else if( $1.toLowerCase().startsWith("0o") ){
			$1 =  $1.toLowerCase().replace("0o", "");
			return Byte.parseByte($1,8);
		}
		else if( $1.toLowerCase().startsWith("0b") ){
			$1 =  $1.toLowerCase().replace("0b", "");
			return Byte.parseByte($1,2);
		}
		else if( $1.equals("true") ){
			return 1;
		}
		else if( $1.equals( "false") ){
			return 0;
		}
		else{
			return Byte.parseByte($1);
		}
	}
	
	/**
	 * Normalize an byte data by retrieving byte value from object
	 * @param $1 (String : Byte Object)
	 * @return
	 */
	public static byte normalizeByte(Object $1){
		return normalizeByte($1.toString());
	}
	
	/**
	 * Normalize an integer data by retrieving integer value from Alef++
	 * integer format
	 * @param $1 (String : Integer Value)
	 * @return
	 */
	public static int normalizeInteger(String $1){
		if( $1.toLowerCase().startsWith("0x") ){
			$1 =  $1.toLowerCase().replace("0x", "");
			return Integer.parseInt($1,16);
		}
		else if( $1.toLowerCase().startsWith("0o") ){
			$1 =  $1.toLowerCase().replace("0o", "");
			return Integer.parseInt($1,8);
		}
		else if( $1.toLowerCase().startsWith("0b") ){
			$1 =  $1.toLowerCase().replace("0b", "");
			return Integer.parseInt($1,2);
		}
		else if( $1.equals("true") ){
			return 1;
		}
		else if( $1.equals( "false") ){
			return 0;
		}
		else{
			return Integer.parseInt($1);
		}
	}
	
	/**
	 * Normalize an integer data by retrieving integer value from Object
	 * @param $1 (Object : Integer Object)
	 * @return
	 */
	public static int normalizeInteger(Object $1){
		return normalizeInteger($1.toString()) ;
	}
	
	/**
	 * Normalize an long data by retrieving long value from Alef++
	 * long format
	 * @param $1 (String : Long Value)
	 * @return
	 */
	public static long normalizeLong(String $1){
		
		$1 = $1.toLowerCase().endsWith("l") ? $1.toLowerCase().replace("l", "") : $1;
		
		if( $1.toLowerCase().startsWith("0x") ){
			$1 =  $1.toLowerCase().replace("0x", "");
			return Long.parseLong($1,16);
		}
		else if( $1.toLowerCase().startsWith("0o") ){
			$1 =  $1.toLowerCase().replace("0o", "");
			return Long.parseLong($1,8);
		}
		else if( $1.toLowerCase().startsWith("0b") ){
			$1 =  $1.toLowerCase().replace("0b", "");
			return Long.parseLong($1,2);
		}
		else if( $1.equals("true") ){
			return 1;
		}
		else if( $1.equals( "false") ){
			return 0;
		}
		else{
			return Long.parseLong($1);
		}
	}
	
	/**
	 * Normalize an long data by retrieving long value from Object
	 * @param $1 (Object : Long Object)
	 * @return
	 */
	public static long normalizeLong(Object $1){
		return normalizeLong($1.toString()) ;
	}
	
	/**
	 * Normalize an double data by retrieving double value from Alef++
	 * @param $1 (String : Double Value)
	 * @return
	 */
	public static double normalizeDouble(String $1){
		if( $1.equals("true") ){
			return 1;
		}
		else if( $1 == null || $1.equals( "false") || $1.length() == 0 ){
			return 0;
		}
		return Double.parseDouble($1);
	}
	
	/**
	 * Normalize an double data by retrieving double value from object
	 * @param $1 (String : Double Object)
	 * @return
	 */
	public static double normalizeDouble(Object $1){
		return normalizeDouble($1.toString());
	}
	
	/**
	 * Normalize a String data
	 * @param $1 (String : String Value)
	 * @return
	 */
	public static String normalizeString(String $1){
		$1 = $1.substring(1, $1.length() - 1 );
		$1 = $1.replace("\\n", "\n");
		$1 = $1.replace("\\r", "\r");
		$1 = $1.replace("\\b", "\b");
		$1 = $1.replace("\\f", "\f");
		$1 = $1.replace("\\t", "\t");
		$1 = $1.replace("\\'", "'");
		$1 = $1.replace("\\\\", "\\");
		return $1;
	}
}