/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.interpreter.utils;

import alefpp.core.AValue;

/**
 * This class is a class base for all Alef++ operations (add, minus, ... )
 * and bitwise , shifts ... ect
 * @author Adrabi Abderrahim
 * @version 1.13
 */
public class Operations {

	/**
	 * Alef++ Add "+" operator
	 * @param $1 (AArgument : list of argument)
	 * @return
	 */
	public static AValue addOperation(AValue ... $1 ){
		String type 	= null;
		Object value 	= null;
		
		for( AValue arg : $1 ){
			
			if( arg.getType() == "-Z" && type != "-I" && type != "-L" && type != "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( arg.getType() == "-I" && type != "-L" && type != "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( arg.getType() == "-L" && type != "-D"  && type != "java.lang.String" ){
				type = arg.getType();
			}
			else if( arg.getType() == "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( arg.getType() != "-Z" && arg.getType() != "-I" && arg.getType() != "-L" && arg.getType() != "-D" ){
				type = "java.lang.String";
			}
			
			if( type == "-Z" ){
				if( value == null ){
					value = Normalize.normalizeBoolean(arg.getValue())  ;
				}
				else{
					value = Normalize.normalizeBoolean(value) && Normalize.normalizeBoolean(arg.getValue()) ;
				}
			}
			else if( type == "-I" ){
				if( value == null ){
					value = Normalize.normalizeInteger(arg.getValue());
				}
				else{
					value = Normalize.normalizeInteger(value) + Normalize.normalizeInteger(arg.getValue());
				}
			}
			else if( type == "-L" ){
				if( value == null ){
					value = Normalize.normalizeLong(arg.getValue());
				}
				else{
					value = Normalize.normalizeLong(value) + Normalize.normalizeLong(arg.getValue());
				}
			}
			else if( type == "-D" ){
				if( value == null ){
					value = Normalize.normalizeDouble(arg.getValue());
				}
				else{
					value = Normalize.normalizeDouble(value) + Normalize.normalizeDouble(arg.getValue());
				}
			}
			else{
				if( value == null ){
					value = arg.getValue().toString();
				}
				else{
					value = value.toString() + arg.getValue().toString();
				}
			}
		}
		
		return new AValue(null, type , value);
	}
	
	/**
	 * Alef++ Add "+" operator
	 * @param $1 (Object : list of argument)
	 * @return
	 */
	public static AValue addOperation(Object ... $1 ){
		return addOperation ( (AValue[])$1 );
	}
	
	/**
	 * Alef++ Minus "-" operator
	 * @param $1 (AArgument : list of argument)
	 * @return
	 */
	public static AValue minusOperation(AValue ... $1 ){
		String type 	= null;
		Object value 	= null;
		
		for( AValue arg : $1 ){
			
			if( arg.getType() == "-Z" && type != "-I" && type != "-L" && type != "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( arg.getType() == "-I" && type != "-L" && type != "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( arg.getType() == "-L" && type != "-D"  && type != "java.lang.String" ){
				type = arg.getType();
			}
			else if( arg.getType() == "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( arg.getType() != "-Z" && arg.getType() != "-I" && arg.getType() != "-L" && arg.getType() != "-D" ){
				type = "java.lang.String";
			}
			
			if( type == "-Z" ){
				if( value == null ){
					value = $1.length != 1 && Normalize.normalizeBoolean(arg.getValue())  ;
				}
				else{
					value = Normalize.normalizeBoolean(value) || Normalize.normalizeBoolean(arg.getValue()) ;
				}
			}
			else if( type == "-I" ){
				if( value == null ){
					value = Normalize.normalizeInteger(arg.getValue()) * ($1.length != 1 ? 1 : -1) ;
				}
				else{
					value = Normalize.normalizeInteger(value) - Normalize.normalizeInteger(arg.getValue());
				}
			}
			else if( type == "-L" ){
				if( value == null ){
					value = Normalize.normalizeLong(arg.getValue()) * ($1.length != 1 ? 1 : -1) ;
				}
				else{
					value = Normalize.normalizeLong(value) - Normalize.normalizeLong(arg.getValue());
				}
			}
			else if( type == "-D" ){
				if( value == null ){
					value = Normalize.normalizeDouble(arg.getValue()) * ($1.length != 1 ? 1 : -1) ;
				}
				else{
					value = Normalize.normalizeDouble(value) - Normalize.normalizeDouble(arg.getValue());
				}
			}
			else{
				if( value == null ){
					value = $1.length != 1 ? arg.getValue().toString() : new StringBuilder(arg.getValue().toString()).reverse().toString();
				}
				else{
					value = value.toString().replaceFirst(arg.getValue().toString() , "");
				}
			}
		}
		
		return new AValue(null, type , value);
	}
	
	/**
	 * Alef++ Minus "-" operator
	 * @param $1 (AArgument : list of argument)
	 * @return
	 */
	public static AValue minusOperation(Object ... $1 ){
		return minusOperation( (AValue[])$1 );
	}
	
	/**
	 * Alef++ Multiply "*" operator
	 * @param $1 (AArgument : list of argument)
	 * @return
	 */
	public static AValue multiplyOperation(AValue ... $1 ){
		String type 	= null;
		Object value 	= null;
		
		for( AValue arg : $1 ){
			
			if( arg.getType() == "-Z" && type != "-I" && type != "-L" && type != "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( arg.getType() == "-I" && type != "-L" && type != "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( arg.getType() == "-L" && type != "-D"  && type != "java.lang.String" ){
				type = arg.getType();
			}
			else if( arg.getType() == "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( arg.getType() != "-Z" && arg.getType() != "-I" && arg.getType() != "-L" && arg.getType() != "-D" ){
				type = "java.lang.String";
			}
			
			if( type == "-Z" ){
				if( value == null ){
					value = Normalize.normalizeBoolean(arg.getValue())  ;
				}
				else{
					value = Normalize.normalizeBoolean(value) ^ Normalize.normalizeBoolean(arg.getValue()) ;
				}
			}
			else if( type == "-I" ){
				if( value == null ){
					value = Normalize.normalizeInteger(arg.getValue());
				}
				else{
					value = Normalize.normalizeInteger(value) * Normalize.normalizeInteger(arg.getValue());
				}
			}
			else if( type == "-L" ){
				if( value == null ){
					value = Normalize.normalizeLong(arg.getValue());
				}
				else{
					value = Normalize.normalizeLong(value) * Normalize.normalizeLong(arg.getValue());
				}
			}
			else if( type == "-D" ){
				if( value == null ){
					value = Normalize.normalizeDouble(arg.getValue());
				}
				else{
					value = Normalize.normalizeDouble(value) * Normalize.normalizeDouble(arg.getValue());
				}
			}
			else{
				if( value == null ){
					value = arg.getValue().toString();
				}
				else{
					if( arg.getType() == "-I"  ){
						int x = (int)Math.ceil(Normalize.normalizeInteger(arg.getValue()));
						StringBuilder tmp = new StringBuilder();
						for(int d = 0 ; d < x ; d++ ){
							tmp.append( value.toString() );
						}
						value = tmp.toString();
					}
					else{
						value = value.toString();
					}
				}
			}
		}
		
		return new AValue(null, type , value);
	}
	
	/**
	 * Alef++ Multiply "*" operator
	 * @param $1 (Object : list of argument)
	 * @return
	 */
	public static AValue multiplyOperation(Object ... $1 ){
		return multiplyOperation( (AValue[])$1 );
	}
	
	/**
	 * Alef++ Multiply "*" operator
	 * @param $1 (AArgument : list of argument)
	 * @return
	 */
	public static AValue divideOperation(AValue ... $1 ){
		String type 	= null;
		Object value 	= null;
		
		for( AValue arg : $1 ){
			
			if( arg.getType() == "-Z" && type != "-I" && type != "-L" && type != "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( arg.getType() == "-I" && type != "-L" && type != "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( arg.getType() == "-L" && type != "-D"  && type != "java.lang.String" ){
				type = arg.getType();
			}
			else if( arg.getType() == "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( arg.getType() != "-Z" && arg.getType() != "-I" && arg.getType() != "-L" && arg.getType() != "-D" ){
				type = "java.lang.String";
			}
			
			if( type == "-Z" ){
				if( value == null ){
					value = Normalize.normalizeBoolean(arg.getValue())  ;
				}
				else{
					value = Normalize.normalizeBoolean(value) ^ (!Normalize.normalizeBoolean(arg.getValue())) ;
				}
			}
			else if( type == "-I" ){
				if( value == null ){
					value = $1.length != 1 ? Normalize.normalizeInteger(arg.getValue()) : 1 / Normalize.normalizeInteger(arg.getValue());
				}
				else{
					value = Normalize.normalizeInteger(value) / Normalize.normalizeInteger(arg.getValue());
				}
			}
			else if( type == "-L" ){
				if( value == null ){
					value = $1.length != 1 ? Normalize.normalizeLong(arg.getValue()) : 1 / Normalize.normalizeLong(arg.getValue());
				}
				else{
					value = Normalize.normalizeLong(value) / Normalize.normalizeLong(arg.getValue());
				}
			}
			else if( type == "-D" ){
				if( value == null ){
					value = $1.length != 1 ? Normalize.normalizeDouble(arg.getValue()) : 1 / Normalize.normalizeDouble(arg.getValue());
				}
				else{
					value = Normalize.normalizeDouble(value) / Normalize.normalizeDouble(arg.getValue());
				}
			}
			else{
				if( value == null ){
					value = arg.getValue().toString();
				}
				else{
					value = value.toString().replace( arg.getValue().toString(), "" );
				}
			}
		}
		
		return new AValue(null, type , value);
	}
	
	/**
	 * Alef++ Multiply "*" operator
	 * @param $1 (Object : list of argument)
	 * @return
	 */
	public static AValue divideOperation(Object ... $1 ){
		return divideOperation( (AValue[])  $1 );
	}
	
	/**
	 * Alef++ Modulo "%" operator
	 * @param $1 (AArgument : list of argument)
	 * @return
	 */
	public static AValue moduloOperation(AValue ... $1 ){
		String type 	= null;
		Object value 	= null;
		
		for( AValue arg : $1 ){
			
			if( arg.getType() == "-Z" && type != "-I" && type != "-L" && type != "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( arg.getType() == "-I" && type != "-L" && type != "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( arg.getType() == "-L" && type != "-D"  && type != "java.lang.String" ){
				type = arg.getType();
			}
			else if( arg.getType() == "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( arg.getType() != "-Z" && arg.getType() != "-I" && arg.getType() != "-L" && arg.getType() != "-D" ){
				type = "java.lang.String";
			}
			
			if( type == "-Z" ){
				if( value == null ){
					value = Normalize.normalizeBoolean(arg.getValue())  ;
				}
			}
			else if( type == "-I" ){
				if( value == null ){
					value = Normalize.normalizeInteger(arg.getValue());
				}
				else{
					value = Normalize.normalizeInteger(value) % Normalize.normalizeInteger(arg.getValue());
				}
			}
			else if( type == "-L" ){
				if( value == null ){
					value = Normalize.normalizeLong(arg.getValue()) ;
				}
				else{
					value = Normalize.normalizeLong(value) % Normalize.normalizeLong(arg.getValue());
				}
			}
			else if( type == "-D" ){
				if( value == null ){
					value = Normalize.normalizeDouble(arg.getValue());
				}
				else{
					value = Normalize.normalizeDouble(value) % Normalize.normalizeDouble(arg.getValue());
				}
			}
			else{
				if( value == null ){
					value = arg.getValue().toString();
				}
				else{
					value = "" + (value.toString().length() % arg.getValue().toString().length());
				}
			}
		}
		
		return new AValue(null, type , value);
	}
	
	/**
	 * Alef++ Modulo "%" operator
	 * @param $1 (Object : list of argument)
	 * @return
	 */
	public static AValue moduloOperation(Object ... $1 ){
		return moduloOperation( (AValue[]) $1 );
	}
	
	/**
	 * Alef++ Power "**" operator
	 * @param $1 (AArgument : list of argument)
	 * @return
	 */
	public static AValue powerOperation(AValue ... $1 ){
		String type 	= null;
		Object value 	= null;
		
		for( AValue arg : $1 ){
			
			if( arg.getType() == "-Z" && type != "-I" && type != "-L" && type != "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( ( arg.getType() == "-D" || arg.getType() == "-L" || arg.getType() == "-I" ) && type != "java.lang.String"  ){
				type = "-D";
			}
			else if( arg.getType() != "-Z" && arg.getType() != "-I" && arg.getType() != "-L" && arg.getType() != "-D" ){
				type = "java.lang.String";
			}
			
			if( type == "-Z" ){
				if( value == null ){
					value = Normalize.normalizeBoolean(arg.getValue())  ;
				}
				else{
					value = Normalize.normalizeBoolean(value) && (Normalize.normalizeBoolean(arg.getValue())) ;
				}
			}
			else if( type == "-D" ){
				if( value == null ){
					value = $1.length != 1 ? Normalize.normalizeDouble(arg.getValue()) : Math.pow(Normalize.normalizeDouble(arg.getValue()), 2 );
				}
				else{
					value = Math.pow(Normalize.normalizeDouble(value) , Normalize.normalizeDouble(arg.getValue()));
				}
			}
			else{
				if( value == null ){
					value = arg.getValue().toString();
				}
			}
		}
		
		return new AValue(null, type , value);
	}
	
	/**
	 * Alef++ Power "**" operator
	 * @param $1 (Object : list of argument)
	 * @return
	 */
	public static AValue powerOperation(Object ... $1 ){
		return powerOperation( (AValue[]) $1 );
	}
	
	/**
	 * Alef++ IEEERemainder "//" operator
	 * @param $1 (AArgument : list of argument)
	 * @return
	 */
	public static AValue IEEERemainderOperation(AValue ... $1 ){
		String type 	= null;
		Object value 	= null;
		
		for( AValue arg : $1 ){
			
			if( arg.getType() == "-Z" && type != "-I" && type != "-L" && type != "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( ( arg.getType() == "-D" || arg.getType() == "-L" || arg.getType() == "-I" ) && type != "java.lang.String"  ){
				type = "-D";
			}
			else if( arg.getType() != "-Z" && arg.getType() != "-I" && arg.getType() != "-L" && arg.getType() != "-D" ){
				type = "java.lang.String";
			}
			
			if( type == "-Z" ){
				if( value == null ){
					value = Normalize.normalizeBoolean(arg.getValue())  ;
				}
				else{
					value = Normalize.normalizeBoolean(value) && (Normalize.normalizeBoolean(arg.getValue())) ;
				}
			}
			else if( type == "-D" ){
				if( value == null ){
					value = $1.length != 1 ? Normalize.normalizeDouble(arg.getValue()) : Math.IEEEremainder(Normalize.normalizeDouble(arg.getValue()), 2 );
				}
				else{
					value = Math.IEEEremainder(Normalize.normalizeDouble(value) , Normalize.normalizeDouble(arg.getValue()));
				}
			}
			else{
				if( value == null ){
					value = arg.getValue().toString();
				}
			}
		}
		
		return new AValue(null, type , value);
	}
	
	/**
	 * Alef++ IEEERemainder "//" operator
	 * @param $1 (Object : list of argument)
	 * @return
	 */
	public static AValue IEEERemainderOperation(Object ... $1 ){
		return IEEERemainderOperation( (AValue[])$1 );
	}
	
	/**
	 * Alef++ Bitwise AND "&" operator
	 * @param $1 (AArgument : list of argument)
	 * @return
	 */
	public static AValue bitwiseAndOperation(AValue ... $1 ){
		String type 	= null;
		Object value 	= null;
		
		for( AValue arg : $1 ){
			
			if( arg.getType() == "-Z" && type != "-I" && type != "-L" && type != "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( (  arg.getType() == "-L" || arg.getType() == "-I" ) && type != "java.lang.String"  ){
				type = "-L";
			}
			else if( arg.getType() != "-Z" && arg.getType() != "-I" && arg.getType() != "-L" && arg.getType() != "-D" ){
				type = "java.lang.String";
			}
			
			if( type == "-Z" ){
				if( value == null ){
					value = Normalize.normalizeBoolean(arg.getValue())  ;
				}
				else{
					value = Normalize.normalizeBoolean(value) && (Normalize.normalizeBoolean(arg.getValue())) ;
				}
			}
			else if( type == "-L" ){
				if( value == null ){
					value =  Normalize.normalizeLong(arg.getValue());
				}
				else{
					value = Normalize.normalizeLong(value) & Normalize.normalizeLong(arg.getValue()) ;
				}
			}
			else{
				if( value == null ){
					value = arg.getValue().toString();
				}
			}
		}
		
		return new AValue(null, type , value);
	}
	
	/**
	 * Alef++ Bitwise AND "&" operator
	 * @param $1 (Object : list of argument)
	 * @return
	 */
	public static AValue bitwiseAndOperation(Object ... $1 ){
		return bitwiseAndOperation( (AValue[]) $1 );
	}
	
	/**
	 * Alef++ Bitwise OR "|" operator
	 * @param $1 (AArgument : list of argument)
	 * @return
	 */
	public static AValue bitwiseOrOperation(AValue ... $1 ){
		String type 	= null;
		Object value 	= null;
		
		for( AValue arg : $1 ){
			
			if( arg.getType() == "-Z" && type != "-I" && type != "-L" && type != "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( (  arg.getType() == "-L" || arg.getType() == "-I" ) && type != "java.lang.String"  ){
				type = "-L";
			}
			else if( arg.getType() != "-Z" && arg.getType() != "-I" && arg.getType() != "-L" && arg.getType() != "-D" ){
				type = "java.lang.String";
			}
			
			if( type == "-Z" ){
				if( value == null ){
					value = Normalize.normalizeBoolean(arg.getValue())  ;
				}
				else{
					value = Normalize.normalizeBoolean(value) && (Normalize.normalizeBoolean(arg.getValue())) ;
				}
			}
			else if( type == "-L" ){
				if( value == null ){
					value =  Normalize.normalizeLong(arg.getValue());
				}
				else{
					value = Normalize.normalizeLong(value) | Normalize.normalizeLong(arg.getValue()) ;
				}
			}
			else{
				if( value == null ){
					value = arg.getValue().toString();
				}
			}
		}
		
		return new AValue(null, type , value);
	}
	
	/**
	 * Alef++ Bitwise OR "|" operator
	 * @param $1 (Object : list of argument)
	 * @return
	 */
	public static AValue bitwiseOrOperation(Object ... $1 ){
		return bitwiseOrOperation( (AValue[]) $1 );
	}

	/**
	 * Alef++ Left Shift "<<" operator
	 * @param $1 (AArgument : list of argument)
	 * @return
	 */
	public static AValue leftShiftOperation(AValue ... $1 ){
		String type 	= null;
		Object value 	= null;
		
		for( AValue arg : $1 ){
			
			if( arg.getType() == "-Z" && type != "-I" && type != "-L" && type != "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( (  arg.getType() == "-L" || arg.getType() == "-I" ) && type != "java.lang.String"  ){
				type = "-L";
			}
			else if( arg.getType() != "-Z" && arg.getType() != "-I" && arg.getType() != "-L" && arg.getType() != "-D" ){
				type = "java.lang.String";
			}
			
			if( type == "-Z" ){
				if( value == null ){
					value = Normalize.normalizeBoolean(arg.getValue())  ;
				}
				else{
					value = Normalize.normalizeBoolean(value) && (Normalize.normalizeBoolean(arg.getValue())) ;
				}
			}
			else if( type == "-L" ){
				if( value == null ){
					value =  Normalize.normalizeLong(arg.getValue());
				}
				else{
					value = Normalize.normalizeLong(value) << Normalize.normalizeLong(arg.getValue()) ;
				}
			}
			else{
				if( value == null ){
					value = arg.getValue().toString();
				}
			}
		}
		
		return new AValue(null, type , value);
	}
	
	/**
	 * Alef++ Left Shift "<<" operator
	 * @param $1 (Object : list of argument)
	 * @return
	 */
	public static AValue leftShiftOperation(Object ... $1 ){
		return leftShiftOperation( (AValue[]) $1 );
	}
	
	/**
	 * Alef++ Right Shift ">>" operator
	 * @param $1 (AArgument : list of argument)
	 * @return
	 */
	public static AValue rightShiftOperation(AValue ... $1 ){
		String type 	= null;
		Object value 	= null;
		
		for( AValue arg : $1 ){
			
			if( arg.getType() == "-Z" && type != "-I" && type != "-L" && type != "-D" && type != "java.lang.String"  ){
				type = arg.getType();
			}
			else if( (  arg.getType() == "-L" || arg.getType() == "-I" ) && type != "java.lang.String"  ){
				type = "-L";
			}
			else if( arg.getType() != "-Z" && arg.getType() != "-I" && arg.getType() != "-L" && arg.getType() != "-D" ){
				type = "java.lang.String";
			}
			
			if( type == "-Z" ){
				if( value == null ){
					value = Normalize.normalizeBoolean(arg.getValue())  ;
				}
				else{
					value = Normalize.normalizeBoolean(value) && (Normalize.normalizeBoolean(arg.getValue())) ;
				}
			}
			else if( type == "-L" ){
				if( value == null ){
					value =  Normalize.normalizeLong(arg.getValue());
				}
				else{
					value = Normalize.normalizeLong(value) >> Normalize.normalizeLong(arg.getValue()) ;
				}
			}
			else{
				if( value == null ){
					value = arg.getValue().toString();
				}
			}
		}
		
		return new AValue(null, type , value);
	}
	
	/**
	 * Alef++ Right Shift ">>" operator
	 * @param $1 (Object : list of argument)
	 * @return
	 */
	public static AValue rightShiftOperation(Object ... $1 ){
		return rightShiftOperation( (AValue[]) $1 );
	}
}
