/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.plugins;

import static alefpp.core.Methods.methods;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import alefpp.core.Pkgs;

/**
 * Class for loading all plug-ins methods for XML file
 * 
 * @author Adrabi Abderrahim
 * @version 1.2
 *
 */

public class Loader {
	
	static{
		File plugins = new File("plugins.xml");
		if( plugins.exists() ){
			try {
				try {
					processPlugins( plugins );
				} catch (SAXException e) {
				
				} catch (IOException e) {
					
				}
			} catch (ParserConfigurationException e) {
				
			}
		}
		
	}
	
	/**
	 * parse a plug-ins xml files
	 * &lt;Plug-ins&gt;
	 * 		&lt;Method&gt;
	 * 			&lt;Name&gt; [Method name] &lt;/Name&gt;
	 * 			&lt;Class&gt; [Class FullName] &lt;/Class&gt;
	 * 		&lt;/Method&gt;
	 * &lt;/Plug-ins&gt;
	 * 
	 * @param $1 (File : XML file)
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws IOException
	 */
	private static void processPlugins(File $1) throws ParserConfigurationException, SAXException, IOException{
		DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder domBuilder = domFactory.newDocumentBuilder();
	    Document doc = domBuilder.parse( $1 );
	    Element element = doc.getDocumentElement();
	  
	    NodeList methodsInfo = element.getElementsByTagName("Method");
	    
	    //~ getting methods
	    for( int index = 0 ; index < methodsInfo.getLength() ; index++ ){
	    	Node node = methodsInfo.item( index );
	    	NodeList methodInfo = node.getChildNodes();
	    	
	    	//->[Empty]
	    	//->Element Name
	    	//->[Empty]
	    	//->Element Class
	    	//->[Empty]
	    	//->Element Package Name
	    	//->[Empty]
	    	//->Element Path
	    	//->[Empty]
	    	if( methodInfo.getLength() == 5 ){
	    		Node name = methodInfo.item(1);
	    		Node clazz= methodInfo.item(3);
	    		methods.put( name.getTextContent() , clazz.getTextContent() );
	    	}
	    	else if( methodInfo.getLength() == 9 ){
	    		Node name = methodInfo.item(1);
	    		Node clazz= methodInfo.item(3);
	    		Node pkg  = methodInfo.item(5);
	    		Node path = methodInfo.item(7);
	    		Pkgs.pushPackage( pkg.getTextContent() , path.getTextContent() );
	    		methods.put( name.getTextContent() , clazz.getTextContent() );
	    	}
	    	
	    }
	}
}
