/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.plugins.methods.proxy;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;

import alefpp.core.AArray;
import alefpp.core.ABlock;
import alefpp.core.ASubroutine;
import alefpp.core.AValue;
import alefpp.core.AVariable;
import alefpp.interpreter.Interpret;
import alefpp.parser.syntaxtree.Block;
import alefpp.tools.Types;

/**
 * 
 * @author Adrabi Abderrahim
 * @version 1.1
 */
public class AlefppInvocationHandler implements InvocationHandler{

	private Object   interp  = null;
	private String[] methods = null;
	
	/**
	 * 
	 * @param $1 (Object  : Interpreter)
	 * @param $1 (String[]: Methods)
	 */
	public AlefppInvocationHandler(Object $1 , String ... $2 ){
		this.interp  = $1; 
		this.methods = $2;
	}
	
	public Object getInterpreter(){
		return this.interp;
	}
	
	public String[] getMethods(){
		return this.methods;
	}
	
	public void setMethods(String ... $1){
		this.methods = $1;
	}
	
	public boolean hasMethods(){
		return this.methods != null && this.methods.length > 0;
	}
	
	/**
	 * Search for existens method name to be linked to proxy
	 * @param $1 (String : Method Name)
	 * @return
	 */
	public String search( String $1 ){
		String methodName = null;
		int index = java.util.Arrays.binarySearch( this.methods , $1 ) ;
		if( index < 0 ){
			for( int i = 0 ; i < this.methods.length ; i++ ){
				if( this.methods[ i ].startsWith( $1 + ":" ) ){
					methodName	= this.methods[ i ];
					int colon	= methodName.indexOf(":") + 1;
					methodName	= methodName.substring( colon , methodName.length() );
					break;
				}
			}
		}
		else{
			methodName = this.methods[ index - 1 ];
		}
		return methodName;
	}
	
	/**
	 * 
	 */
	@SuppressWarnings("unchecked")
	public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
		Interpret interp = (Interpret) this.interp;
		
		String methodName = this.search( method.getName() ) ;
		
		if( methodName != null && interp.getGlobalModule().existsSubroutine( methodName ) ){
			ASubroutine sub = (ASubroutine) interp.getGlobalModule().subroutines( methodName ).nextElement();
			sub.parent( interp.getGlobalModule() );
			
			ABlock block = new ABlock();
			block.parent( sub );
			
			AArray array = new AArray();
			
			if( args != null ){
				for( Object value : args ){
					array.push( new AValue(null, Types.getObjectType( value ), value ) );
				}
			}
			
			AVariable var = new AVariable("$0" , "Array" , new AValue(null, "Array", array) , block, true );
			block.child("$0",  var);
			
			Object tmp = interp.visit( (Block) sub.getBlock() , block, null);
			return new AValue( null , Types.getObjectType(tmp) , tmp);
		}
		return null;
	}

}
