/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.reflect;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.LinkedList;

import alefpp.core.AValue;

/**
 * Class base for invoration Java constructors by reflection
 * @author Adrabi Abderrahim
 * @version 1.1
 */

class Initializer {
	
	/**
	 * Check existence of a class constructor with a specified parameters
	 * @param $1 (String : Class Name)
	 * @param $2 (String[] : Constructor Parameters Type)
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public boolean existsConstructor(String $1, AValue ... $2 ){
		//~ Public Constructors
		try {
			if( $2 != null ){
				LinkedList<Class> classes = new LinkedList<Class>();
				for( AValue value : $2 ){
					classes.add( ClassType.forgeClass( value.getType() ) );
				}
				Class clazz = Class.forName($1);
				clazz.getConstructor( classes.toArray(new Class[]{}) );
			}
			else{
				Class clazz = Class.forName($1);
				clazz.getConstructor( new Class[]{} );
			}
			
			return true;
		} catch (ClassNotFoundException e) {
			
		} catch (SecurityException e) {
			
		} catch (NoSuchMethodException e) {
			
		}
		
		//~ Private or Protected Constructors
		try {
			if( $2 != null ){
				LinkedList<Class> classes = new LinkedList<Class>();
				for( AValue value : $2 ){
					classes.add(ClassType.forgeClass( value.getType() ));
				}
				Class clazz = Class.forName($1);
				clazz.getDeclaredConstructor( classes.toArray(new Class[]{}) );
			}
			else{
				Class clazz = Class.forName($1);
				clazz.getDeclaredConstructor( new Class[]{} );
			}
			
			return true;
		} catch (ClassNotFoundException e) {
			
		} catch (SecurityException e) {
			
		} catch (NoSuchMethodException e) {
			
		}
		
		if ($2.length == 0 ) return false;
		
		// Autofind
		try {
			Class clazz = Class.forName($1);
			
			LinkedList<Constructor> ctrs = new LinkedList<Constructor>();
			//~ public constructors
			for( Constructor ctr : clazz.getConstructors() ){
				if(  ctr.getName().equals($1) &&  ctr.getParameterTypes().length == $2.length ){
					ctrs.add(ctr);
				}
			}
			
			//~ private/protected/default constructors
			for( Constructor ctr : clazz.getDeclaredConstructors() ){
				if( ctr.getName().equals($1) && ctr.getParameterTypes().length == $2.length ){
					if( !ctrs.contains(ctr)  )  ctrs.add(ctr);
				}
			}
			
			for( Constructor sub : ctrs ){
				boolean check = true;
				Class[] localParams = sub.getParameterTypes();
				
				for( int index = 0 ; index < localParams.length ; index++ ){
					if( localParams[ index ].isInterface() ){
						check = check && $2[ index ].getValue().getClass().isAssignableFrom( localParams[ index ]  ) ;
					}else{
						check = check && localParams[ index ].isInstance( $2[ index ].getValue()  );
						 //check parent
						 if( !check && localParams[ index ].getSuperclass() != null )
							 check = !check && localParams[ index ].getSuperclass().isInstance( $2[ index ].getValue()  );
					}	 
				}
				
				if(check) return check;
			}
			
		} catch (ClassNotFoundException e) {
			
		}
		
		return false;
	}
	
	/**
	 * Invoke a class constructor with a specified parameters
	 * @param $1 (String : Class Name)
	 * @param $2 (String[] : Constructor Parameters Type)
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public Object invokeConstructor(String $1, AValue ... $2 ){
		//~ Public Invokation
		try {
			if( $2 != null ){
				LinkedList<Class> classes = new LinkedList<Class>();
				for( AValue type : $2 ){
					classes.add(ClassType.forgeClass( type.getType() ));
				}
				Class clazz = Class.forName($1);
				Constructor ctr = clazz.getConstructor( classes.toArray(new Class[]{}) );
				
				LinkedList<Object> values = new LinkedList<Object>();
				for( AValue arg : $2 ){
					values.add(arg.getValue());
				}
				return ctr.newInstance(values.toArray(  ));
			}
			else{
				Class clazz = Class.forName($1);
				Constructor ctr = clazz.getConstructor( new Class[]{} );
				return ctr.newInstance( new Object[]{} );
			}
			
		} catch (ClassNotFoundException e) {
			
		} catch (SecurityException e) {
			
		} catch (NoSuchMethodException e) {
			
		} catch (IllegalArgumentException e) {
		
		} catch (InstantiationException e) {
			
		} catch (IllegalAccessException e) {
			
		} catch (InvocationTargetException e) {
			
		}
		
		//~ Private or Protected Invokation
		try {
			if( $2 != null ){
				LinkedList<Class> classes = new LinkedList<Class>();
				for( AValue type : $2 ){
					classes.add( ClassType.forgeClass( type.getType() ) );
				}
				Class clazz = Class.forName($1);
				Constructor ctr = clazz.getDeclaredConstructor( classes.toArray(new Class[]{}) );
				ctr.setAccessible(true);
				
				LinkedList<Object> values = new LinkedList<Object>();
				for( AValue arg : $2 ){
					values.add(arg.getValue());
				}
				return ctr.newInstance(values.toArray(  ));
			}
			else{
				Class clazz = Class.forName($1);
				Constructor ctr = clazz.getDeclaredConstructor( new Class[]{} );
				ctr.setAccessible( true );
				return ctr.newInstance( new Object[]{} );
			}
			
		} catch (ClassNotFoundException e) {
			
		} catch (SecurityException e) {
			
		} catch (NoSuchMethodException e) {
			
		} catch (IllegalArgumentException e) {
		
		} catch (InstantiationException e) {
			
		} catch (IllegalAccessException e) {
			
		} catch (InvocationTargetException e) {
			
		}
		
		if( $2.length == 0) return null;
		
		// Autofind
		Class clazz;
		try {
			clazz = Class.forName( $1 );
			
			//~ get objects
			LinkedList objects = new LinkedList();
			for( AValue value : $2 ){
				objects.add( value.getValue() );
			}
			
			LinkedList<Constructor> ctrs = new LinkedList<Constructor>();
			//~ public constructors
			for( Constructor ctr : clazz.getConstructors() ){
				if( ctr.getName().equals($1) && ctr.getParameterTypes().length == $2.length ){
					ctrs.add( ctr );
				}
			}
			
			//~ private/protected/default constructors
			for( Constructor ctr : clazz.getDeclaredConstructors() ){
				if( ctr.getName().equals($1) && ctr.getParameterTypes().length == $2.length ){
					if( !ctrs.contains( ctr )  )  ctrs.add( ctr );
				}
			}
			
			//~ exists constructors
			LinkedList<Constructor> foundSubs = new LinkedList<Constructor>();
			for( Constructor ctr : ctrs ){
				boolean check = true;
				Class[] localParams = ctr.getParameterTypes();
				
				for( int index = 0 ; index < localParams.length ; index++ ){
					 check = check && localParams[ index ].isInstance( objects.get( index )  );
					 //check parent
					 if( !check && localParams[ index ].getSuperclass() != null )
						 check = !check && localParams[ index ].getSuperclass().isInstance( $2[ index ].getValue()  );
				}
				
				if(check) foundSubs.add(ctr);
			}
			
			// invoke constructors
			if( foundSubs.size() > 0 ){
			
				return foundSubs.getFirst().newInstance( objects.toArray() );
				
			}
			//else if( foundSubs.size() > 1 ){
				
			//}
		} catch (ClassNotFoundException e) {
			
		} catch (IllegalArgumentException e) {
			
		} catch (InstantiationException e) {
		
		} catch (IllegalAccessException e) {
			
		} catch (InvocationTargetException e) {
			
		}
		
		return null;
	}
}
