/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.reflect;


import alefpp.core.AValue;
import alefpp.core.Pkgs;
import alefpp.core.Walo;

/**
 * This class is base of Alef++ Interpreter for reflection (getting field values,
 * types, invoking Java method, creating a classes instances ...) 
 * 
 * @author Adrabi Abderrahim
 * @version 1.17
 */
public class Reflection {

	/**
	 * Create a Class object from type
	 * @param $1 (String : Type Name)
	 * @return
	 * @throws ClassNotFoundException
	 */
	public static Class classType(String $1) throws ClassNotFoundException{
		if( $1 == null ){
			return Walo.class;
		}
		else if( $1.equals("-I") || $1.equals("int")){
			return int.class ;
		}
		else if( $1.equals("-L") || $1.equals("long") ){
			return long.class ;
		}
		else if( $1.equals("-D") || $1.equals("double") ){
			return double.class ;
		}
		else if( $1.equals("-Z") || $1.equals("boolean")){
			return boolean.class ;
		}
		else if( $1.equals("-W") ){
			return null;
		}else{
			return Class.forName($1);
		}
	}
	
	/**
	 * Get a full class identifier 
	 * @param $1 (String : Class Name)
	 * @return
	 */
	public static String fullIdentifier(String $1){
		// Get a list of the used packages
		for( String pkg : Pkgs.packages() ){
			try {
				
				pkg = pkg.replace("*", "");
				//try to get a full identifier
				//like java.lang.System->out->println[]; and package java.lang.System
				//has imported
				if( $1.startsWith(pkg) ){
					Class.forName($1);
					return $1;
				}
				else{
					
					//like use java.lang.System;
					if( pkg.endsWith($1) ){
						Class.forName( pkg );
						return pkg;
					}
					//like use java.lang.*;
					else{
						Class.forName( pkg + $1);
						return pkg + $1;
					}
				}
				
			} catch (ClassNotFoundException e) {}
		}
		
		// full identifier like java.lang.System->out->println[ ];
		// without imported java.lang.System
		try {
			Class.forName( $1 );
			return  $1;
		} catch (ClassNotFoundException e) {}
		
		return null;
	}

	/**
	 * Get a Class Object
	 * @param $1 (String : Class Name)
	 * @return
	 */
	public static Class getClassObject(String $1){
		try{
			return Class.forName($1);
		}catch(ClassNotFoundException cnfe){
			
		}
		return null;
	}
	
	/**
	 * Check is object instance of spcified class
	 * @param $1 (String : Class Type)
	 * @param $2 (Object : Object tested)
	 * @return
	 */
	public static boolean instanceOf(String $1, Object $2){
		
		try {
			Class clazz = classType($1);
			return clazz.isInstance( $2 );
		} catch (ClassNotFoundException e) {
		
		}
		
		return false;
	}
	
	/**
	 * Check existence of the field 
	 * @param $1 (Object : Class Object)
	 * @param $2 (String : Filed Name)
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static boolean existsField(Object $1, String $2){
		return new Scalar().existsField($1, $2);
	}
	
	/**
	 * Return a field object from spicified class name for statics
	 * @param $1 (String : Class Name)
	 * @param $2 (String : Field Name)
	 * @return
	 */
	public static Object getFieldObject(String $1 , String $2){
		return new Scalar().getFieldObject($1, $2);
	}
	
	/**
	 * Return a field object from spicified class name for statics
	 * @param $1 (Object : Class Object)
	 * @param $2 (String : Field Name)
	 * @return
	 */
	public static Object getFieldObject(Object $1 , String $2){
		return new Scalar().getFieldObject($1, $2);
	}
	
	/**
	 * Set a field object from spicified class name for statics
	 * @param $1 (String : Class Name)
	 * @param $2 (String : Field Name)
	 * @return
	 */
	public static Object setFieldObject(String $1 , String $2, AValue $3){
		return new Scalar().setFieldObject($1, $2, $3);
	}
	
	/**
	 * Set a field object from spicified class name for statics
	 * @param $1 (Object : Class Object)
	 * @param $2 (String : Field Name)
	 * @return
	 */
	public static Object setFieldObject(Object $1 , String $2, AValue $3){
		return new Scalar().setFieldObject($1, $2, $3);
	}

	/**
	 * Check existence of a class constructor with a specified parameters
	 * @param $1 (String : Class Name)
	 * @param $2 (String[] : Constructor Parameters Type)
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static boolean existsConstructor(String $1, AValue ... $2 ){
		return new Initializer().existsConstructor($1, $2);
	}
	
	/**
	 * Invoke a class constructor with a specified parameters
	 * @param $1 (String : Class Name)
	 * @param $2 (String[] : Constructor Parameters Type)
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static Object invokeConstructor(String $1, AValue ... $2 ){
		return new Initializer().invokeConstructor($1, $2);
	}
	
	/**
	 * Check exists of the subroutine with a specified parameters
	 * @param $1 (String : Class Name)
	 * @param $2 (String : Subroutine Name)
	 * @param $3 (String[] : Types)
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static boolean existsSubroutine(String $1, String $2, AValue ... $3){
		return new Subroutine().existsSubroutine($1, $2, $3);
	}
	
	/**
	 * Check existence of the subroutine with a specified parameters
	 * @param $1 (Object : Class Object)
	 * @param $2 (String : Subroutine Name)
	 * @param $3 (String[] : Types)
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static boolean existsSubroutine(Object $1, String $2, AValue ... $3){
		return new Subroutine().existsSubroutine($1, $2, $3);
	}
	
	/**
	 * For invoking a static subroutines and return value if is not void
	 * @param $1 (String : Class Name)
	 * @param $2 (String : Method Name)
	 * @param $3 (AArgument[] : Method Arguments) 
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static Object invokeStaticSubroutine(String $1, String $2, AValue ... $3){
		return new Subroutine().invokeStaticSubroutine($1, $2, $3);
	}
	
	/**
	 * For invoking a static subroutines and return value if is not void
	 * @param $1 (Object : Class Object)
	 * @param $2 (String : Method Name)
	 * @param $3 (AArgument[] : Method Arguments) 
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static Object invokeStaticSubroutine(Object $1, String $2, AValue ... $3){
		return new Subroutine().invokeStaticSubroutine($1, $2, $3);
	}

	
	/**
	 * For invoking an instance subroutines and return value if is not void
	 * @param $1 (Object : Class Object)
	 * @param $2 (String : Method Name)
	 * @param $3 (AArgument[] : Method Arguments) 
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static Object invokeInstanceSubroutine(Object $1, String $2, AValue ... $3){
		return new Subroutine().invokeInstanceSubroutine($1, $2, $3);
	}
	
}
