/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.reflect;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.LinkedList;

import alefpp.core.AValue;

/**
 * Class for Invoking a Java subroutines by reflection
 * @author Adrabi Abderrahim
 * @version 1.1
 */

public class Subroutine {
	
	/**
	 * Check exists of the subroutine with a specified parameters
	 * @param $1 (String : Class Name)
	 * @param $2 (String : Subroutine Name)
	 * @param $3 (String[] : Types)
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public boolean existsSubroutine(String $1, String $2, AValue ... $3){
		//~ Public Subroutine
		try{
			LinkedList<Class> classes = new LinkedList<Class>();
			for( AValue value : $3 ){
				classes.add( ClassType.forgeClass( value.getType() ) );
			}
			Class clazz = ClassType.forgeClass( $1 );
			clazz.getMethod($2, classes.toArray(new Class[]{}) );
			return true;
		}catch(ClassNotFoundException cnfe){
			
		}catch(NoSuchMethodException nsfe){
			
		}
		
		//~ Private or Protected Subroutine
		try{
			LinkedList<Class> classes = new LinkedList<Class>();
			for( AValue value : $3 ){
				classes.add( ClassType.forgeClass( value.getType() ) );
			}
			Class clazz = ClassType.forgeClass($1);
			clazz.getDeclaredMethod( $2, (Class[])classes.toArray() );
			return true;
		}catch(ClassNotFoundException cnfe){
			
		}catch(NoSuchMethodException nsfe){
			
		}
		
		if ($3.length == 0 ) return false;
		
		// Autofind
		try {
			Class clazz = Class.forName($1);
			
			LinkedList<Method> subs = new LinkedList<Method>();
			//~ public subs
			for( Method sub : clazz.getMethods() ){
				if(  sub.getName().equals($2) &&  sub.getParameterTypes().length == $3.length ){
					subs.add(sub);
				}
			}
			
			//~ private/protected/default subs
			for( Method sub : clazz.getDeclaredMethods() ){
				if( sub.getName().equals($2) && sub.getParameterTypes().length == $3.length ){
					if( !subs.contains(sub)  )  subs.add(sub);
				}
			}
			
			for( Method sub : subs ){
				boolean check = true;
				Class[] localParams = sub.getParameterTypes();
				
				for( int index = 0 ; index < localParams.length ; index++ ){
					 check = check && localParams[ index ].isInstance( $3[ index ].getValue() );
					 //check parent
					 if( !check && localParams[ index ].getSuperclass() != null )
						 check = !check && localParams[ index ].getSuperclass().isInstance( $3[ index ].getValue() );
					 if( !check )
						 check = !check && Cast.tryCast(localParams[ index ] , $3[ index ].getValue()) != null ;
				}
				
				if(check) return check;
			}
			
		} catch (ClassNotFoundException e) {
			
		}
		
		return false;
	}
	
	/**
	 * Check existence of the subroutine with a specified parameters
	 * @param $1 (Object : Class Object)
	 * @param $2 (String : Subroutine Name)
	 * @param $3 (String[] : Types)
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public boolean existsSubroutine(Object $1, String $2, AValue ... $3){
		//~ Public Subroutine
		try{
			Class clazz = $1 instanceof Class ? (Class)$1 : $1.getClass();
			
			// Subtoutine with parameters "sub[param_1,params_2,...]"
			if( $3 != null ){
				LinkedList<Class> classes = new LinkedList<Class>();
				for( AValue value : $3 ){
					classes.add( ClassType.forgeClass( value.getType() ) );
				}
				clazz.getMethod($2, classes.toArray(new Class[]{}));
			}
			// Subroutine without parameters "sub[]"
			else{
				clazz.getMethod($2, new Class[]{} );
			}
			
			return true;
		}catch(ClassNotFoundException cnfe){
			
		}catch(NoSuchMethodException nsfe){
			
		}
		
		//~ Private or Protected Subroutine
		try{
			Class clazz = $1 instanceof Class ? (Class)$1 : $1.getClass();
			
			// Subtoutine with parameters "sub[param_1,params_2,...]"
			if( $3 != null ){
				LinkedList<Class> classes = new LinkedList<Class>();
				for( AValue value : $3 ){
					classes.add( ClassType.forgeClass( value.getType() ) );
				}
				clazz.getDeclaredMethod($2, classes.toArray(new Class[]{}));
			}
			// Subroutine without parameters "sub[]"
			else{
				clazz.getDeclaredMethod($2, new Class[]{} );
			}
			
			return true;
		}catch(ClassNotFoundException cnfe){
			
		}catch(NoSuchMethodException nsfe){
			
		}
		
		if ($3.length == 0 ) return false;
		
		// Autofind
		Class clazz = $1 instanceof Class ? (Class)$1 : $1.getClass();
		
		LinkedList<Method> subs = new LinkedList<Method>();
		//~ public subs
		for( Method sub : clazz.getMethods() ){
			if(  sub.getName().equals($2) &&  sub.getParameterTypes().length == $3.length ){
				subs.add(sub);
			}
		}
		
		//~ private/protected/default subs
		for( Method sub : clazz.getDeclaredMethods() ){
			if( sub.getName().equals($2) && sub.getParameterTypes().length == $3.length ){
				if( !subs.contains(sub)  )  subs.add(sub);
			}
		}
		
		for( Method sub : subs ){
			boolean check = true;
			Class[] localParams = sub.getParameterTypes();
			
			for( int index = 0 ; index < localParams.length ; index++ ){
				 check = check && localParams[ index ].isInstance( $3[ index ].getValue()  );
				 //check parent
				 if( !check && localParams[ index ].getSuperclass() != null )
					 check = !check && localParams[ index ].getSuperclass().isInstance( $3[ index ].getValue()  );
				 if( !check )
					 check = !check && Cast.tryCast(localParams[ index ] , $3[ index ].getValue()) != null ;
			}
			
			if(check) return check;
		}
		
		return false;
	}

	/**
	 * For invoking a static subroutines and return value if is not void
	 * @param $1 (String : Class Name)
	 * @param $2 (String : Method Name)
	 * @param $3 (AArgument[] : Method Arguments) 
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public Object invokeStaticSubroutine(String $1, String $2, AValue ... $3){
		//~ Public Invoking
		try{
			//~ Get Class
			Class clazz = ClassType.forgeClass($1);
			//~ Get Static Method
			LinkedList<Class> params = new LinkedList<Class>();
			for( AValue arg : $3 ){
				params.add( ClassType.forgeClass(arg.getType()) );
			}
			Method method = clazz.getMethod($2, params.toArray(new Class[]{}));
			
			//~ Invoke Static Method
			LinkedList<Object> values = new LinkedList<Object>();
			for( AValue arg : $3 ){
				values.add(arg.getValue());
			}
			return method.invoke(clazz, values.toArray(new Object[]{}));
		}catch( ClassNotFoundException cnfe){
			
		} catch (SecurityException se) {
			
		} catch (NoSuchMethodException nsme) {
			
		} catch (IllegalArgumentException iae) {
			
		} catch (IllegalAccessException iae) {
			
		} catch (InvocationTargetException ate) {
			
		}
		
		//~ Private or Protected Invoking
		try{
			//~ Get Class
			Class clazz = ClassType.forgeClass($1);
			//~ Get Static Method
			LinkedList<Class> params = new LinkedList<Class>();
			for( AValue arg : $3 ){
				params.add( ClassType.forgeClass(arg.getType()) );
			}
			Method method = clazz.getDeclaredMethod($2, params.toArray(new Class[]{}));
			method.setAccessible(true);
			
			//~ Invoke Static Method
			LinkedList<Object> values = new LinkedList<Object>();
			for( AValue arg : $3 ){
				values.add(arg.getValue());
			}
			return method.invoke(clazz, values.toArray(new Object[]{}));
		}catch( ClassNotFoundException cnfe){
			
		} catch (SecurityException se) {
			
		} catch (NoSuchMethodException nsme) {
			
		} catch (IllegalArgumentException iae) {
			
		} catch (IllegalAccessException iae) {
			
		} catch (InvocationTargetException ate) {
			
		}
		
		if( $3.length == 0 ) return null;
		
		// Autofind
		try {
			Class clazz = Class.forName($1);
			
			//~ get objects
			LinkedList objects = new LinkedList();
			for( AValue value : $3 ){
				objects.add( value.getValue() );
			}
			
			LinkedList<Method> subs = new LinkedList<Method>();
			//~ public subs
			for( Method sub : clazz.getMethods() ){
				if(  sub.getName().equals($2) &&  sub.getParameterTypes().length == $3.length ){
					subs.add(sub);
				}
			}
			
			//~ private/protected/default subs
			for( Method sub : clazz.getDeclaredMethods() ){
				if( sub.getName().equals($2) && sub.getParameterTypes().length == $3.length ){
					if( !subs.contains(sub)  )  subs.add(sub);
				}
			}
			
			//~ exists subroutines
			LinkedList<Method> foundSubs = new LinkedList<Method>();
			for( Method sub : subs ){
				boolean check = true;
				Class[] localParams = sub.getParameterTypes();
				
				for( int index = 0 ; index < localParams.length ; index++ ){
					 check = check && localParams[ index ].isInstance( objects.get( index )  );
					 //check parent
					 if( !check && localParams[ index ].getSuperclass() != null )
						 check = !check && localParams[ index ].getSuperclass().isInstance( $3[ index ].getValue()  );
					 if( !check )
						 check = !check && Cast.tryCast(localParams[ index ] , $3[ index ].getValue()) != null ;
				}
				
				if(check) foundSubs.add(sub);
			}
			
			// invoke subroutines
			//if( foundSubs.size() == 1 ){
				try {
					Method sub = foundSubs.getFirst();
					if( !sub.isAccessible() ){
						sub.setAccessible(true);
					}
					return sub.invoke( $1 , objects.toArray() );
				} catch (IllegalArgumentException e) {
					
				} catch (IllegalAccessException e) {
					
				} catch (InvocationTargetException e) {
					
				}
			//}
			//else if( foundSubs.size() > 1 ){
				
			//}
		} catch (ClassNotFoundException e1) {
			
		}
		
			
		return null;
	}
	
	/**
	 * For invoking a static subroutines and return value if is not void
	 * @param $1 (Object : Class Object)
	 * @param $2 (String : Method Name)
	 * @param $3 (AArgument[] : Method Arguments) 
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public Object invokeStaticSubroutine(Object $1, String $2, AValue ... $3){
		//~ Public Invokation
		try{
			Class clazz = $1 instanceof Class ? (Class)$1 : $1.getClass();
			
			// Invoke Subroutine with parameters
			if( $3 != null ){
				//~ Get Static Method
				LinkedList<Class> params = new LinkedList<Class>();
				for( AValue arg : $3 ){
					params.add( ClassType.forgeClass( arg.getType() ) );
				}
				
				Method method = clazz.getMethod($2, params.toArray(new Class[]{}));
				
				//~ Invoke Static Method
				LinkedList<Object> values = new LinkedList<Object>();
				for( AValue arg : $3 ){
					values.add(arg.getValue());
				}
				
				return  method.invoke($1, values.toArray(new Object[]{}));
			}
			// Invoke Subroutine without parameters
			else{
				Method method = clazz.getMethod($2, new Class[]{});
				return  method.invoke($1, new Object[]{});
			}
			
			
		}catch( ClassNotFoundException cnfe){
			
		} catch (SecurityException se) {
			
		} catch (NoSuchMethodException nsme) {
			
		} catch (IllegalArgumentException iae) {
			
		} catch (IllegalAccessException iae) {
			
		} catch (InvocationTargetException ate) {
			
		}
		
		//~ Private or Protected Invokation
		try{
			Class clazz = $1 instanceof Class ? (Class)$1 : $1.getClass();
			
			// Invoke Subroutine with parameters
			if( $3 != null ){
				//~ Get Static Method
				LinkedList<Class> params = new LinkedList<Class>();
				for( AValue arg : $3 ){
					params.add( ClassType.forgeClass( arg.getType() ) );
				}
				
				Method method = clazz.getDeclaredMethod($2, params.toArray(new Class[]{}));
				method.setAccessible(true);
				//~ Invoke Static Method
				LinkedList<Object> values = new LinkedList<Object>();
				for( AValue arg : $3 ){
					values.add(arg.getValue());
				}
				
				return  method.invoke($1, values.toArray(new Object[]{}));
			}
			// Invoke Subroutine without parameters
			else{
				Method method = clazz.getDeclaredMethod($2, new Class[]{});
				method.setAccessible(true);
				return  method.invoke($1, new Object[]{});
			}
			
			
		}catch( ClassNotFoundException cnfe){
			
		} catch (SecurityException se) {
			
		} catch (NoSuchMethodException nsme) {
			
		} catch (IllegalArgumentException iae) {
			
		} catch (IllegalAccessException iae) {
			
		} catch (InvocationTargetException ate) {
			
		}
		
		// Autofind
		Class clazz = $1 instanceof Class ? (Class)$1 : $1.getClass();
		
		//~ get objects
		LinkedList objects = new LinkedList();
		for( AValue value : $3 ){
			objects.add( value.getValue() );
		}
		
		LinkedList<Method> subs = new LinkedList<Method>();
		//~ public subs
		for( Method sub : clazz.getMethods() ){
			if(  sub.getName().equals($2) &&  sub.getParameterTypes().length == $3.length ){
				subs.add(sub);
			}
		}
		
		//~ private/protected/default subs
		for( Method sub : clazz.getDeclaredMethods() ){
			if( sub.getName().equals($2) && sub.getParameterTypes().length == $3.length ){
				if( !subs.contains(sub)  )  subs.add(sub);
			}
		}
		
		//~ exists subroutines
		LinkedList<Method> foundSubs = new LinkedList<Method>();
		for( Method sub : subs ){
			boolean check = true;
			Class[] localParams = sub.getParameterTypes();
			
			for( int index = 0 ; index < localParams.length ; index++ ){
				 check = check && localParams[ index ].isInstance( objects.get( index ) );
				 //check parent
				 if( !check && localParams[ index ].getSuperclass() != null )
					 check = !check && localParams[ index ].getSuperclass().isInstance( $3[ index ].getValue()  );
				 if( !check )
					 check = !check && Cast.tryCast(localParams[ index ] , $3[ index ].getValue()) != null ;
			}
			
			if(check) foundSubs.add(sub);
		}
		
		// invoke subroutines
		//if( foundSubs.size() == 1 ){
			try {
				Method sub = foundSubs.getFirst();
				if( !sub.isAccessible() ){
					sub.setAccessible(true);
				}
				return sub.invoke( $1 , objects.toArray() );
			} catch (IllegalArgumentException e) {
			
			} catch (IllegalAccessException e) {
				
			} catch (InvocationTargetException e) {
			
			}
		//}
		//else if( foundSubs.size() > 1 ){
			
		//}
		
		return null;
	}
	
	/**
	 * For invoking an instance subroutines and return value if is not void
	 * @param $1 (Object : Class Object)
	 * @param $2 (String : Method Name)
	 * @param $3 (AArgument[] : Method Arguments) 
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public Object invokeInstanceSubroutine(Object $1, String $2, AValue ... $3){
		//~ Public Invokation
		try{
			Class clazz = $1.getClass();
			
			// Invoke Subroutine with parameters
			if( $3 != null ){
				//~ Get Method
				LinkedList<Class> params = new LinkedList<Class>();
				for( AValue arg : $3 ){
					params.add( ClassType.forgeClass( arg.getType() ) );
				}
				
				Method method = clazz.getMethod($2, params.toArray(new Class[]{}));
				
				//~ Invoke Static Method
				LinkedList<Object> values = new LinkedList<Object>();
				for( AValue arg : $3 ){
					values.add(arg.getValue());
				}
				
				return  method.invoke($1, values.toArray());
			}
			// Invoke Subroutine without parameters
			else{
				Method method = clazz.getMethod($2, new Class[]{});
				return  method.invoke($1, new Object[]{});
			}
			
			
		}catch( ClassNotFoundException cnfe){
			
		} catch (SecurityException se) {
			
		} catch (NoSuchMethodException nsme) {
			
		} catch (IllegalArgumentException iae) {
			
		} catch (IllegalAccessException iae) {
			
		} catch (InvocationTargetException ate) {
			
		}
		
		//~ Private or Protected invokation
		try{
			Class clazz = $1.getClass();
			
			// Invoke Subroutine with parameters
			if( $3 != null ){
				//~ Get Method
				LinkedList<Class> params = new LinkedList<Class>();
				for( AValue arg : $3 ){
					params.add( ClassType.forgeClass( arg.getType() ) );
				}
				
				Method method = clazz.getDeclaredMethod($2, params.toArray(new Class[]{}));
				
				//~ Invoke Static Method
				LinkedList<Object> values = new LinkedList<Object>();
				for( AValue arg : $3 ){
					values.add(arg.getValue());
				}
				method.setAccessible(true);
				
				return  method.invoke($1, values.toArray());
			}
			// Invoke Subroutine without parameters
			else{
				Method method = clazz.getDeclaredMethod($2, new Class[]{});
				method.setAccessible(true);
				return  method.invoke($1, new Object[]{});
			}
			
			
		}catch( ClassNotFoundException cnfe){
			
		} catch (SecurityException se) {
			
		} catch (NoSuchMethodException nsme) {
			
		} catch (IllegalArgumentException iae) {
			
		} catch (IllegalAccessException iae) {
			
		} catch (InvocationTargetException ate) {
			
		}
		
		
		if ($3.length == 0 ) return false;
		
		// Autofind
		Class clazz = $1 instanceof Class ? (Class)$1 : $1.getClass();
		
		//~ get objects
		LinkedList objects = new LinkedList();
		for( AValue value : $3 ){
			objects.add( value.getValue() );
		}
		
		LinkedList<Method> subs = new LinkedList<Method>();
		//~ public subs
		for( Method sub : clazz.getMethods() ){
			if(  sub.getName().equals($2) &&  sub.getParameterTypes().length == $3.length ){
				subs.add(sub);
			}
		}
		
		//~ private/protected/default subs
		for( Method sub : clazz.getDeclaredMethods() ){
			if( sub.getName().equals($2) && sub.getParameterTypes().length == $3.length ){
				if( !subs.contains(sub)  )  subs.add(sub);
			}
		}
		
		//~ exists subroutines
		LinkedList<Method> foundSubs = new LinkedList<Method>();
		for( Method sub : subs ){
			boolean check = true;
			Class[] localParams = sub.getParameterTypes();
			
			for( int index = 0 ; index < localParams.length ; index++ ){
				 check = check && localParams[ index ].isInstance( objects.get( index )  );
				 //check parent
				 if( !check && localParams[ index ].getSuperclass() != null )
					 check = !check && localParams[ index ].getSuperclass().isInstance( $3[ index ].getValue()  );
			}
			
			if(check) foundSubs.add(sub);
		}
		
		// invoke subroutines
		//if( foundSubs.size() == 1 ){
			try {
				Method sub = foundSubs.getFirst();
				if( !sub.isAccessible() ){
					sub.setAccessible(true);
				}
				return sub.invoke( $1 , objects.toArray() );
			} catch (IllegalArgumentException e) {
				
			} catch (IllegalAccessException e) {
				
			} catch (InvocationTargetException e) {
				
			}
		//}
		//else if( foundSubs.size() > 1 ){
			
		//}
		
		return null;
	}
	
}
