/**
 *     _    _       __             
 * 	  / \  | | ___ / _|  _     _   
 *	 / _ \ | |/ _ \ |_ _| |_ _| |_ 
 *	/ ___ \| |  __/  _|_   _|_   _|
 * /_/   \_\_|\___|_|   |_|   |_|  
 *
 * Author	:: Adrabi Abderrahim
 * Mail		:: Adrabi[at]Gmail[dot]Com
 * License	:: files Artistic.htm, GPL.htm
 */

package alefpp.tools;

import java.util.LinkedList;

import alefpp.core.AValue;
import alefpp.reflect.Reflection;


/**
 * For getting a types from arguments or objects
 * @author Adrabi Abderrahim
 * @version 1.8
 */

public class Types{
	
	/**
	 * Get a Object type and nomalize this in Alef++ types
	 * @param o (Object : Object Value)
	 * @return
	 */
	public static String getObjectType (Object $1){
		
		if( $1 instanceof Boolean ){
			return "-Z";
		}
		else if( $1 instanceof Integer ){
			return "-I";
		}
		else if( $1 instanceof Long ){
			return "-L";
		}
		else if( $1 instanceof Double ){
			return "-D";
		}
		else if ( $1 != null ){
			return $1.getClass().getName();
		}
		
		return null;
	}
	
	/**
	 * Get a Object type and nomalize this in Alef++ types
	 * @param o (String : String Value)
	 * @return
	 */
	public static String getObjectType (String $1){
		
		if( $1.equals("boolean") ){
			return "-Z";
		}
		else if( $1.equals("int") ){
			return "-I";
		}
		else if( $1.equals("long") ){
			return "-L";
		}
		else if( $1.equals("double") ){
			return "-D";
		}
		else if( $1.equals( "void" ) ){
			return $1;
		}
		else {
			return Reflection.fullIdentifier($1);
		}
	}
	
	/**
	 * List parameters types
	 * @param $1 (Object : Arguments list)
	 * @return
	 */
	public static String[] getParametersType(Object $1){
		if ( $1 instanceof AValue[]){
			return aArgument( (AValue[])$1 );
		}
		return null;
	}
	
	private static String[] aArgument(AValue[] $1){
		LinkedList<String> params = new LinkedList<String>();
		for( AValue arg : $1 ){
			params.add( arg.getType() );
		}
		return params != null ? params.toArray(new String[]{}) : null;
	}
	
	/**
	 * 
	 * @param $1 (String : Type)
	 * @param $2 (String : Array)
	 * @return
	 */
	public static String getJavaArrayType( String $1 , String $2 ){
		$1 = Types.getObjectType($1) ;
		if( $1.startsWith("-") ){
			$1 = $1.replace("-", "");
		}
		else{
			$1 = "L" + $1 + ";" ;
		}
		return ( $2 + $1  );
	}

}
